# -- coding: UTF-8 --
import sys
import unittest
sys.path.append('..\\')
import vowei_hook_common

class TestClass(unittest.TestCase):
    def test_post_comment(self):
        vw = vowei_hook_common.vowei_hook_base()

        comments = []
        comments.append({
            'PostId': 7,
            'PostType': 'Ticket',
            'Text': u'关闭任务#7',
            'User': 'yimin_shi@hotmail.com'
        })
        comments.append({
            'PostId': 8,
            'PostType': 'Ticket',
            'Text': u'关闭任务#8',
            'User': 'yimin_shi@hotmail.com'
        })
        
        vw.post_comments(comments, 'localhost', '/vowei/Comment/PostComments');
        
    def test_no_ids(self):
        vw = vowei_hook_common.vowei_hook_base()
        expect = []
        actual = vw.find_work_item_ids('fix 123')
        compare_sets(expect, actual)
            
    def test_find_unicode_ids(self):
        vw = vowei_hook_common.vowei_hook_base()
        expect = [12, 34, 56]
        actual = vw.find_work_item_ids(u'完成工作任务#12和bug #34还有风险#56')
        compare_sets(expect, actual)

    def test_find_single_id(self):
        vw = vowei_hook_common.vowei_hook_base()
        expect = [123]
        actual = vw.find_work_item_ids('fix #123')
        compare_sets(expect, actual)

    def test_find_multiple_ids(self):
        vw = vowei_hook_common.vowei_hook_base()
        expect = [123, 456]
        actual = vw.find_work_item_ids('fix #123, #456')
        compare_sets(expect, actual)

        actual = vw.find_work_item_ids('fix #123 #456')
        compare_sets(expect, actual)

    def test_find_single_guid_id(self):
        vw = vowei_hook_common.vowei_hook_base()
        expect = ['D09E3925-0981-4F88-AF3D-7A4BC668263C']
        actual = vw.find_work_item_ids('fix #D09E3925-0981-4F88-AF3D-7A4BC668263C')
        compare_sets(expect, actual)
        
        expect = ['d09e3925-0981-4f88-af3d-7a4bc668263c']
        actual = vw.find_work_item_ids('fix #d09e3925-0981-4f88-af3d-7a4bc668263c')
        compare_sets(expect, actual)

    def test_find_multiple_guid_ids(self):
        vw = vowei_hook_common.vowei_hook_base()
        expect = ['D09E3925-0981-4F88-AF3D-7A4BC668263C', 'AB810B41-2A46-4F94-B2E8-D36FBAF9FC8F']
        actual = vw.find_work_item_ids('fix #D09E3925-0981-4F88-AF3D-7A4BC668263C #AB810B41-2A46-4F94-B2E8-D36FBAF9FC8F')
        compare_sets(expect, actual)
        actual = vw.find_work_item_ids('fix #D09E3925-0981-4F88-AF3D-7A4BC668263C, #AB810B41-2A46-4F94-B2E8-D36FBAF9FC8F')
        compare_sets(expect, actual)
        actual = vw.find_work_item_ids('fix #D09E3925-0981-4F88-AF3D-7A4BC668263C,#AB810B41-2A46-4F94-B2E8-D36FBAF9FC8F')
        compare_sets(expect, actual)

    def test_find_duplicate_guid_ids(self):
        vw = vowei_hook_common.vowei_hook_base()
        expect = ['D09E3925-0981-4F88-AF3D-7A4BC668263C']
        actual = vw.find_work_item_ids('fix #D09E3925-0981-4F88-AF3D-7A4BC668263C #D09E3925-0981-4F88-AF3D-7A4BC668263C')
        compare_sets(expect, actual)
        
        actual = vw.find_work_item_ids('fix #D09E3925-0981-4F88-AF3D-7A4BC668263C, #D09E3925-0981-4F88-AF3D-7A4BC668263C')
        compare_sets(expect, actual)
        
        actual = vw.find_work_item_ids('fix #D09E3925-0981-4F88-AF3D-7A4BC668263C,#D09E3925-0981-4F88-AF3D-7A4BC668263C')
        compare_sets(expect, actual)
            
    def test_find_unicode_guid_ids(self):
        vw = vowei_hook_common.vowei_hook_base()
        expect = ['D09E3925-0981-4F88-AF3D-7A4BC668263C']
        actual = vw.find_work_item_ids(u'完成工作任务bug #D09E3925-0981-4F88-AF3D-7A4BC668263C')
        compare_sets(expect, actual)
        
        expect = ['AB810B41-2A46-4F94-B2E8-D36FBAF9FC8F', 'D09E3925-0981-4F88-AF3D-7A4BC668263C']
        actual = vw.find_work_item_ids(u'完成工作任务#AB810B41-2A46-4F94-B2E8-D36FBAF9FC8F和bug #D09E3925-0981-4F88-AF3D-7A4BC668263C')
        compare_sets(expect, actual)
        
    # 这个测试用例暂时还不能通过,以数字格式作为ID和GUID格式在正则表达式匹配的时候,
    # 会有很多的出入,因此不支持在一条注释里面混合使用两个ID格式的做法.        
    def test_find_mixed_int_guid_ids(self):
        vw = vowei_hook_common.vowei_hook_base()
        expect = [12, 'D09E3925-0981-4F88-AF3D-7A4BC668263C', 56]
        actual = vw.find_work_item_ids(u'完成工作任务#12和bug #D09E3925-0981-4F88-AF3D-7A4BC668263C还有风险#56')
        compare_sets(expect, actual)

    def test_find_start_with_int_guid_ids(self):
        vw = vowei_hook_common.vowei_hook_base()
        expect = ['3106da8e-1270-48f1-97ac-5a055254d380']
        actual = vw.find_work_item_ids(u'签入temp.txt来完成任务 #3106da8e-1270-48f1-97ac-5a055254d380')
        compare_sets(expect, actual)
        
def compare_sets(expect, actual):
    if ( len(expect) != len(actual) ):
        raise Exception('two set counts are different, expect: %d, actual: %d' % (len(expect), len(actual)))

    actual.difference_update(expect)
    if ( len(actual) != 0 ):
        raise Exception('two set have different items')

unittest.main()
