# -- coding: UTF-8 --
import sys
import unittest
sys.path.append('..\\')
import vowei_hook

class TestClass(unittest.TestCase):
    def test_read_config(self):
        vw = vowei_hook.svn_hook('read_config_test.cfg')
        if ( vw.host != 'www.vowei.com' ):
            raise Exception('Host configuration is not readable')
        if ( vw.svnlook_path != '"D:\\Program Files\\VisualSVN Server\\bin\\svnlook.exe"' ):
            raise Exception('svnlook path configuration is not readable')
        if ( vw.path != '/Comment/PostComments' ):
            raise Exception('path configuration is not readable')
    
    def test_read_usermap(self):
        vw = vowei_hook.svn_hook('read_config_test.cfg')
        expect = [('shiyimin', 'yimin_shi@hotmail.com'),
                  ('chenweixing', 'weixch@hotmail.com')]
        actual = vw.configitems('usermap')
        if ( len(expect) != len(actual) ):
            raise Exception('user map is not readable')

        for i in range(len(actual)):
            if ( actual[i][0] != expect[i][0] ):
                raise Exception('failed to read user map keys')
            if ( actual[i][1] != expect[i][1] ):
                raise Exception('failed to read user map values')

    def test_svnlookpath_contain_quotes(self):
        vw = vowei_hook.svn_hook('test_svnlookpath_contain_quotes.cfg')
        if ( vw.svnlook_path != '"D:\\Program Files\\VisualSVN Server\\bin\\svnlook.exe"' ):
            raise Exception('svnlook path configuration is not readable')

    def test_svnlookpath_withno_quotes(self):
        vw = vowei_hook.svn_hook('test_svnlookpath_withno_quotes.cfg')
        if ( vw.svnlook_path != '"D:\\svnlook.exe"' ):
            raise Exception('svnlook path configuration is not readable')

    def test_svninfo_get_committer_positive1(self):
        message = u'shiyimin\n \
                    2010-07-25 18:16:33 +0800 (周日, 25 七月 2010)\n \
                    12\n \
                    修复bug #155'
        actual = vowei_hook.get_svn_committer(message)
        if ( actual != 'shiyimin' ):
            raise Exception('Failed to get committer from message')

    def test_svninfo_get_committer_positive2(self):
        message = u'shiyimin\n'
        actual = vowei_hook.get_svn_committer(message)
        if ( actual != 'shiyimin' ):
            raise Exception('Failed to get committer from message')

    def test_svninfo_get_committer_negative(self):
        message = u'2010-11-21 16:51:29 +0800 (周日, 21 十一月 2010)\n \
                    16 \n\
                    fix work item #7'
        actual = vowei_hook.get_svn_committer(message)
        if ( actual != vowei_hook.ANNONYMOUS_USER_NAME ):
            raise Exception('Error get annony user from annonymous committer')

    def test_svninfo_get_committer_positive_ansii(self):
        message = 'shiyimin\n \
                   2010-07-25 18:16:33 +0800 (周日, 25 七月 2010)\n \
                   12\n \
                   修复bug #155'
        actual = vowei_hook.get_svn_committer(message)
        if ( actual != 'shiyimin' ):
            raise Exception('Failed to get committer from message')

    def test_svninfo_get_committer_oneline_positive_ansii(self):
        message = 'shiyimin\n'
        actual = vowei_hook.get_svn_committer(message)
        if ( actual != 'shiyimin' ):
            raise Exception('Failed to get committer from message')
        
        message = 'shiyimin'
        actual = vowei_hook.get_svn_committer(message)
        if ( actual != 'shiyimin' ):
            raise Exception('Failed to get committer from message')

    def test_svninfo_get_committer_oneline_negative_ansii(self):
        message = ''
        actual = vowei_hook.get_svn_committer(message)
        if ( actual != vowei_hook.ANNONYMOUS_USER_NAME ):
            raise Exception('Failed to get committer from message')
        
unittest.main()
