﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel.DataAnnotations;
using System.Runtime.Serialization;
using Vowei.Core.Models;

namespace Vowei.Core.Models
{
    /// <summary>
    /// 代表一个日常工作任务
    /// </summary>
    /// <remarks>
    /// 在Outlook里面的Calendar, FollowUp都可以使用Task模拟
    /// 
    /// 日常工作任务的状态是有版本控制的，因为用户可以设置一个基线（baseline）
    /// 这样后面实际工作时，由负责任务的人员更新任务的开始和结束执行日期
    /// </remarks>
    [DataContract]
    public class Task : Ticket, ISupportOws, IDuration
    {
        #region IDuration Members
        /// <summary>
        /// 任务开始执行的日期
        /// </summary>
        [DataMember]
        [Display(Name = "开始日期")]
        public DateTime? StartDate
        {
            get;
            set;
        }

        /// <summary>
        /// 任务完成得日期
        /// </summary>
        [DataMember]
        [Display(Name = "计划结束日期")]
        // TODO: 这个值需要限制在所从属的Milestone的DueDate属性的范围里。
        public DateTime? DueDate
        {
            get;
            set;
        }
        #endregion
               
        public int IdForOws { get; set; }
    }
}
