﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Vowei.Core.Models;
using System.Data.Entity.Infrastructure;
using Vowei.Core;
using System.Diagnostics;
using Vowei.Data.Models;
using System.Data.Entity;

namespace Vowei.Data
{
    public static class BuiltinSettingInitializer
    {
        public static void Feed(VoweiContextImpl context)
        {
            FeedPermissions(context);
            FeedPermissionSet(context);
            FeedResourceCalendar(context);
        }

        private static void FeedPermissions(VoweiContextImpl context)
        {
            var fields = typeof(Constants.Permission.Read).GetFields();

            foreach (var field in fields)
            {
                var value = field.GetValue(null) as string;
                Debug.Assert(value != null);
                context.Permissions.Add(new Permission()
                {
                    Title = value
                });
            }

            fields = typeof(Constants.Permission.Write).GetFields();

            foreach (var field in fields)
            {
                var value = field.GetValue(null) as string;
                Debug.Assert(value != null);
                context.Permissions.Add(new Permission()
                {
                    Title = value
                });
            }

            context.SaveChanges();
        }

        private static DbResourceCalendar CreateStandardResourceCalendar()
        {
            var calendar = new DbResourceCalendar("标准8小时工作制 - 八点上班")
            {
                ID = Constants.BuiltIn.ResouceCalendar.StandardCalendarId
            };
            var workweek = new WorkWeek("默认")
            {
                EffectPeriod = new Period { Start = DateTime.MinValue, Finish = DateTime.MaxValue }
            };

            workweek.Days.Add(new WorkDay(DayOfWeek.Sunday)
            {
                Type = WorkingTimeType.Nonworking
            });

            var am8 = new DateTime(1, 1, 1, 8, 0, 0);
            var pm12 = new DateTime(1, 1, 1, 12, 0, 0);
            var pm1 = new DateTime(1, 1, 1, 13, 0, 0);
            var pm5 = new DateTime(1, 1, 1, 17, 0, 0);

            for (var d = DayOfWeek.Monday; d < DayOfWeek.Saturday; ++d)
            {
                var day = new WorkDay(d)
                {
                    Type = WorkingTimeType.Working
                };

                day.Periods.Add(new Period { Start = am8, Finish = pm12 });
                day.Periods.Add(new Period { Start = pm1, Finish = pm5 });
                workweek.Days.Add(day);
            }

            workweek.Days.Add(new WorkDay(DayOfWeek.Saturday)
            {
                Type = WorkingTimeType.Nonworking
            });

            calendar.WorkWeeks.Add(workweek);
            return calendar;
        }

        private static DbResourceCalendar CreateChinaTop1CitiesResourceCalendar()
        {
            var calendar = new DbResourceCalendar("标准8小时工作制 - 九点上班");
            var workweek = new WorkWeek("默认")
            {
                EffectPeriod = new Period { Start = DateTime.MinValue, Finish = DateTime.MaxValue }
            };

            workweek.Days.Add(new WorkDay(DayOfWeek.Sunday)
            {
                Type = WorkingTimeType.Nonworking
            });

            var am9 = new DateTime(1, 1, 1, 9, 0, 0);
            var pm12 = new DateTime(1, 1, 1, 12, 0, 0);
            var pm1 = new DateTime(1, 1, 1, 13, 0, 0);
            var pm6 = new DateTime(1, 1, 1, 18, 0, 0);

            for (var d = DayOfWeek.Monday; d < DayOfWeek.Saturday; ++d)
            {
                var day = new WorkDay(d)
                {
                    Type = WorkingTimeType.Working
                };

                day.Periods.Add(new Period { Start = am9, Finish = pm12 });
                day.Periods.Add(new Period { Start = pm1, Finish = pm6 });
                workweek.Days.Add(day);
            }

            workweek.Days.Add(new WorkDay(DayOfWeek.Saturday)
            {
                Type = WorkingTimeType.Nonworking
            });

            calendar.WorkWeeks.Add(workweek);
            return calendar;
        }

        private static DbResourceCalendar Create24HoursResourceCalendar()
        {
            var calendar = new DbResourceCalendar("24小时工作制")
            {
                ID = Constants.BuiltIn.ResouceCalendar.TwentyFourHoursCalendarId
            };
            var workweek = new WorkWeek("默认")
            {
                EffectPeriod = new Period { Start = DateTime.MinValue, Finish = DateTime.MaxValue }
            };

            for (var d = DayOfWeek.Sunday; d <= DayOfWeek.Saturday; ++d)
            {
                var day = new WorkDay(d)
                {
                    Type = WorkingTimeType.Working
                };

                day.Periods.Add(new Period
                {
                    Start = new DateTime(1, 1, 1, 0, 0, 0),
                    Finish = new DateTime(1, 1, 2, 0, 0, 0, 0)
                });
                workweek.Days.Add(day);
            }

            calendar.WorkWeeks.Add(workweek);
            return calendar;
        }

        private static DbResourceCalendar CreateNightShiftResourceCalendar()
        {
            var calendar = new DbResourceCalendar("Night Shift")
            {
                ID = Constants.BuiltIn.ResouceCalendar.NightShiftCalendarId
            };
            var workweek = new WorkWeek("Default")
            {
                EffectPeriod = new Period { Start = DateTime.MinValue, Finish = DateTime.MaxValue }
            };

            workweek.Days.Add(new WorkDay(DayOfWeek.Sunday)
            {
                Type = WorkingTimeType.Nonworking
            });

            var am12 = new DateTime(1, 1, 1, 0, 0, 0);
            var am3 = new DateTime(1, 1, 1, 3, 0, 0);
            var am4 = new DateTime(1, 1, 1, 4, 0, 0);
            var am8 = new DateTime(1, 1, 1, 8, 0, 0);
            var pm11 = new DateTime(1, 1, 1, 11, 0, 0);
            var am12n = new DateTime(1, 1, 2, 0, 0, 0);

            var day = new WorkDay(DayOfWeek.Monday) { Type = WorkingTimeType.Working };
            day.Periods.Add(new Period { Start = pm11, Finish = am12n });

            for (var d = DayOfWeek.Tuesday; d <= DayOfWeek.Friday; ++d)
            {
                day = new WorkDay(d)
                {
                    Type = WorkingTimeType.Working
                };

                day.Periods.Add(new Period { Start = am12, Finish = am3 });
                day.Periods.Add(new Period { Start = am4, Finish = am8 });
                day.Periods.Add(new Period { Start = pm11, Finish = am12n });
                workweek.Days.Add(day);
            }

            day = new WorkDay(DayOfWeek.Saturday) { Type = WorkingTimeType.Working };
            day.Periods.Add(new Period { Start = am12, Finish = am3 });
            day.Periods.Add(new Period { Start = am4, Finish = am8 });

            calendar.WorkWeeks.Add(workweek);
            return calendar;
        }

        private static void FeedResourceCalendar(VoweiContextImpl context)
        {
            context.DbResourceCalendars.Add(CreateStandardResourceCalendar());
            context.DbResourceCalendars.Add(Create24HoursResourceCalendar());
            context.DbResourceCalendars.Add(CreateNightShiftResourceCalendar());
            context.DbResourceCalendars.Add(CreateChinaTop1CitiesResourceCalendar());

            context.SaveChanges();
        }

        private static void FeedPermissionSet(VoweiContextImpl context)
        {
            #region 项目管理权限集
            var pset = new PermissionSet()
            {
                Title = Constants.BuiltIn.PermissionSet.ManageProjectPermissionSet
            };

            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Project));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Milestone));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Requirement));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Task));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Risk));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Bug));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Comment));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.ChangeSet));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Comment));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.MeetingRequest));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Permission));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.PermissionSet));

            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Write.Project));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Write.Milestone));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Write.Requirement));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Write.Task));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Write.Risk));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Write.Bug));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Write.MeetingRequest));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Write.Permission));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Write.PermissionSet));

            context.PermissionSet.Add(pset);
            #endregion

            #region 浏览项目状态权限集
            pset = new PermissionSet()
            {
                Title = Constants.BuiltIn.PermissionSet.ProjectViewerPermissionSet
            };

            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Project));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Milestone));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Requirement));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Task));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Risk));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Bug));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Comment));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.ChangeSet));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Comment));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.MeetingRequest));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Permission));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.PermissionSet));

            context.PermissionSet.Add(pset);
            #endregion

            #region 项目概览权限集
            pset = new PermissionSet()
            {
                Title = "项目概览"
            };

            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Project));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Requirement));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Risk));

            context.PermissionSet.Add(pset);
            #endregion

            #region 项目成员权限集
            pset = new PermissionSet()
            {
                Title = Constants.BuiltIn.PermissionSet.ProjectMemeberPermissionSet
            };

            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Project));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Milestone));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Requirement));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Task));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Risk));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Bug));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Comment));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.ChangeSet));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Comment));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.MeetingRequest));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.Permission));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Read.PermissionSet));

            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Write.Task));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Write.Risk));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Write.Bug));
            pset.Children.Add(context.Permissions.First(
                p => p.Title == Constants.Permission.Write.MeetingRequest));

            context.PermissionSet.Add(pset);
            #endregion

            context.SaveChanges();
        }
    }

#if TEST
    public static class VirtualContextInitializer
    {
        public static readonly string[,] MsfMilestoneTitles = new string[,] {
            {"Vision/Scope Approved",""},
            {"", "Core Team Organized" },
            {"", "Vision/Scope Drafted" },
            {"Project Plans Approved",""},
            {"", "Technology Validatoin"},
            {"", "Functional Specification Baselined"},
            {"", "Master Plan Baselined"},
            {"", "Master Schedule Baselined"},
            {"", "Development and Test Environment Set Up"},
            {"Scope Complete", "" },
            {"", "Proof of Concept Complete"},
            {"", "Internal Build n Complete, Internal build n+1 Complete"},
            {"Release Readiness Approved", ""},
            {"", "Bug Convergence" },
            {"", "Zero Bug Bounce" },
            {"", "Release Candidates" },
            {"", "Pre-Production Test Complete"},
            {"", "User Acceptance Testing Complete"},
            {"", "Pilot Complete"},
            {"Deployment Complete", ""},
            {"", "Core Components Deployed" },
            {"", "Site Deployments Complete Interim Milestone"},
            {"", "Deployment Stable Interim Miletone"}
        };

        public static void Feed(VoweiContextImpl context)
        {
            BuiltinSettingInitializer.Feed(context);

            var user = Constants.BuiltIn.Users.Administrator;
            var calendar = context.DbResourceCalendars.Find(Constants.BuiltIn.ResouceCalendar.StandardCalendarId);

            #region MSF项目类型
            var project = new Project()
            {
                Title = "MSF Project",
                Owner = user,
                StartDate = DateTime.Now,
                ProjectBaseCalendar = calendar
            };
            context.Projects.Add(project);
            context.SaveChanges();

            Milestone prevMajor = null;
            for (int i = 0; i < MsfMilestoneTitles.GetLength(0); ++i)
            {
                var msfMilestone = new Milestone()
                {
                    DueDate = project.StartDate + TimeSpan.FromDays(i * 30)
                };
                context.Milestones.Add(msfMilestone);

                // 小里程碑 - Interim Milestone
                if (string.IsNullOrEmpty(MsfMilestoneTitles[i, 0]))
                {
                    msfMilestone.Title = MsfMilestoneTitles[i, 1];
                    prevMajor.Children.Add(msfMilestone);
                    context.SaveChanges();
                }
                else
                {
                    msfMilestone.Title = MsfMilestoneTitles[i, 0];
                    project.Children.Add(msfMilestone);
                    prevMajor = msfMilestone;
                    context.SaveChanges();
                }
            }

            context.SaveChanges();
            #endregion

            #region Scrum项目类型
            project = new Project()
            {
                Title = "Scrum Project",
                Owner = user,
                StartDate = DateTime.Now,
                ProjectBaseCalendar = calendar
            };
            context.Projects.Add(project);
            context.SaveChanges();
            
            for (int i = 1; i < 13; ++i)
            {
                var sprint = new Milestone()
                {
                    Title = string.Format("Spring {0}", i),
                    DueDate = project.StartDate + TimeSpan.FromDays(i * 30)
                };
                context.Milestones.Add(sprint);
                context.SaveChanges();
            }
            #endregion

            #region Svn项目
            project = new Project()
            {
                Title = "Svn测试项目",
                DueDate = new DateTime(2011, 1, 31),
                Owner = user,
                StartDate = new DateTime(2010, 12, 1),
                ScmType = Project.SupportedScm.Svn,
                ScmUrl = "http://shiyimin:88/svn/vowei/",
                ProjectBaseCalendar = calendar
            };
            context.Projects.Add(project);

            project = new Project()
            {
                Title = "不由yimin_shi@hotmail.com负责的项目",
                Owner = "otherpeoople@test.com",
                StartDate = DateTime.Now,
                DueDate = DateTime.Now + TimeSpan.FromDays(30),
                ProjectBaseCalendar = calendar
            };
            context.Projects.Add(project);
            context.SaveChanges();
            #endregion

            #region CMMI 项目类型
            project = new Project()
            {
                Title = "CMMI 项目",
                Owner = user,
                ProjectBaseCalendar = calendar
            };
            // project.ID = DefaultProjectId;
            context.Projects.Add(project);
            context.SaveChanges();

            var titles = new string[] 
            {
                "Performed Process",
                "Managed Process",
                "Defined Process",
                "Quantitative Managed Process",
                "Optimizing Process"
            };

            foreach (var title in titles)
            {
                var item = context.Create<Milestone>(title);
                project.Children.Add(item);
                context.Milestones.Add(item);
            }
            context.SaveChanges();

            var milestone = project.Children[1];
            titles = new string[] 
            {
                "Establish an Organizational Policy",
                "Plan the Process",
                "Provide Resources",
                "Assign Responsibility",
                "Train People",
                "Manage Configuration",
                "Identify and Involve Relevant Stackholders",
                "Monitor and Control the Process",
                "Objectively Evaluate Adherence",
                "Review Status with Higher Level Management"
            };
            var requirements = CreateRequirementHelper(context, titles, milestone);
            // context.SaveChanges();

            var requirement = requirements[1];
            // 跟需求相关的任务
            titles = new string[]
            {
                "Process description",
                "Standards and tasks for the work products and services of the process",
                "Specific objectives for the performance of the process (e.g., quality, time scale, cycle time, use of resources)",
                "Dependencies among the activities, work products, and services of the process",
                "Resources (e.g., funding, people, tools) needed to perform the process",
                "ResourceAssignment of responsibility and authority",
                "Training needed for performing and supporting the process",
                "Work products to be controlled and the level of control to be applied",
                "Measurement tasks to provide insight into the performance of the process, its work products, and its services",
                "Involvement of identified stakeholders",
                "Activities for monitoring and controlling the process",
                "Objective evaluation activities of the process",
                "Management review activities for the process and the work products"
            };
            CreateTasksInRequirement(context, titles, requirement);
            context.SaveChanges();
            #endregion

            #region 在测试过程中不会变更的测试项目
            project = new Project()
            {
                Title = "在测试过程中不会变更的测试项目",
                Owner = user,
                ProjectBaseCalendar = calendar
            };
            project.ID = UnchangedProjectId;
            context.Projects.Add(project);
            context.SaveChanges();

            titles = new string[] 
            {
                "Performed Process",
                "Managed Process",
                "Defined Process",
                "Quantitative Managed Process",
                "Optimizing Process"
            };

            foreach (var title in titles)
            {
                var item = context.Create<Milestone>(title);
                project.Children.Add(item);
                context.Milestones.Add(item);
            }
            context.SaveChanges();

            milestone = project.Children[1];
            titles = new string[] 
            {
                "Establish an Organizational Policy",
                "Plan the Process",
                "Provide Resources",
                "Assign Responsibility",
                "Train People",
                "Manage Configuration",
                "Identify and Involve Relevant Stackholders",
                "Monitor and Control the Process",
                "Objectively Evaluate Adherence",
                "Review Status with Higher Level Management"
            };
            requirements = CreateRequirementHelper(context, titles, milestone);

            requirement = requirements[1];
            // 跟需求相关的任务
            titles = new string[]
            {
                "Process description",
                "Standards and tasks for the work products and services of the process",
                "Specific objectives for the performance of the process (e.g., quality, time scale, cycle time, use of resources)",
                "Dependencies among the activities, work products, and services of the process",
                "Resources (e.g., funding, people, tools) needed to perform the process",
                "ResourceAssignment of responsibility and authority",
                "Training needed for performing and supporting the process",
                "Work products to be controlled and the level of control to be applied",
                "Measurement tasks to provide insight into the performance of the process, its work products, and its services",
                "Involvement of identified stakeholders",
                "Activities for monitoring and controlling the process",
                "Objective evaluation activities of the process",
                "Management review activities for the process and the work products"
            };
            CreateTasksInRequirement(context, titles, requirement);
            context.SaveChanges();
            #endregion

            #region Add Comment [Comment Samples]
            var comment = new Comment()
            {
                User = user,
                CommentDate = DateTime.Now,
                // 用户输入的字符串应该是下面这个格式
                //     "签入[f16569d55800]以修复任务#7"
                //
                // 网站自动替换成下面这个格式
                //     其中SCM的链接是从Project.ScmUrl里面获取的
                //
                // 后面要做的工作是
                // TODO: 编写一个Mercurial或SVN客户端，自动生成上面的字符串
                Text = "签入<a href='https://hg01.codeplex.com/vowei/rev/f16569d55800'>f16569d55800</a>以修复任务<a href='/Task/Details/8'>8</a>"
            };
            var task = context.Tasks.Where(t => t.Title == "Process description").First();
            task.Comments.Add(comment);
            context.Comments.Add(comment);
            #endregion

            context.SaveChanges();
        }
        private static void CreateTasksInRequirement(VoweiContextImpl context, string[] titles, Requirement requirement)
        {
            foreach (var title in titles)
            {
                var item = context.Create<Task>(title);
                item.AssignedTo = Constants.BuiltIn.Users.Administrator;
                context.Tasks.Add(item);
                requirement.Children.Add(item);
            }
        }
            
        private static Requirement[] CreateRequirementHelper(VoweiContextImpl context, string[] titles, Milestone parent)
        {
            var results = new List<Requirement>();

            foreach (var title in titles)
            {
                var item = context.Create<Requirement>(title);
                results.Add(item);
                item.AssignedTo = Constants.BuiltIn.Users.Administrator;
                item.BelongsToMilestone = parent;

                context.Requirements.Add(item);
            }

            return results.ToArray();
        }

        public static string MsfProjectTitle { get { return "MSF Project"; } }

        public static string MsfSubMilestoneTitle { get { return "Vision/Scope Drafted"; } }

        public static string ScrumSprintTitle { get { return "Spring 1"; } }

        private static Guid _UnchangedProjectId = new Guid("00000000-0000-0000-0000-000000000001");
        public static Guid UnchangedProjectId { get { return _UnchangedProjectId; } }

        private static Guid _defaultProjectId = new Guid("00000000-0000-0000-1000-000000000001");
        public static Guid DefaultProjectId { get { return _defaultProjectId; } }
    }
#endif

    public class VoweiContextTestInitializer : DropCreateDatabaseAlways<VoweiContextImpl>
    {
        protected override void Seed(VoweiContextImpl context)
        {
            VirtualContextInitializer.Feed(context);
            base.Seed(context);
        }
    }

    public class VoweiContextInitializer : DropCreateDatabaseIfModelChanges<VoweiContextImpl>
    {
        protected override void Seed(VoweiContextImpl context)
        {
            BuiltinSettingInitializer.Feed(context);
            base.Seed(context);
        }
    }
}
