﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Services;
using System.Xml;
using System.Data;
using System.Collections.Specialized;
using Vowei.Core.Models;
using Vowei.Services.SharePoint;
using System.Web.Services.Protocols;
using System.Web.Services.Description;
using Vowei.Web;
using Autofac;
using Vowei.Data.Models;
using Vowei.Service;
using vowei.Web.Helpers;
using Vowei.Web.Models;

namespace Vowei.Services.SharePoint
{
    [System.ComponentModel.ToolboxItem(false)]
    [WebServiceAttribute(Namespace = "http://schemas.microsoft.com/sharepoint/soap/")]
    [System.Web.Script.Services.ScriptService]
    [System.CodeDom.Compiler.GeneratedCodeAttribute("wsdl", "4.0.30319.1")]
    [WebServiceBindingAttribute(Name = "ListsSoap", Namespace = "http://schemas.microsoft.com/sharepoint/soap/")]
    public class TicketService : WebService, ISharePointList
    {        
        static XmlDocument _doc = null;
        static XmlNamespaceManager _nsm = null;
        static XmlNamespaceManager _snsm = null;
        static XmlNamespaceManager snsm
        {
            get
            {
                if (_snsm == null) { object throwaway = _Doc; }
                return _snsm;
            }
        }

        private static XmlDocument _Doc
        {
            get
            {
                if (_doc == null)
                {
                    _doc = new XmlDocument();
                    _nsm = new XmlNamespaceManager(_doc.NameTable);
                    _nsm.AddNamespace(string.Empty, "http://schemas.microsoft.com/sharepoint/soap/");
                    _snsm = new XmlNamespaceManager(_doc.NameTable);
                    _snsm.AddNamespace("s", _nsm.DefaultNamespace);
                }

                return _doc;
            }
        }

        private IVoweiContext _context;
        private IUser _currentUser;
        private IServiceHttpContext _httpContext;

        public TicketService()
            : this(MvcApplication.ContainerProviderStatic.ApplicationContainer.Resolve<IVoweiContext>(), 
                   new TicketServiceContext(),
                   // TODO: make this Ioc
                   new User())

        {
        }

        // TODO: remove this ctor once we refactor the code to use Ioc approach.
        public TicketService(IVoweiContext context, 
            IServiceHttpContext httpContext, IUser user)
        {
            _context = context;
            _httpContext = httpContext;
            _currentUser = user;
        }

        // TODO: remove this hard coded task list ID, it should be generated depends on listName
        private static Guid _hardCodeProviderId = new Guid("{7765B84F-6D32-4d31-B28E-6BC615D2F187}");
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/GetList", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode GetList(string listName)
        {
            XmlElement listElement = _Doc.CreateElement("List", _nsm.DefaultNamespace);
            var iProv = new TaskProvider(_context, _currentUser);

            ListsHelper.FillListAttributes(listName, listElement, _httpContext.ApplicationPath);
            XmlElement fieldsElement = NewElement(listElement, "Fields", null, true);
            List<Field> fields = ListsHelper.GetFieldsForListType(iProv.GetProviderType(_hardCodeProviderId));

            foreach (Field field in fields)
            {
                fieldsElement.AppendChild(field.GetFieldDefXml(_Doc, _context));
            }

            XmlElement regionalSettingsNode = NewElement(listElement, "RegionalSettings", null, true);
            NewElement(regionalSettingsNode, "Language", "1033");
            NewElement(regionalSettingsNode, "Locale", "1033");
            NewElement(regionalSettingsNode, "AdvanceHijri", "0");
            NewElement(regionalSettingsNode, "CalendarType", "1");
            NewElement(regionalSettingsNode, "Time24", "False");
            NewElement(regionalSettingsNode, "TimeZone", "300");
            NewElement(regionalSettingsNode, "SortOrder", "2070");
            NewElement(regionalSettingsNode, "Presence", "True");

            XmlElement serverSettings = NewElement(listElement, "ServerSettings", null, true);
            NewElement(serverSettings, "ServerVersion", "12.0.0.6219");
            NewElement(serverSettings, "RecycleBinEnabled", "True");
            NewElement(serverSettings, "ServerRelativeUrl", _httpContext.ApplicationPath);

            return listElement;
        }

        private XmlElement NewElement(XmlElement parentElement, string nodeName, string innerXml)
        {
            XmlElement retElement = NewElement(parentElement, nodeName, null, true);
            if (innerXml != null)
            {
                retElement.InnerText = innerXml;
            }
            return retElement;
        }

        private XmlElement NewElement(XmlElement parentElement, string nodeName, XmlElement refNode, bool appearBefore)
        {
            XmlElement retElement = NewElement(parentElement, nodeName);
            if (refNode != null)
            {
                if (appearBefore)
                    parentElement.InsertBefore(retElement, refNode);
                else
                    parentElement.InsertAfter(retElement, refNode);
            }
            else
            {
                parentElement.AppendChild(retElement);
            }
            return retElement;
        }

        private XmlElement NewElement(XmlElement parentElement, string nodeName)
        {
            return parentElement.OwnerDocument.CreateElement(nodeName, parentElement.NamespaceURI);
        }
        
        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/GetListAndView", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode GetListAndView(string listName, string viewName)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/DeleteList", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public void DeleteList(string listName)
        {
            throw new NotSupportedException("尚不支持从SharePoint web服务上删除项目这个功能，请从网站上进行操作！");
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/AddList", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode AddList(string listName, string description, int templateID)
        {
            throw new NotSupportedException("尚不支持从SharePoint web服务上创建项目这个功能，请从网站上进行操作！");
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/AddListFromFeature", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode AddListFromFeature(string listName, string description, System.Guid featureID, int templateID)
        {
            throw new NotSupportedException("尚不支持从SharePoint web服务上创建项目这个功能，请从网站上进行操作！");
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/UpdateList", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode UpdateList(string listName, XmlNode listProperties, XmlNode newFields, XmlNode updateFields, XmlNode deleteFields, string listVersion)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/GetListCollection", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode GetListCollection()
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/GetListItems", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode GetListItems(string listName, string viewName, XmlNode query, XmlNode viewFields, string rowLimit, XmlNode queryOptions, string webID)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/GetListItemChanges", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode GetListItemChanges(string listName, XmlNode viewFields, string since, XmlNode contains)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/GetListItemChangesWithKnowledge", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode GetListItemChangesWithKnowledge(string listName, string viewName, XmlNode query, XmlNode viewFields, string rowLimit, XmlNode queryOptions, string syncScope, XmlNode knowledge, XmlNode contains)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/GetListItemChangesSinceToken", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode GetListItemChangesSinceToken(string listName, string viewName, XmlNode query, XmlNode viewFields, string rowLimit, XmlNode queryOptions, string changeToken, XmlNode contains)
        {
            //List name should correspond to the provider ID if the stssync link was constructed correctly.
            const char sTab = '\t';
            System.Diagnostics.Debug.Print("GetListItemChangesSinceToken called with params:\n" +
                sTab + "listName = " + ((null == listName) ? string.Empty : listName).ToString() + "\n" +
                sTab + "viewName = " + ((null == viewName) ? string.Empty : viewName).ToString() + "\n" +
                sTab + "query = " + ((query == null) ? string.Empty : query.OuterXml.ToString()).ToString() + "\n" +
                sTab + "viewFields = " + ((null == viewFields) ? string.Empty : viewFields.OuterXml.ToString()).ToString() + "\n" +
                sTab + "rowLimit = " + ((null == rowLimit) ? string.Empty : rowLimit.ToString()).ToString() + "\n" +
                sTab + "queryOptions = " + ((null == queryOptions) ? string.Empty : queryOptions.OuterXml.ToString()).ToString() + "\n" +
                sTab + "changeToken = " + ((null == changeToken) ? string.Empty : changeToken).ToString() + "\n" +
                sTab + "contains = " + ((contains == null) ? string.Empty : contains.OuterXml.ToString()).ToString() + "\n=======================================");

            var iProv = new TaskProvider(_context, _currentUser);
            var id = Guid.Parse(listName);
            ChangeKey changeKey = null;
            if (changeToken != null) changeKey = new ChangeKey(changeToken);
            DataSet changes = iProv.GetUpdatesSinceToken(id, changeKey);
            ListType listType = ListType.Tasks;
            StringDictionary fieldMappings = iProv.GetFieldMappingsForListType(listType);

            int intRowLimit = (null == rowLimit) ? 0 : Int32.Parse(rowLimit);
            int startRow = 0;
            string requestThread = null;
            if (queryOptions != null)
            {
                XmlElement xelPaging = (XmlElement)queryOptions.SelectSingleNode("//s:Paging", snsm);
                string sPosNext = (xelPaging != null) ? xelPaging.GetAttribute("ListItemCollectionPositionNext") : null;
                if (sPosNext != null && sPosNext.IndexOf("&") > -1)
                {
                    string[] sPosNextParts = sPosNext.Split('&');
                    requestThread = sPosNextParts[0];
                    startRow = (sPosNext != null) ? Int32.Parse(sPosNextParts[1]) : 0;
                }
            }
            
            DataSet wssDS = new DataSet();
            try
            {
                object cacheValue = null;

                if (requestThread != null)
                {
                    cacheValue = _httpContext.Cache.Get(requestThread);
                }

                if (cacheValue != null)
                {
                    wssDS = (DataSet)cacheValue;
                }
                else
                {
                    requestThread = Guid.NewGuid().ToString("N");
                    wssDS = DataSetMapper.MapCustomDSToWSSDS(changes, listType.ToString(), fieldMappings, listType);
                    _httpContext.Cache.Insert(requestThread, wssDS, null, DateTime.MaxValue, new TimeSpan(0, 10, 0), System.Web.Caching.CacheItemPriority.Normal, null);
                }
            }
            catch { }
            /*
            <listitems MinTimeBetweenSyncs='0' RecommendedTimeBetweenSyncs='180' MaxBulkDocumentSyncSize='500' AlternateUrls='http://initechwss/' EffectivePermMask='FullMask' xmlns:s='uuid:BDC6E3F0-6DA3-11d1-A2A3-00AA00C14882'
     xmlns:dt='uuid:C2F41010-65B3-11d1-A29F-00AA00C14882'
     xmlns:rs='urn:schemas-microsoft-com:rowset'
     xmlns:z='#RowsetSchema'>
            */

            XmlElement listItems = _Doc.CreateElement("listitems", _nsm.DefaultNamespace);
            listItems.SetAttribute("MinTimeBetweenSyncs", "0");
            listItems.SetAttribute("RecommendedTimeBetweenSyncs", "180");
            listItems.SetAttribute("MaxBulkDocumentSyncSize", "500");
            listItems.SetAttribute("EffectivePermMask", "FullMask");
            listItems.SetAttribute("xmlns:s", "uuid:BDC6E3F0-6DA3-11d1-A2A3-00AA00C14882");
            listItems.SetAttribute("xmlns:dt", "uuid:C2F41010-65B3-11d1-A29F-00AA00C14882");
            listItems.SetAttribute("xmlns:rs", "urn:schemas-microsoft-com:rowset");
            listItems.SetAttribute("xmlns:z", "#RowsetSchema");

            XmlElement xelChanges = NewElement(listItems, "Changes", null, true);
            xelChanges.SetAttribute("LastChangeToken", new ChangeKey(1, 3, new Guid("{7765B84F-6D32-4d31-B28E-6BC615D2F187}"), DateTime.Now, 1).ToString());


            XmlElement xelDataNode = ListsHelper.DataSetToDataNode(listItems, wssDS, listType, intRowLimit, startRow, requestThread);
            if (xelDataNode.HasAttribute("ListItemCollectionPositionNext"))
            {
                xelChanges.SetAttribute("MoreChanges", "TRUE");
            }

            return listItems;
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/UpdateListItems", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode UpdateListItems(string listName, XmlNode updates)
        {
            XmlElement xelResults = _Doc.CreateElement("Results", _nsm.DefaultNamespace);

            var iProv = new TaskProvider(_context, _currentUser);

            ListType listType = ListType.Tasks;
            StringDictionary fieldListMapping = iProv.GetFieldMappingsForListType(listType);
            List<Field> fieldList = ListsHelper.GetFieldsForListType(listType);
            FieldDictionary fieldDictionary = ListsHelper.GetFieldDictionaryForListType(listType);

            var xelUpdates = (XmlElement)updates;

            string onError = xelUpdates.GetAttribute("OnError");
            string listVersion = xelUpdates.GetAttribute("ListVersion");
            string viewName = xelUpdates.GetAttribute("ViewName");

            XmlNodeList xnlMethods = xelUpdates.SelectNodes("//s:Method", snsm);

            DataSet emptyDS = iProv.GetEmptyDataSet();
            //DataSet wssDS = DataSetMapper.MapCustomDSToWSSDS(updatesDS, listType.ToString(), iProv.GetFieldMappingsForListType(prov.ID, listType), listType);
            DataTable emptyDT = emptyDS.Tables[0];
            XmlElement xelZRow = null;
            foreach (XmlElement xelMethod in xnlMethods)
            {
                XmlElement xelResult = NewElement(xelResults, "Result", null, true);
                string sMethodID = xelMethod.GetAttribute("ID");
                string sMethod = xelMethod.GetAttribute("Cmd");
                xelResult.SetAttribute("ID", sMethodID + "," + sMethod);

                uint errCode = 0;
                string errText = null;
                DataRow resultDR;

                try
                {
                    XmlNodeList xnlFields = xelMethod.SelectNodes("s:Field[@Name != 'MetaInfo']", snsm);
                    XmlElement xelMetaInfo = MetaInfoHelper.GetMetaInfoXml(xelMethod, snsm);
                    XmlElement xelIDField = (XmlElement)xelMethod.SelectSingleNode("s:Field[@Name='ID']", snsm);
                    int id = 0;
                    string sID = xelIDField.InnerText;
                    if (sID.ToLower() != "new")
                    {
                        try //to get the ID -- won't be there for the "New" method.
                        {
                            id = int.Parse(xelIDField.InnerText);
                        }
                        catch (FormatException)
                        {
                        }
                    }
                    DataRow updateDR = null;
                    switch (sMethod.ToLower())
                    {
                        case "update":
                        case "new":
                            if (sMethod.ToLower() == "update")
                            {
                                updateDR = iProv.GetSingleRow(id);
                            }
                            else //new
                            {
                                DataTable dt = iProv.GetEmptyDataSet().Tables[0];
                                updateDR = dt.NewRow();
                                dt.Rows.Add(updateDR);
                            }

                            if (updateDR != null)
                            {
                                foreach (XmlElement updateField in xnlFields)
                                {

                                    string wssName = updateField.GetAttribute("Name");
                                    string fieldName = DataSetMapper.GetCustomFieldName(wssName, fieldListMapping);
                                    if (emptyDT.Columns.Contains(fieldName))
                                    {
                                        if (fieldDictionary[wssName].ReadOnly) continue; //don't update readonly fields
                                        Type typeOfColumn = emptyDT.Columns[fieldName].DataType;
                                        object fieldValue = DataSetMapper.GetTypedValue(typeOfColumn, updateField.InnerText);
                                        if (fieldValue == null)
                                        {
                                            updateDR[fieldName] = DBNull.Value;
                                        }
                                        else
                                        {
                                            updateDR[fieldName] = fieldValue;
                                        }

                                    }
                                    else
                                    {
                                        xelMetaInfo.AppendChild(xelMetaInfo.OwnerDocument.ImportNode(updateField, true));
                                    }

                                }

                                if (xelMetaInfo != null)
                                {
                                    updateDR[MetaInfoHelper.GetMetaInfoColName(fieldListMapping)] = xelMetaInfo.OuterXml;
                                }
                            }

                            // TOOD: 需要考虑下面这段代码是否破坏了类之间的封装性！
                            updateDR["AssignedTo"] = _currentUser.GetIdentifier();

                            resultDR = iProv.Update(updateDR);

                            xelZRow = ListsHelper.CreateZRow(xelResult, fieldList, fieldListMapping, resultDR);


                            if (xelMetaInfo != null)
                            {
                                MetaInfoHelper.AddMetaInfoFromMetaInfoXml(xelZRow, xelMetaInfo);
                            }

                            xelResult.AppendChild(xelZRow);
                            break;

                        case "delete":
                            throw new NotSupportedException("尚不支持直接从Outlook里删除任务！");
                            // iProv.Delete(id);
                            // break;
                    }


                }
                catch (ApplicationException ex)
                {
                    errCode = 0x8007DA7E;
                    errText = ex.Message;
                }

                XmlElement xelErrorCode = NewElement(xelResult, "ErrorCode", xelZRow, true);
                string sErrCode = errCode.ToString("X").PadLeft(8, '0');
                xelErrorCode.InnerText = "0x" + sErrCode;

                if (errCode > 0)
                {
                    XmlElement xelErrorText = NewElement(xelResult, "ErrorText", errText);
                }


            }

            return xelResults;
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/UpdateListItemsWithKnowledge", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode UpdateListItemsWithKnowledge(string listName, XmlNode updates, string syncScope, XmlNode knowledge)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/AddDiscussionBoardItem", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode AddDiscussionBoardItem(string listName, [System.Xml.Serialization.XmlElementAttribute(DataType = "base64Binary")] byte[] message)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/AddWikiPage", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode AddWikiPage(string strListName, string listRelPageUrl, string wikiContent)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/GetVersionCollection", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode GetVersionCollection(string strlistID, string strlistItemID, string strFieldName)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/AddAttachment", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public string AddAttachment(string listName, string listItemID, string fileName, [System.Xml.Serialization.XmlElementAttribute(DataType = "base64Binary")] byte[] attachment)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/GetAttachmentCollection", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode GetAttachmentCollection(string listName, string listItemID)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/DeleteAttachment", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public void DeleteAttachment(string listName, string listItemID, string url)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/CheckOutFile", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public bool CheckOutFile(string pageUrl, string checkoutToLocal, string lastmodified)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/UndoCheckOut", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public bool UndoCheckOut(string pageUrl)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/CheckInFile", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public bool CheckInFile(string pageUrl, string comment, string CheckinType)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/GetListContentTypes", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode GetListContentTypes(string listName, string contentTypeId)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/GetListContentTypesAndProperties", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode GetListContentTypesAndProperties(string listName, string contentTypeId, string propertyPrefix, bool includeWebProperties, [System.Xml.Serialization.XmlIgnoreAttribute()] bool includeWebPropertiesSpecified)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/GetListContentType", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode GetListContentType(string listName, string contentTypeId)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/CreateContentType", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public string CreateContentType(string listName, string displayName, string parentType, XmlNode fields, XmlNode contentTypeProperties, string addToView)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/UpdateContentType", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode UpdateContentType(string listName, string contentTypeId, XmlNode contentTypeProperties, XmlNode newFields, XmlNode updateFields, XmlNode deleteFields, string addToView)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/DeleteContentType", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode DeleteContentType(string listName, string contentTypeId)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/UpdateContentTypeXmlDocument", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode UpdateContentTypeXmlDocument(string listName, string contentTypeId, XmlNode newDocument)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/UpdateContentTypesXmlDocument", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode UpdateContentTypesXmlDocument(string listName, XmlNode newDocument)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/DeleteContentTypeXmlDocument", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode DeleteContentTypeXmlDocument(string listName, string contentTypeId, string documentUri)
        {
            throw new NotImplementedException();
        }

        /// <remarks/>
        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/ApplyContentTypeToList", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public XmlNode ApplyContentTypeToList(string webUrl, string contentTypeId, string listName)
        {
            throw new NotImplementedException();
        }

        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/CreateRequirement", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public void CreateRequirement(Requirement requirement)
        {
            _context.Requirements.Add(requirement);
            _context.SaveChanges();
            // TODO: 需要添加可以自动释放资源的代码
        }

        [WebMethodAttribute()]
        [SoapDocumentMethodAttribute("http://schemas.microsoft.com/sharepoint/soap/CreateTask", RequestNamespace = "http://schemas.microsoft.com/sharepoint/soap/", ResponseNamespace = "http://schemas.microsoft.com/sharepoint/soap/", Use = System.Web.Services.Description.SoapBindingUse.Literal, ParameterStyle = System.Web.Services.Protocols.SoapParameterStyle.Wrapped)]
        public void CreateTask(Task task)
        {
            throw new NotImplementedException("暂时还只是一个占位符,用来导出Task类型！");
        }
    }
}
