﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using DocumentFormat.OpenXml.Packaging;
using Ap = DocumentFormat.OpenXml.ExtendedProperties;
using Vt = DocumentFormat.OpenXml.VariantTypes;
using DocumentFormat.OpenXml;
using DocumentFormat.OpenXml.Spreadsheet;
using X14 = DocumentFormat.OpenXml.Office2010.Excel;
using Xdr = DocumentFormat.OpenXml.Drawing.Spreadsheet;
using A = DocumentFormat.OpenXml.Drawing;
using C = DocumentFormat.OpenXml.Drawing.Charts;
using C14 = DocumentFormat.OpenXml.Office2010.Drawing.Charts;
using Vowei.Core;

namespace Vowei.Converters.Excel
{
    public class RequirementStatisticsExcelGenerator : DataConnectionConverter
    {
        public RequirementStatisticsExcelGenerator(ITranslator translator)
            : base(translator)
        {
            ConnectionName = "需求统计信息";
        }

        protected override void CreateParts(SpreadsheetDocument document)
        {
            WorkbookPart workbookPart1 = document.AddWorkbookPart();
            GenerateWorkbookPart1Content(workbookPart1);

            WorkbookStylesPart workbookStylesPart1 = workbookPart1.AddNewPart<WorkbookStylesPart>("rId8");
            GenerateWorkbookStylesPart1Content(workbookStylesPart1);

            WorksheetPart worksheetPart1 = workbookPart1.AddNewPart<WorksheetPart>("rId3");
            GenerateWorksheetPart1Content(worksheetPart1);

            DrawingsPart drawingsPart1 = worksheetPart1.AddNewPart<DrawingsPart>("rId2");
            GenerateDrawingsPart1Content(drawingsPart1);

            ChartPart chartPart1 = drawingsPart1.AddNewPart<ChartPart>("rId1");
            GenerateChartPart1Content(chartPart1);

            PivotTablePart pivotTablePart1 = worksheetPart1.AddNewPart<PivotTablePart>("rId1");
            GeneratePivotTablePart1Content(pivotTablePart1);

            PivotTableCacheDefinitionPart pivotTableCacheDefinitionPart1 = pivotTablePart1.AddNewPart<PivotTableCacheDefinitionPart>("rId1");
            GeneratePivotTableCacheDefinitionPart1Content(pivotTableCacheDefinitionPart1);

            PivotTableCacheRecordsPart pivotTableCacheRecordsPart1 = pivotTableCacheDefinitionPart1.AddNewPart<PivotTableCacheRecordsPart>("rId1");
            GeneratePivotTableCacheRecordsPart1Content(pivotTableCacheRecordsPart1);

            ConnectionsPart connectionsPart1 = workbookPart1.AddNewPart<ConnectionsPart>("rId7");
            GenerateConnectionsPart1Content(connectionsPart1);

            WorksheetPart worksheetPart2 = workbookPart1.AddNewPart<WorksheetPart>("rId2");
            GenerateWorksheetPart2Content(worksheetPart2);

            DrawingsPart drawingsPart2 = worksheetPart2.AddNewPart<DrawingsPart>("rId2");
            GenerateDrawingsPart2Content(drawingsPart2);

            ChartPart chartPart2 = drawingsPart2.AddNewPart<ChartPart>("rId1");
            GenerateChartPart2Content(chartPart2);

            PivotTablePart pivotTablePart2 = worksheetPart2.AddNewPart<PivotTablePart>("rId1");
            GeneratePivotTablePart2Content(pivotTablePart2);

            pivotTablePart2.AddPart(pivotTableCacheDefinitionPart1, "rId1");

            WorksheetPart worksheetPart3 = workbookPart1.AddNewPart<WorksheetPart>("rId1");
            GenerateWorksheetPart3Content(worksheetPart3);

            DrawingsPart drawingsPart3 = worksheetPart3.AddNewPart<DrawingsPart>("rId2");
            GenerateDrawingsPart3Content(drawingsPart3);

            ChartPart chartPart3 = drawingsPart3.AddNewPart<ChartPart>("rId1");
            GenerateChartPart3Content(chartPart3);

            PivotTablePart pivotTablePart3 = worksheetPart3.AddNewPart<PivotTablePart>("rId1");
            GeneratePivotTablePart3Content(pivotTablePart3);

            pivotTablePart3.AddPart(pivotTableCacheDefinitionPart1, "rId1");

            workbookPart1.AddPart(pivotTableCacheDefinitionPart1, "rId5");

            WorksheetPart worksheetPart4 = workbookPart1.AddNewPart<WorksheetPart>("rId4");
            GenerateWorksheetPart4Content(worksheetPart4);

            QueryTablePart queryTablePart1 = worksheetPart4.AddNewPart<QueryTablePart>("rId1");
            GenerateQueryTablePart1Content(queryTablePart1);

            SharedStringTablePart sharedStringTablePart1 = workbookPart1.AddNewPart<SharedStringTablePart>("rId9");
            GenerateSharedStringTablePart1Content(sharedStringTablePart1);

            SetPackageProperties(document);
        }

        protected void GenerateConnectionsPart1Content(ConnectionsPart connectionsPart1)
        {
            Connections connections1 = new Connections();

            Connection connection1 = new Connection() { Id = (UInt32Value)1U, RefreshOnLoad = true, Name = ConnectionName, Type = (UInt32Value)4U, RefreshedVersion = 4, Background = true, SaveData = true };

            WebQueryProperties webQueryProperties1 = new WebQueryProperties() { SourceData = true, ParsePreTag = true, Consecutive = true, RefreshedInExcel2000 = true, Url = StatisticsUrl, HtmlTables = true };

            Tables tables1 = new Tables() { Count = (UInt32Value)1U };
            CharacterValue characterValue1 = new CharacterValue() { Val = "tickets" };

            tables1.Append(characterValue1);

            webQueryProperties1.Append(tables1);

            connection1.Append(webQueryProperties1);

            connections1.Append(connection1);

            connectionsPart1.Connections = connections1;
        }

        // Generates content of queryTablePart1.
        protected void GenerateQueryTablePart1Content(QueryTablePart queryTablePart1)
        {
            QueryTable queryTable1 = new QueryTable() { Name = ConnectionName, ConnectionId = (UInt32Value)1U, AutoFormatId = (UInt32Value)16U, ApplyNumberFormats = false, ApplyBorderFormats = false, ApplyFontFormats = true, ApplyPatternFormats = true, ApplyAlignmentFormats = false, ApplyWidthHeightFormats = false };

            queryTablePart1.QueryTable = queryTable1;
        }

        // Generates content of workbookPart1.
        private void GenerateWorkbookPart1Content(WorkbookPart workbookPart1)
        {
            Workbook workbook1 = new Workbook();
            workbook1.AddNamespaceDeclaration("r", "http://schemas.openxmlformats.org/officeDocument/2006/relationships");
            FileVersion fileVersion1 = new FileVersion() { ApplicationName = "xl", LastEdited = "5", LowestEdited = "5", BuildVersion = "9302" };
            WorkbookProperties workbookProperties1 = new WorkbookProperties() { DefaultThemeVersion = (UInt32Value)124226U };

            BookViews bookViews1 = new BookViews();
            WorkbookView workbookView1 = new WorkbookView() { XWindow = 120, YWindow = 45, WindowWidth = (UInt32Value)13260U, WindowHeight = (UInt32Value)18270U, FirstSheet = (UInt32Value)1U, ActiveTab = (UInt32Value)3U };

            bookViews1.Append(workbookView1);

            Sheets sheets1 = new Sheets();
            Sheet sheet1 = new Sheet() { Name = "需求工作量分析", SheetId = (UInt32Value)2U, Id = "rId1" };
            Sheet sheet2 = new Sheet() { Name = "需求商业价值分析", SheetId = (UInt32Value)3U, Id = "rId2" };
            Sheet sheet3 = new Sheet() { Name = "需求工作量按工时分析", SheetId = (UInt32Value)4U, Id = "rId3" };
            Sheet sheet4 = new Sheet() { Name = ConnectionName, SheetId = (UInt32Value)1U, Id = "rId4" };

            sheets1.Append(sheet1);
            sheets1.Append(sheet2);
            sheets1.Append(sheet3);
            sheets1.Append(sheet4);

            DefinedNames definedNames1 = new DefinedNames();
            DefinedName definedName1 = new DefinedName() { Name = ConnectionName, LocalSheetId = (UInt32Value)3U };
            definedName1.Text = string.Format("\'{0}\'!$A$1:$M$25", ConnectionName);

            definedNames1.Append(definedName1);
            CalculationProperties calculationProperties1 = new CalculationProperties() { CalculationId = (UInt32Value)0U };

            PivotCaches pivotCaches1 = new PivotCaches();
            PivotCache pivotCache1 = new PivotCache() { CacheId = (UInt32Value)7U, Id = "rId5" };

            pivotCaches1.Append(pivotCache1);

            workbook1.Append(fileVersion1);
            workbook1.Append(workbookProperties1);
            workbook1.Append(bookViews1);
            workbook1.Append(sheets1);
            workbook1.Append(definedNames1);
            workbook1.Append(calculationProperties1);
            workbook1.Append(pivotCaches1);

            workbookPart1.Workbook = workbook1;
        }

        // Generates content of workbookStylesPart1.
        private void GenerateWorkbookStylesPart1Content(WorkbookStylesPart workbookStylesPart1)
        {
            Stylesheet stylesheet1 = new Stylesheet() { MCAttributes = new MarkupCompatibilityAttributes() { Ignorable = "x14ac" } };
            stylesheet1.AddNamespaceDeclaration("mc", "http://schemas.openxmlformats.org/markup-compatibility/2006");
            stylesheet1.AddNamespaceDeclaration("x14ac", "http://schemas.microsoft.com/office/spreadsheetml/2009/9/ac");

            Fonts fonts1 = new Fonts() { Count = (UInt32Value)18U, KnownFonts = true };

            Font font1 = new Font();
            FontSize fontSize1 = new FontSize() { Val = 11D };
            Color color1 = new Color() { Theme = (UInt32Value)1U };
            FontName fontName1 = new FontName() { Val = "Calibri" };
            FontFamilyNumbering fontFamilyNumbering1 = new FontFamilyNumbering() { Val = 2 };
            FontScheme fontScheme1 = new FontScheme() { Val = FontSchemeValues.Minor };

            font1.Append(fontSize1);
            font1.Append(color1);
            font1.Append(fontName1);
            font1.Append(fontFamilyNumbering1);
            font1.Append(fontScheme1);

            Font font2 = new Font();
            FontSize fontSize2 = new FontSize() { Val = 11D };
            Color color2 = new Color() { Theme = (UInt32Value)1U };
            FontName fontName2 = new FontName() { Val = "Calibri" };
            FontFamilyNumbering fontFamilyNumbering2 = new FontFamilyNumbering() { Val = 2 };
            FontScheme fontScheme2 = new FontScheme() { Val = FontSchemeValues.Minor };

            font2.Append(fontSize2);
            font2.Append(color2);
            font2.Append(fontName2);
            font2.Append(fontFamilyNumbering2);
            font2.Append(fontScheme2);

            Font font3 = new Font();
            Bold bold1 = new Bold();
            FontSize fontSize3 = new FontSize() { Val = 18D };
            Color color3 = new Color() { Theme = (UInt32Value)3U };
            FontName fontName3 = new FontName() { Val = "Cambria" };
            FontFamilyNumbering fontFamilyNumbering3 = new FontFamilyNumbering() { Val = 2 };
            FontScheme fontScheme3 = new FontScheme() { Val = FontSchemeValues.Major };

            font3.Append(bold1);
            font3.Append(fontSize3);
            font3.Append(color3);
            font3.Append(fontName3);
            font3.Append(fontFamilyNumbering3);
            font3.Append(fontScheme3);

            Font font4 = new Font();
            Bold bold2 = new Bold();
            FontSize fontSize4 = new FontSize() { Val = 15D };
            Color color4 = new Color() { Theme = (UInt32Value)3U };
            FontName fontName4 = new FontName() { Val = "Calibri" };
            FontFamilyNumbering fontFamilyNumbering4 = new FontFamilyNumbering() { Val = 2 };
            FontScheme fontScheme4 = new FontScheme() { Val = FontSchemeValues.Minor };

            font4.Append(bold2);
            font4.Append(fontSize4);
            font4.Append(color4);
            font4.Append(fontName4);
            font4.Append(fontFamilyNumbering4);
            font4.Append(fontScheme4);

            Font font5 = new Font();
            Bold bold3 = new Bold();
            FontSize fontSize5 = new FontSize() { Val = 13D };
            Color color5 = new Color() { Theme = (UInt32Value)3U };
            FontName fontName5 = new FontName() { Val = "Calibri" };
            FontFamilyNumbering fontFamilyNumbering5 = new FontFamilyNumbering() { Val = 2 };
            FontScheme fontScheme5 = new FontScheme() { Val = FontSchemeValues.Minor };

            font5.Append(bold3);
            font5.Append(fontSize5);
            font5.Append(color5);
            font5.Append(fontName5);
            font5.Append(fontFamilyNumbering5);
            font5.Append(fontScheme5);

            Font font6 = new Font();
            Bold bold4 = new Bold();
            FontSize fontSize6 = new FontSize() { Val = 11D };
            Color color6 = new Color() { Theme = (UInt32Value)3U };
            FontName fontName6 = new FontName() { Val = "Calibri" };
            FontFamilyNumbering fontFamilyNumbering6 = new FontFamilyNumbering() { Val = 2 };
            FontScheme fontScheme6 = new FontScheme() { Val = FontSchemeValues.Minor };

            font6.Append(bold4);
            font6.Append(fontSize6);
            font6.Append(color6);
            font6.Append(fontName6);
            font6.Append(fontFamilyNumbering6);
            font6.Append(fontScheme6);

            Font font7 = new Font();
            FontSize fontSize7 = new FontSize() { Val = 11D };
            Color color7 = new Color() { Rgb = "FF006100" };
            FontName fontName7 = new FontName() { Val = "Calibri" };
            FontFamilyNumbering fontFamilyNumbering7 = new FontFamilyNumbering() { Val = 2 };
            FontScheme fontScheme7 = new FontScheme() { Val = FontSchemeValues.Minor };

            font7.Append(fontSize7);
            font7.Append(color7);
            font7.Append(fontName7);
            font7.Append(fontFamilyNumbering7);
            font7.Append(fontScheme7);

            Font font8 = new Font();
            FontSize fontSize8 = new FontSize() { Val = 11D };
            Color color8 = new Color() { Rgb = "FF9C0006" };
            FontName fontName8 = new FontName() { Val = "Calibri" };
            FontFamilyNumbering fontFamilyNumbering8 = new FontFamilyNumbering() { Val = 2 };
            FontScheme fontScheme8 = new FontScheme() { Val = FontSchemeValues.Minor };

            font8.Append(fontSize8);
            font8.Append(color8);
            font8.Append(fontName8);
            font8.Append(fontFamilyNumbering8);
            font8.Append(fontScheme8);

            Font font9 = new Font();
            FontSize fontSize9 = new FontSize() { Val = 11D };
            Color color9 = new Color() { Rgb = "FF9C6500" };
            FontName fontName9 = new FontName() { Val = "Calibri" };
            FontFamilyNumbering fontFamilyNumbering9 = new FontFamilyNumbering() { Val = 2 };
            FontScheme fontScheme9 = new FontScheme() { Val = FontSchemeValues.Minor };

            font9.Append(fontSize9);
            font9.Append(color9);
            font9.Append(fontName9);
            font9.Append(fontFamilyNumbering9);
            font9.Append(fontScheme9);

            Font font10 = new Font();
            FontSize fontSize10 = new FontSize() { Val = 11D };
            Color color10 = new Color() { Rgb = "FF3F3F76" };
            FontName fontName10 = new FontName() { Val = "Calibri" };
            FontFamilyNumbering fontFamilyNumbering10 = new FontFamilyNumbering() { Val = 2 };
            FontScheme fontScheme10 = new FontScheme() { Val = FontSchemeValues.Minor };

            font10.Append(fontSize10);
            font10.Append(color10);
            font10.Append(fontName10);
            font10.Append(fontFamilyNumbering10);
            font10.Append(fontScheme10);

            Font font11 = new Font();
            Bold bold5 = new Bold();
            FontSize fontSize11 = new FontSize() { Val = 11D };
            Color color11 = new Color() { Rgb = "FF3F3F3F" };
            FontName fontName11 = new FontName() { Val = "Calibri" };
            FontFamilyNumbering fontFamilyNumbering11 = new FontFamilyNumbering() { Val = 2 };
            FontScheme fontScheme11 = new FontScheme() { Val = FontSchemeValues.Minor };

            font11.Append(bold5);
            font11.Append(fontSize11);
            font11.Append(color11);
            font11.Append(fontName11);
            font11.Append(fontFamilyNumbering11);
            font11.Append(fontScheme11);

            Font font12 = new Font();
            Bold bold6 = new Bold();
            FontSize fontSize12 = new FontSize() { Val = 11D };
            Color color12 = new Color() { Rgb = "FFFA7D00" };
            FontName fontName12 = new FontName() { Val = "Calibri" };
            FontFamilyNumbering fontFamilyNumbering12 = new FontFamilyNumbering() { Val = 2 };
            FontScheme fontScheme12 = new FontScheme() { Val = FontSchemeValues.Minor };

            font12.Append(bold6);
            font12.Append(fontSize12);
            font12.Append(color12);
            font12.Append(fontName12);
            font12.Append(fontFamilyNumbering12);
            font12.Append(fontScheme12);

            Font font13 = new Font();
            FontSize fontSize13 = new FontSize() { Val = 11D };
            Color color13 = new Color() { Rgb = "FFFA7D00" };
            FontName fontName13 = new FontName() { Val = "Calibri" };
            FontFamilyNumbering fontFamilyNumbering13 = new FontFamilyNumbering() { Val = 2 };
            FontScheme fontScheme13 = new FontScheme() { Val = FontSchemeValues.Minor };

            font13.Append(fontSize13);
            font13.Append(color13);
            font13.Append(fontName13);
            font13.Append(fontFamilyNumbering13);
            font13.Append(fontScheme13);

            Font font14 = new Font();
            Bold bold7 = new Bold();
            FontSize fontSize14 = new FontSize() { Val = 11D };
            Color color14 = new Color() { Theme = (UInt32Value)0U };
            FontName fontName14 = new FontName() { Val = "Calibri" };
            FontFamilyNumbering fontFamilyNumbering14 = new FontFamilyNumbering() { Val = 2 };
            FontScheme fontScheme14 = new FontScheme() { Val = FontSchemeValues.Minor };

            font14.Append(bold7);
            font14.Append(fontSize14);
            font14.Append(color14);
            font14.Append(fontName14);
            font14.Append(fontFamilyNumbering14);
            font14.Append(fontScheme14);

            Font font15 = new Font();
            FontSize fontSize15 = new FontSize() { Val = 11D };
            Color color15 = new Color() { Rgb = "FFFF0000" };
            FontName fontName15 = new FontName() { Val = "Calibri" };
            FontFamilyNumbering fontFamilyNumbering15 = new FontFamilyNumbering() { Val = 2 };
            FontScheme fontScheme15 = new FontScheme() { Val = FontSchemeValues.Minor };

            font15.Append(fontSize15);
            font15.Append(color15);
            font15.Append(fontName15);
            font15.Append(fontFamilyNumbering15);
            font15.Append(fontScheme15);

            Font font16 = new Font();
            Italic italic1 = new Italic();
            FontSize fontSize16 = new FontSize() { Val = 11D };
            Color color16 = new Color() { Rgb = "FF7F7F7F" };
            FontName fontName16 = new FontName() { Val = "Calibri" };
            FontFamilyNumbering fontFamilyNumbering16 = new FontFamilyNumbering() { Val = 2 };
            FontScheme fontScheme16 = new FontScheme() { Val = FontSchemeValues.Minor };

            font16.Append(italic1);
            font16.Append(fontSize16);
            font16.Append(color16);
            font16.Append(fontName16);
            font16.Append(fontFamilyNumbering16);
            font16.Append(fontScheme16);

            Font font17 = new Font();
            Bold bold8 = new Bold();
            FontSize fontSize17 = new FontSize() { Val = 11D };
            Color color17 = new Color() { Theme = (UInt32Value)1U };
            FontName fontName17 = new FontName() { Val = "Calibri" };
            FontFamilyNumbering fontFamilyNumbering17 = new FontFamilyNumbering() { Val = 2 };
            FontScheme fontScheme17 = new FontScheme() { Val = FontSchemeValues.Minor };

            font17.Append(bold8);
            font17.Append(fontSize17);
            font17.Append(color17);
            font17.Append(fontName17);
            font17.Append(fontFamilyNumbering17);
            font17.Append(fontScheme17);

            Font font18 = new Font();
            FontSize fontSize18 = new FontSize() { Val = 11D };
            Color color18 = new Color() { Theme = (UInt32Value)0U };
            FontName fontName18 = new FontName() { Val = "Calibri" };
            FontFamilyNumbering fontFamilyNumbering18 = new FontFamilyNumbering() { Val = 2 };
            FontScheme fontScheme18 = new FontScheme() { Val = FontSchemeValues.Minor };

            font18.Append(fontSize18);
            font18.Append(color18);
            font18.Append(fontName18);
            font18.Append(fontFamilyNumbering18);
            font18.Append(fontScheme18);

            fonts1.Append(font1);
            fonts1.Append(font2);
            fonts1.Append(font3);
            fonts1.Append(font4);
            fonts1.Append(font5);
            fonts1.Append(font6);
            fonts1.Append(font7);
            fonts1.Append(font8);
            fonts1.Append(font9);
            fonts1.Append(font10);
            fonts1.Append(font11);
            fonts1.Append(font12);
            fonts1.Append(font13);
            fonts1.Append(font14);
            fonts1.Append(font15);
            fonts1.Append(font16);
            fonts1.Append(font17);
            fonts1.Append(font18);

            Fills fills1 = new Fills() { Count = (UInt32Value)33U };

            Fill fill1 = new Fill();
            PatternFill patternFill1 = new PatternFill() { PatternType = PatternValues.None };

            fill1.Append(patternFill1);

            Fill fill2 = new Fill();
            PatternFill patternFill2 = new PatternFill() { PatternType = PatternValues.Gray125 };

            fill2.Append(patternFill2);

            Fill fill3 = new Fill();

            PatternFill patternFill3 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor1 = new ForegroundColor() { Rgb = "FFC6EFCE" };

            patternFill3.Append(foregroundColor1);

            fill3.Append(patternFill3);

            Fill fill4 = new Fill();

            PatternFill patternFill4 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor2 = new ForegroundColor() { Rgb = "FFFFC7CE" };

            patternFill4.Append(foregroundColor2);

            fill4.Append(patternFill4);

            Fill fill5 = new Fill();

            PatternFill patternFill5 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor3 = new ForegroundColor() { Rgb = "FFFFEB9C" };

            patternFill5.Append(foregroundColor3);

            fill5.Append(patternFill5);

            Fill fill6 = new Fill();

            PatternFill patternFill6 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor4 = new ForegroundColor() { Rgb = "FFFFCC99" };

            patternFill6.Append(foregroundColor4);

            fill6.Append(patternFill6);

            Fill fill7 = new Fill();

            PatternFill patternFill7 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor5 = new ForegroundColor() { Rgb = "FFF2F2F2" };

            patternFill7.Append(foregroundColor5);

            fill7.Append(patternFill7);

            Fill fill8 = new Fill();

            PatternFill patternFill8 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor6 = new ForegroundColor() { Rgb = "FFA5A5A5" };

            patternFill8.Append(foregroundColor6);

            fill8.Append(patternFill8);

            Fill fill9 = new Fill();

            PatternFill patternFill9 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor7 = new ForegroundColor() { Rgb = "FFFFFFCC" };

            patternFill9.Append(foregroundColor7);

            fill9.Append(patternFill9);

            Fill fill10 = new Fill();

            PatternFill patternFill10 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor8 = new ForegroundColor() { Theme = (UInt32Value)4U };

            patternFill10.Append(foregroundColor8);

            fill10.Append(patternFill10);

            Fill fill11 = new Fill();

            PatternFill patternFill11 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor9 = new ForegroundColor() { Theme = (UInt32Value)4U, Tint = 0.79998168889431442D };
            BackgroundColor backgroundColor1 = new BackgroundColor() { Indexed = (UInt32Value)65U };

            patternFill11.Append(foregroundColor9);
            patternFill11.Append(backgroundColor1);

            fill11.Append(patternFill11);

            Fill fill12 = new Fill();

            PatternFill patternFill12 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor10 = new ForegroundColor() { Theme = (UInt32Value)4U, Tint = 0.59999389629810485D };
            BackgroundColor backgroundColor2 = new BackgroundColor() { Indexed = (UInt32Value)65U };

            patternFill12.Append(foregroundColor10);
            patternFill12.Append(backgroundColor2);

            fill12.Append(patternFill12);

            Fill fill13 = new Fill();

            PatternFill patternFill13 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor11 = new ForegroundColor() { Theme = (UInt32Value)4U, Tint = 0.39997558519241921D };
            BackgroundColor backgroundColor3 = new BackgroundColor() { Indexed = (UInt32Value)65U };

            patternFill13.Append(foregroundColor11);
            patternFill13.Append(backgroundColor3);

            fill13.Append(patternFill13);

            Fill fill14 = new Fill();

            PatternFill patternFill14 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor12 = new ForegroundColor() { Theme = (UInt32Value)5U };

            patternFill14.Append(foregroundColor12);

            fill14.Append(patternFill14);

            Fill fill15 = new Fill();

            PatternFill patternFill15 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor13 = new ForegroundColor() { Theme = (UInt32Value)5U, Tint = 0.79998168889431442D };
            BackgroundColor backgroundColor4 = new BackgroundColor() { Indexed = (UInt32Value)65U };

            patternFill15.Append(foregroundColor13);
            patternFill15.Append(backgroundColor4);

            fill15.Append(patternFill15);

            Fill fill16 = new Fill();

            PatternFill patternFill16 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor14 = new ForegroundColor() { Theme = (UInt32Value)5U, Tint = 0.59999389629810485D };
            BackgroundColor backgroundColor5 = new BackgroundColor() { Indexed = (UInt32Value)65U };

            patternFill16.Append(foregroundColor14);
            patternFill16.Append(backgroundColor5);

            fill16.Append(patternFill16);

            Fill fill17 = new Fill();

            PatternFill patternFill17 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor15 = new ForegroundColor() { Theme = (UInt32Value)5U, Tint = 0.39997558519241921D };
            BackgroundColor backgroundColor6 = new BackgroundColor() { Indexed = (UInt32Value)65U };

            patternFill17.Append(foregroundColor15);
            patternFill17.Append(backgroundColor6);

            fill17.Append(patternFill17);

            Fill fill18 = new Fill();

            PatternFill patternFill18 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor16 = new ForegroundColor() { Theme = (UInt32Value)6U };

            patternFill18.Append(foregroundColor16);

            fill18.Append(patternFill18);

            Fill fill19 = new Fill();

            PatternFill patternFill19 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor17 = new ForegroundColor() { Theme = (UInt32Value)6U, Tint = 0.79998168889431442D };
            BackgroundColor backgroundColor7 = new BackgroundColor() { Indexed = (UInt32Value)65U };

            patternFill19.Append(foregroundColor17);
            patternFill19.Append(backgroundColor7);

            fill19.Append(patternFill19);

            Fill fill20 = new Fill();

            PatternFill patternFill20 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor18 = new ForegroundColor() { Theme = (UInt32Value)6U, Tint = 0.59999389629810485D };
            BackgroundColor backgroundColor8 = new BackgroundColor() { Indexed = (UInt32Value)65U };

            patternFill20.Append(foregroundColor18);
            patternFill20.Append(backgroundColor8);

            fill20.Append(patternFill20);

            Fill fill21 = new Fill();

            PatternFill patternFill21 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor19 = new ForegroundColor() { Theme = (UInt32Value)6U, Tint = 0.39997558519241921D };
            BackgroundColor backgroundColor9 = new BackgroundColor() { Indexed = (UInt32Value)65U };

            patternFill21.Append(foregroundColor19);
            patternFill21.Append(backgroundColor9);

            fill21.Append(patternFill21);

            Fill fill22 = new Fill();

            PatternFill patternFill22 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor20 = new ForegroundColor() { Theme = (UInt32Value)7U };

            patternFill22.Append(foregroundColor20);

            fill22.Append(patternFill22);

            Fill fill23 = new Fill();

            PatternFill patternFill23 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor21 = new ForegroundColor() { Theme = (UInt32Value)7U, Tint = 0.79998168889431442D };
            BackgroundColor backgroundColor10 = new BackgroundColor() { Indexed = (UInt32Value)65U };

            patternFill23.Append(foregroundColor21);
            patternFill23.Append(backgroundColor10);

            fill23.Append(patternFill23);

            Fill fill24 = new Fill();

            PatternFill patternFill24 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor22 = new ForegroundColor() { Theme = (UInt32Value)7U, Tint = 0.59999389629810485D };
            BackgroundColor backgroundColor11 = new BackgroundColor() { Indexed = (UInt32Value)65U };

            patternFill24.Append(foregroundColor22);
            patternFill24.Append(backgroundColor11);

            fill24.Append(patternFill24);

            Fill fill25 = new Fill();

            PatternFill patternFill25 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor23 = new ForegroundColor() { Theme = (UInt32Value)7U, Tint = 0.39997558519241921D };
            BackgroundColor backgroundColor12 = new BackgroundColor() { Indexed = (UInt32Value)65U };

            patternFill25.Append(foregroundColor23);
            patternFill25.Append(backgroundColor12);

            fill25.Append(patternFill25);

            Fill fill26 = new Fill();

            PatternFill patternFill26 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor24 = new ForegroundColor() { Theme = (UInt32Value)8U };

            patternFill26.Append(foregroundColor24);

            fill26.Append(patternFill26);

            Fill fill27 = new Fill();

            PatternFill patternFill27 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor25 = new ForegroundColor() { Theme = (UInt32Value)8U, Tint = 0.79998168889431442D };
            BackgroundColor backgroundColor13 = new BackgroundColor() { Indexed = (UInt32Value)65U };

            patternFill27.Append(foregroundColor25);
            patternFill27.Append(backgroundColor13);

            fill27.Append(patternFill27);

            Fill fill28 = new Fill();

            PatternFill patternFill28 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor26 = new ForegroundColor() { Theme = (UInt32Value)8U, Tint = 0.59999389629810485D };
            BackgroundColor backgroundColor14 = new BackgroundColor() { Indexed = (UInt32Value)65U };

            patternFill28.Append(foregroundColor26);
            patternFill28.Append(backgroundColor14);

            fill28.Append(patternFill28);

            Fill fill29 = new Fill();

            PatternFill patternFill29 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor27 = new ForegroundColor() { Theme = (UInt32Value)8U, Tint = 0.39997558519241921D };
            BackgroundColor backgroundColor15 = new BackgroundColor() { Indexed = (UInt32Value)65U };

            patternFill29.Append(foregroundColor27);
            patternFill29.Append(backgroundColor15);

            fill29.Append(patternFill29);

            Fill fill30 = new Fill();

            PatternFill patternFill30 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor28 = new ForegroundColor() { Theme = (UInt32Value)9U };

            patternFill30.Append(foregroundColor28);

            fill30.Append(patternFill30);

            Fill fill31 = new Fill();

            PatternFill patternFill31 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor29 = new ForegroundColor() { Theme = (UInt32Value)9U, Tint = 0.79998168889431442D };
            BackgroundColor backgroundColor16 = new BackgroundColor() { Indexed = (UInt32Value)65U };

            patternFill31.Append(foregroundColor29);
            patternFill31.Append(backgroundColor16);

            fill31.Append(patternFill31);

            Fill fill32 = new Fill();

            PatternFill patternFill32 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor30 = new ForegroundColor() { Theme = (UInt32Value)9U, Tint = 0.59999389629810485D };
            BackgroundColor backgroundColor17 = new BackgroundColor() { Indexed = (UInt32Value)65U };

            patternFill32.Append(foregroundColor30);
            patternFill32.Append(backgroundColor17);

            fill32.Append(patternFill32);

            Fill fill33 = new Fill();

            PatternFill patternFill33 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor31 = new ForegroundColor() { Theme = (UInt32Value)9U, Tint = 0.39997558519241921D };
            BackgroundColor backgroundColor18 = new BackgroundColor() { Indexed = (UInt32Value)65U };

            patternFill33.Append(foregroundColor31);
            patternFill33.Append(backgroundColor18);

            fill33.Append(patternFill33);

            fills1.Append(fill1);
            fills1.Append(fill2);
            fills1.Append(fill3);
            fills1.Append(fill4);
            fills1.Append(fill5);
            fills1.Append(fill6);
            fills1.Append(fill7);
            fills1.Append(fill8);
            fills1.Append(fill9);
            fills1.Append(fill10);
            fills1.Append(fill11);
            fills1.Append(fill12);
            fills1.Append(fill13);
            fills1.Append(fill14);
            fills1.Append(fill15);
            fills1.Append(fill16);
            fills1.Append(fill17);
            fills1.Append(fill18);
            fills1.Append(fill19);
            fills1.Append(fill20);
            fills1.Append(fill21);
            fills1.Append(fill22);
            fills1.Append(fill23);
            fills1.Append(fill24);
            fills1.Append(fill25);
            fills1.Append(fill26);
            fills1.Append(fill27);
            fills1.Append(fill28);
            fills1.Append(fill29);
            fills1.Append(fill30);
            fills1.Append(fill31);
            fills1.Append(fill32);
            fills1.Append(fill33);

            Borders borders1 = new Borders() { Count = (UInt32Value)10U };

            Border border1 = new Border();
            LeftBorder leftBorder1 = new LeftBorder();
            RightBorder rightBorder1 = new RightBorder();
            TopBorder topBorder1 = new TopBorder();
            BottomBorder bottomBorder1 = new BottomBorder();
            DiagonalBorder diagonalBorder1 = new DiagonalBorder();

            border1.Append(leftBorder1);
            border1.Append(rightBorder1);
            border1.Append(topBorder1);
            border1.Append(bottomBorder1);
            border1.Append(diagonalBorder1);

            Border border2 = new Border();
            LeftBorder leftBorder2 = new LeftBorder();
            RightBorder rightBorder2 = new RightBorder();
            TopBorder topBorder2 = new TopBorder();

            BottomBorder bottomBorder2 = new BottomBorder() { Style = BorderStyleValues.Thick };
            Color color19 = new Color() { Theme = (UInt32Value)4U };

            bottomBorder2.Append(color19);
            DiagonalBorder diagonalBorder2 = new DiagonalBorder();

            border2.Append(leftBorder2);
            border2.Append(rightBorder2);
            border2.Append(topBorder2);
            border2.Append(bottomBorder2);
            border2.Append(diagonalBorder2);

            Border border3 = new Border();
            LeftBorder leftBorder3 = new LeftBorder();
            RightBorder rightBorder3 = new RightBorder();
            TopBorder topBorder3 = new TopBorder();

            BottomBorder bottomBorder3 = new BottomBorder() { Style = BorderStyleValues.Thick };
            Color color20 = new Color() { Theme = (UInt32Value)4U, Tint = 0.499984740745262D };

            bottomBorder3.Append(color20);
            DiagonalBorder diagonalBorder3 = new DiagonalBorder();

            border3.Append(leftBorder3);
            border3.Append(rightBorder3);
            border3.Append(topBorder3);
            border3.Append(bottomBorder3);
            border3.Append(diagonalBorder3);

            Border border4 = new Border();
            LeftBorder leftBorder4 = new LeftBorder();
            RightBorder rightBorder4 = new RightBorder();
            TopBorder topBorder4 = new TopBorder();

            BottomBorder bottomBorder4 = new BottomBorder() { Style = BorderStyleValues.Medium };
            Color color21 = new Color() { Theme = (UInt32Value)4U, Tint = 0.39997558519241921D };

            bottomBorder4.Append(color21);
            DiagonalBorder diagonalBorder4 = new DiagonalBorder();

            border4.Append(leftBorder4);
            border4.Append(rightBorder4);
            border4.Append(topBorder4);
            border4.Append(bottomBorder4);
            border4.Append(diagonalBorder4);

            Border border5 = new Border();

            LeftBorder leftBorder5 = new LeftBorder() { Style = BorderStyleValues.Thin };
            Color color22 = new Color() { Rgb = "FF7F7F7F" };

            leftBorder5.Append(color22);

            RightBorder rightBorder5 = new RightBorder() { Style = BorderStyleValues.Thin };
            Color color23 = new Color() { Rgb = "FF7F7F7F" };

            rightBorder5.Append(color23);

            TopBorder topBorder5 = new TopBorder() { Style = BorderStyleValues.Thin };
            Color color24 = new Color() { Rgb = "FF7F7F7F" };

            topBorder5.Append(color24);

            BottomBorder bottomBorder5 = new BottomBorder() { Style = BorderStyleValues.Thin };
            Color color25 = new Color() { Rgb = "FF7F7F7F" };

            bottomBorder5.Append(color25);
            DiagonalBorder diagonalBorder5 = new DiagonalBorder();

            border5.Append(leftBorder5);
            border5.Append(rightBorder5);
            border5.Append(topBorder5);
            border5.Append(bottomBorder5);
            border5.Append(diagonalBorder5);

            Border border6 = new Border();

            LeftBorder leftBorder6 = new LeftBorder() { Style = BorderStyleValues.Thin };
            Color color26 = new Color() { Rgb = "FF3F3F3F" };

            leftBorder6.Append(color26);

            RightBorder rightBorder6 = new RightBorder() { Style = BorderStyleValues.Thin };
            Color color27 = new Color() { Rgb = "FF3F3F3F" };

            rightBorder6.Append(color27);

            TopBorder topBorder6 = new TopBorder() { Style = BorderStyleValues.Thin };
            Color color28 = new Color() { Rgb = "FF3F3F3F" };

            topBorder6.Append(color28);

            BottomBorder bottomBorder6 = new BottomBorder() { Style = BorderStyleValues.Thin };
            Color color29 = new Color() { Rgb = "FF3F3F3F" };

            bottomBorder6.Append(color29);
            DiagonalBorder diagonalBorder6 = new DiagonalBorder();

            border6.Append(leftBorder6);
            border6.Append(rightBorder6);
            border6.Append(topBorder6);
            border6.Append(bottomBorder6);
            border6.Append(diagonalBorder6);

            Border border7 = new Border();
            LeftBorder leftBorder7 = new LeftBorder();
            RightBorder rightBorder7 = new RightBorder();
            TopBorder topBorder7 = new TopBorder();

            BottomBorder bottomBorder7 = new BottomBorder() { Style = BorderStyleValues.Double };
            Color color30 = new Color() { Rgb = "FFFF8001" };

            bottomBorder7.Append(color30);
            DiagonalBorder diagonalBorder7 = new DiagonalBorder();

            border7.Append(leftBorder7);
            border7.Append(rightBorder7);
            border7.Append(topBorder7);
            border7.Append(bottomBorder7);
            border7.Append(diagonalBorder7);

            Border border8 = new Border();

            LeftBorder leftBorder8 = new LeftBorder() { Style = BorderStyleValues.Double };
            Color color31 = new Color() { Rgb = "FF3F3F3F" };

            leftBorder8.Append(color31);

            RightBorder rightBorder8 = new RightBorder() { Style = BorderStyleValues.Double };
            Color color32 = new Color() { Rgb = "FF3F3F3F" };

            rightBorder8.Append(color32);

            TopBorder topBorder8 = new TopBorder() { Style = BorderStyleValues.Double };
            Color color33 = new Color() { Rgb = "FF3F3F3F" };

            topBorder8.Append(color33);

            BottomBorder bottomBorder8 = new BottomBorder() { Style = BorderStyleValues.Double };
            Color color34 = new Color() { Rgb = "FF3F3F3F" };

            bottomBorder8.Append(color34);
            DiagonalBorder diagonalBorder8 = new DiagonalBorder();

            border8.Append(leftBorder8);
            border8.Append(rightBorder8);
            border8.Append(topBorder8);
            border8.Append(bottomBorder8);
            border8.Append(diagonalBorder8);

            Border border9 = new Border();

            LeftBorder leftBorder9 = new LeftBorder() { Style = BorderStyleValues.Thin };
            Color color35 = new Color() { Rgb = "FFB2B2B2" };

            leftBorder9.Append(color35);

            RightBorder rightBorder9 = new RightBorder() { Style = BorderStyleValues.Thin };
            Color color36 = new Color() { Rgb = "FFB2B2B2" };

            rightBorder9.Append(color36);

            TopBorder topBorder9 = new TopBorder() { Style = BorderStyleValues.Thin };
            Color color37 = new Color() { Rgb = "FFB2B2B2" };

            topBorder9.Append(color37);

            BottomBorder bottomBorder9 = new BottomBorder() { Style = BorderStyleValues.Thin };
            Color color38 = new Color() { Rgb = "FFB2B2B2" };

            bottomBorder9.Append(color38);
            DiagonalBorder diagonalBorder9 = new DiagonalBorder();

            border9.Append(leftBorder9);
            border9.Append(rightBorder9);
            border9.Append(topBorder9);
            border9.Append(bottomBorder9);
            border9.Append(diagonalBorder9);

            Border border10 = new Border();
            LeftBorder leftBorder10 = new LeftBorder();
            RightBorder rightBorder10 = new RightBorder();

            TopBorder topBorder10 = new TopBorder() { Style = BorderStyleValues.Thin };
            Color color39 = new Color() { Theme = (UInt32Value)4U };

            topBorder10.Append(color39);

            BottomBorder bottomBorder10 = new BottomBorder() { Style = BorderStyleValues.Double };
            Color color40 = new Color() { Theme = (UInt32Value)4U };

            bottomBorder10.Append(color40);
            DiagonalBorder diagonalBorder10 = new DiagonalBorder();

            border10.Append(leftBorder10);
            border10.Append(rightBorder10);
            border10.Append(topBorder10);
            border10.Append(bottomBorder10);
            border10.Append(diagonalBorder10);

            borders1.Append(border1);
            borders1.Append(border2);
            borders1.Append(border3);
            borders1.Append(border4);
            borders1.Append(border5);
            borders1.Append(border6);
            borders1.Append(border7);
            borders1.Append(border8);
            borders1.Append(border9);
            borders1.Append(border10);

            CellStyleFormats cellStyleFormats1 = new CellStyleFormats() { Count = (UInt32Value)42U };
            CellFormat cellFormat1 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)0U, FillId = (UInt32Value)0U, BorderId = (UInt32Value)0U };
            CellFormat cellFormat2 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)2U, FillId = (UInt32Value)0U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyFill = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat3 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)3U, FillId = (UInt32Value)0U, BorderId = (UInt32Value)1U, ApplyNumberFormat = false, ApplyFill = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat4 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)4U, FillId = (UInt32Value)0U, BorderId = (UInt32Value)2U, ApplyNumberFormat = false, ApplyFill = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat5 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)5U, FillId = (UInt32Value)0U, BorderId = (UInt32Value)3U, ApplyNumberFormat = false, ApplyFill = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat6 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)5U, FillId = (UInt32Value)0U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyFill = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat7 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)6U, FillId = (UInt32Value)2U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat8 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)7U, FillId = (UInt32Value)3U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat9 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)8U, FillId = (UInt32Value)4U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat10 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)9U, FillId = (UInt32Value)5U, BorderId = (UInt32Value)4U, ApplyNumberFormat = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat11 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)10U, FillId = (UInt32Value)6U, BorderId = (UInt32Value)5U, ApplyNumberFormat = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat12 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)11U, FillId = (UInt32Value)6U, BorderId = (UInt32Value)4U, ApplyNumberFormat = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat13 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)12U, FillId = (UInt32Value)0U, BorderId = (UInt32Value)6U, ApplyNumberFormat = false, ApplyFill = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat14 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)13U, FillId = (UInt32Value)7U, BorderId = (UInt32Value)7U, ApplyNumberFormat = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat15 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)14U, FillId = (UInt32Value)0U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyFill = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat16 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)1U, FillId = (UInt32Value)8U, BorderId = (UInt32Value)8U, ApplyNumberFormat = false, ApplyFont = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat17 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)15U, FillId = (UInt32Value)0U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyFill = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat18 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)16U, FillId = (UInt32Value)0U, BorderId = (UInt32Value)9U, ApplyNumberFormat = false, ApplyFill = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat19 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)17U, FillId = (UInt32Value)9U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat20 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)1U, FillId = (UInt32Value)10U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat21 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)1U, FillId = (UInt32Value)11U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat22 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)17U, FillId = (UInt32Value)12U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat23 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)17U, FillId = (UInt32Value)13U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat24 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)1U, FillId = (UInt32Value)14U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat25 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)1U, FillId = (UInt32Value)15U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat26 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)17U, FillId = (UInt32Value)16U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat27 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)17U, FillId = (UInt32Value)17U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat28 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)1U, FillId = (UInt32Value)18U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat29 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)1U, FillId = (UInt32Value)19U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat30 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)17U, FillId = (UInt32Value)20U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat31 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)17U, FillId = (UInt32Value)21U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat32 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)1U, FillId = (UInt32Value)22U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat33 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)1U, FillId = (UInt32Value)23U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat34 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)17U, FillId = (UInt32Value)24U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat35 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)17U, FillId = (UInt32Value)25U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat36 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)1U, FillId = (UInt32Value)26U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat37 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)1U, FillId = (UInt32Value)27U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat38 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)17U, FillId = (UInt32Value)28U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat39 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)17U, FillId = (UInt32Value)29U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat40 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)1U, FillId = (UInt32Value)30U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat41 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)1U, FillId = (UInt32Value)31U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };
            CellFormat cellFormat42 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)17U, FillId = (UInt32Value)32U, BorderId = (UInt32Value)0U, ApplyNumberFormat = false, ApplyBorder = false, ApplyAlignment = false, ApplyProtection = false };

            cellStyleFormats1.Append(cellFormat1);
            cellStyleFormats1.Append(cellFormat2);
            cellStyleFormats1.Append(cellFormat3);
            cellStyleFormats1.Append(cellFormat4);
            cellStyleFormats1.Append(cellFormat5);
            cellStyleFormats1.Append(cellFormat6);
            cellStyleFormats1.Append(cellFormat7);
            cellStyleFormats1.Append(cellFormat8);
            cellStyleFormats1.Append(cellFormat9);
            cellStyleFormats1.Append(cellFormat10);
            cellStyleFormats1.Append(cellFormat11);
            cellStyleFormats1.Append(cellFormat12);
            cellStyleFormats1.Append(cellFormat13);
            cellStyleFormats1.Append(cellFormat14);
            cellStyleFormats1.Append(cellFormat15);
            cellStyleFormats1.Append(cellFormat16);
            cellStyleFormats1.Append(cellFormat17);
            cellStyleFormats1.Append(cellFormat18);
            cellStyleFormats1.Append(cellFormat19);
            cellStyleFormats1.Append(cellFormat20);
            cellStyleFormats1.Append(cellFormat21);
            cellStyleFormats1.Append(cellFormat22);
            cellStyleFormats1.Append(cellFormat23);
            cellStyleFormats1.Append(cellFormat24);
            cellStyleFormats1.Append(cellFormat25);
            cellStyleFormats1.Append(cellFormat26);
            cellStyleFormats1.Append(cellFormat27);
            cellStyleFormats1.Append(cellFormat28);
            cellStyleFormats1.Append(cellFormat29);
            cellStyleFormats1.Append(cellFormat30);
            cellStyleFormats1.Append(cellFormat31);
            cellStyleFormats1.Append(cellFormat32);
            cellStyleFormats1.Append(cellFormat33);
            cellStyleFormats1.Append(cellFormat34);
            cellStyleFormats1.Append(cellFormat35);
            cellStyleFormats1.Append(cellFormat36);
            cellStyleFormats1.Append(cellFormat37);
            cellStyleFormats1.Append(cellFormat38);
            cellStyleFormats1.Append(cellFormat39);
            cellStyleFormats1.Append(cellFormat40);
            cellStyleFormats1.Append(cellFormat41);
            cellStyleFormats1.Append(cellFormat42);

            CellFormats cellFormats1 = new CellFormats() { Count = (UInt32Value)4U };
            CellFormat cellFormat43 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)0U, FillId = (UInt32Value)0U, BorderId = (UInt32Value)0U, FormatId = (UInt32Value)0U };
            CellFormat cellFormat44 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)0U, FillId = (UInt32Value)0U, BorderId = (UInt32Value)0U, FormatId = (UInt32Value)0U, PivotButton = true };

            CellFormat cellFormat45 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)0U, FillId = (UInt32Value)0U, BorderId = (UInt32Value)0U, FormatId = (UInt32Value)0U, ApplyAlignment = true };
            Alignment alignment1 = new Alignment() { Horizontal = HorizontalAlignmentValues.Left };

            cellFormat45.Append(alignment1);
            CellFormat cellFormat46 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)0U, FillId = (UInt32Value)0U, BorderId = (UInt32Value)0U, FormatId = (UInt32Value)0U, ApplyNumberFormat = true };

            cellFormats1.Append(cellFormat43);
            cellFormats1.Append(cellFormat44);
            cellFormats1.Append(cellFormat45);
            cellFormats1.Append(cellFormat46);

            CellStyles cellStyles1 = new CellStyles() { Count = (UInt32Value)42U };
            CellStyle cellStyle1 = new CellStyle() { Name = "20% - Accent1", FormatId = (UInt32Value)19U, BuiltinId = (UInt32Value)30U, CustomBuiltin = true };
            CellStyle cellStyle2 = new CellStyle() { Name = "20% - Accent2", FormatId = (UInt32Value)23U, BuiltinId = (UInt32Value)34U, CustomBuiltin = true };
            CellStyle cellStyle3 = new CellStyle() { Name = "20% - Accent3", FormatId = (UInt32Value)27U, BuiltinId = (UInt32Value)38U, CustomBuiltin = true };
            CellStyle cellStyle4 = new CellStyle() { Name = "20% - Accent4", FormatId = (UInt32Value)31U, BuiltinId = (UInt32Value)42U, CustomBuiltin = true };
            CellStyle cellStyle5 = new CellStyle() { Name = "20% - Accent5", FormatId = (UInt32Value)35U, BuiltinId = (UInt32Value)46U, CustomBuiltin = true };
            CellStyle cellStyle6 = new CellStyle() { Name = "20% - Accent6", FormatId = (UInt32Value)39U, BuiltinId = (UInt32Value)50U, CustomBuiltin = true };
            CellStyle cellStyle7 = new CellStyle() { Name = "40% - Accent1", FormatId = (UInt32Value)20U, BuiltinId = (UInt32Value)31U, CustomBuiltin = true };
            CellStyle cellStyle8 = new CellStyle() { Name = "40% - Accent2", FormatId = (UInt32Value)24U, BuiltinId = (UInt32Value)35U, CustomBuiltin = true };
            CellStyle cellStyle9 = new CellStyle() { Name = "40% - Accent3", FormatId = (UInt32Value)28U, BuiltinId = (UInt32Value)39U, CustomBuiltin = true };
            CellStyle cellStyle10 = new CellStyle() { Name = "40% - Accent4", FormatId = (UInt32Value)32U, BuiltinId = (UInt32Value)43U, CustomBuiltin = true };
            CellStyle cellStyle11 = new CellStyle() { Name = "40% - Accent5", FormatId = (UInt32Value)36U, BuiltinId = (UInt32Value)47U, CustomBuiltin = true };
            CellStyle cellStyle12 = new CellStyle() { Name = "40% - Accent6", FormatId = (UInt32Value)40U, BuiltinId = (UInt32Value)51U, CustomBuiltin = true };
            CellStyle cellStyle13 = new CellStyle() { Name = "60% - Accent1", FormatId = (UInt32Value)21U, BuiltinId = (UInt32Value)32U, CustomBuiltin = true };
            CellStyle cellStyle14 = new CellStyle() { Name = "60% - Accent2", FormatId = (UInt32Value)25U, BuiltinId = (UInt32Value)36U, CustomBuiltin = true };
            CellStyle cellStyle15 = new CellStyle() { Name = "60% - Accent3", FormatId = (UInt32Value)29U, BuiltinId = (UInt32Value)40U, CustomBuiltin = true };
            CellStyle cellStyle16 = new CellStyle() { Name = "60% - Accent4", FormatId = (UInt32Value)33U, BuiltinId = (UInt32Value)44U, CustomBuiltin = true };
            CellStyle cellStyle17 = new CellStyle() { Name = "60% - Accent5", FormatId = (UInt32Value)37U, BuiltinId = (UInt32Value)48U, CustomBuiltin = true };
            CellStyle cellStyle18 = new CellStyle() { Name = "60% - Accent6", FormatId = (UInt32Value)41U, BuiltinId = (UInt32Value)52U, CustomBuiltin = true };
            CellStyle cellStyle19 = new CellStyle() { Name = "Accent1", FormatId = (UInt32Value)18U, BuiltinId = (UInt32Value)29U, CustomBuiltin = true };
            CellStyle cellStyle20 = new CellStyle() { Name = "Accent2", FormatId = (UInt32Value)22U, BuiltinId = (UInt32Value)33U, CustomBuiltin = true };
            CellStyle cellStyle21 = new CellStyle() { Name = "Accent3", FormatId = (UInt32Value)26U, BuiltinId = (UInt32Value)37U, CustomBuiltin = true };
            CellStyle cellStyle22 = new CellStyle() { Name = "Accent4", FormatId = (UInt32Value)30U, BuiltinId = (UInt32Value)41U, CustomBuiltin = true };
            CellStyle cellStyle23 = new CellStyle() { Name = "Accent5", FormatId = (UInt32Value)34U, BuiltinId = (UInt32Value)45U, CustomBuiltin = true };
            CellStyle cellStyle24 = new CellStyle() { Name = "Accent6", FormatId = (UInt32Value)38U, BuiltinId = (UInt32Value)49U, CustomBuiltin = true };
            CellStyle cellStyle25 = new CellStyle() { Name = "Bad", FormatId = (UInt32Value)7U, BuiltinId = (UInt32Value)27U, CustomBuiltin = true };
            CellStyle cellStyle26 = new CellStyle() { Name = "Calculation", FormatId = (UInt32Value)11U, BuiltinId = (UInt32Value)22U, CustomBuiltin = true };
            CellStyle cellStyle27 = new CellStyle() { Name = "Check Cell", FormatId = (UInt32Value)13U, BuiltinId = (UInt32Value)23U, CustomBuiltin = true };
            CellStyle cellStyle28 = new CellStyle() { Name = "Explanatory Text", FormatId = (UInt32Value)16U, BuiltinId = (UInt32Value)53U, CustomBuiltin = true };
            CellStyle cellStyle29 = new CellStyle() { Name = "Good", FormatId = (UInt32Value)6U, BuiltinId = (UInt32Value)26U, CustomBuiltin = true };
            CellStyle cellStyle30 = new CellStyle() { Name = "Heading 1", FormatId = (UInt32Value)2U, BuiltinId = (UInt32Value)16U, CustomBuiltin = true };
            CellStyle cellStyle31 = new CellStyle() { Name = "Heading 2", FormatId = (UInt32Value)3U, BuiltinId = (UInt32Value)17U, CustomBuiltin = true };
            CellStyle cellStyle32 = new CellStyle() { Name = "Heading 3", FormatId = (UInt32Value)4U, BuiltinId = (UInt32Value)18U, CustomBuiltin = true };
            CellStyle cellStyle33 = new CellStyle() { Name = "Heading 4", FormatId = (UInt32Value)5U, BuiltinId = (UInt32Value)19U, CustomBuiltin = true };
            CellStyle cellStyle34 = new CellStyle() { Name = "Input", FormatId = (UInt32Value)9U, BuiltinId = (UInt32Value)20U, CustomBuiltin = true };
            CellStyle cellStyle35 = new CellStyle() { Name = "Linked Cell", FormatId = (UInt32Value)12U, BuiltinId = (UInt32Value)24U, CustomBuiltin = true };
            CellStyle cellStyle36 = new CellStyle() { Name = "Neutral", FormatId = (UInt32Value)8U, BuiltinId = (UInt32Value)28U, CustomBuiltin = true };
            CellStyle cellStyle37 = new CellStyle() { Name = "Normal", FormatId = (UInt32Value)0U, BuiltinId = (UInt32Value)0U };
            CellStyle cellStyle38 = new CellStyle() { Name = "Note", FormatId = (UInt32Value)15U, BuiltinId = (UInt32Value)10U, CustomBuiltin = true };
            CellStyle cellStyle39 = new CellStyle() { Name = "Output", FormatId = (UInt32Value)10U, BuiltinId = (UInt32Value)21U, CustomBuiltin = true };
            CellStyle cellStyle40 = new CellStyle() { Name = "Title", FormatId = (UInt32Value)1U, BuiltinId = (UInt32Value)15U, CustomBuiltin = true };
            CellStyle cellStyle41 = new CellStyle() { Name = "Total", FormatId = (UInt32Value)17U, BuiltinId = (UInt32Value)25U, CustomBuiltin = true };
            CellStyle cellStyle42 = new CellStyle() { Name = "Warning Text", FormatId = (UInt32Value)14U, BuiltinId = (UInt32Value)11U, CustomBuiltin = true };

            cellStyles1.Append(cellStyle1);
            cellStyles1.Append(cellStyle2);
            cellStyles1.Append(cellStyle3);
            cellStyles1.Append(cellStyle4);
            cellStyles1.Append(cellStyle5);
            cellStyles1.Append(cellStyle6);
            cellStyles1.Append(cellStyle7);
            cellStyles1.Append(cellStyle8);
            cellStyles1.Append(cellStyle9);
            cellStyles1.Append(cellStyle10);
            cellStyles1.Append(cellStyle11);
            cellStyles1.Append(cellStyle12);
            cellStyles1.Append(cellStyle13);
            cellStyles1.Append(cellStyle14);
            cellStyles1.Append(cellStyle15);
            cellStyles1.Append(cellStyle16);
            cellStyles1.Append(cellStyle17);
            cellStyles1.Append(cellStyle18);
            cellStyles1.Append(cellStyle19);
            cellStyles1.Append(cellStyle20);
            cellStyles1.Append(cellStyle21);
            cellStyles1.Append(cellStyle22);
            cellStyles1.Append(cellStyle23);
            cellStyles1.Append(cellStyle24);
            cellStyles1.Append(cellStyle25);
            cellStyles1.Append(cellStyle26);
            cellStyles1.Append(cellStyle27);
            cellStyles1.Append(cellStyle28);
            cellStyles1.Append(cellStyle29);
            cellStyles1.Append(cellStyle30);
            cellStyles1.Append(cellStyle31);
            cellStyles1.Append(cellStyle32);
            cellStyles1.Append(cellStyle33);
            cellStyles1.Append(cellStyle34);
            cellStyles1.Append(cellStyle35);
            cellStyles1.Append(cellStyle36);
            cellStyles1.Append(cellStyle37);
            cellStyles1.Append(cellStyle38);
            cellStyles1.Append(cellStyle39);
            cellStyles1.Append(cellStyle40);
            cellStyles1.Append(cellStyle41);
            cellStyles1.Append(cellStyle42);
            DifferentialFormats differentialFormats1 = new DifferentialFormats() { Count = (UInt32Value)0U };
            TableStyles tableStyles1 = new TableStyles() { Count = (UInt32Value)0U, DefaultTableStyle = "TableStyleMedium2", DefaultPivotStyle = "PivotStyleLight16" };

            StylesheetExtensionList stylesheetExtensionList1 = new StylesheetExtensionList();

            StylesheetExtension stylesheetExtension1 = new StylesheetExtension() { Uri = "{EB79DEF2-80B8-43e5-95BD-54CBDDF9020C}" };
            stylesheetExtension1.AddNamespaceDeclaration("x14", "http://schemas.microsoft.com/office/spreadsheetml/2009/9/main");
            X14.SlicerStyles slicerStyles1 = new X14.SlicerStyles() { DefaultSlicerStyle = "SlicerStyleLight1" };

            stylesheetExtension1.Append(slicerStyles1);

            stylesheetExtensionList1.Append(stylesheetExtension1);

            stylesheet1.Append(fonts1);
            stylesheet1.Append(fills1);
            stylesheet1.Append(borders1);
            stylesheet1.Append(cellStyleFormats1);
            stylesheet1.Append(cellFormats1);
            stylesheet1.Append(cellStyles1);
            stylesheet1.Append(differentialFormats1);
            stylesheet1.Append(tableStyles1);
            stylesheet1.Append(stylesheetExtensionList1);

            workbookStylesPart1.Stylesheet = stylesheet1;
        }

        // Generates content of worksheetPart1.
        private void GenerateWorksheetPart1Content(WorksheetPart worksheetPart1)
        {
            Worksheet worksheet1 = new Worksheet() { MCAttributes = new MarkupCompatibilityAttributes() { Ignorable = "x14ac" } };
            worksheet1.AddNamespaceDeclaration("r", "http://schemas.openxmlformats.org/officeDocument/2006/relationships");
            worksheet1.AddNamespaceDeclaration("mc", "http://schemas.openxmlformats.org/markup-compatibility/2006");
            worksheet1.AddNamespaceDeclaration("x14ac", "http://schemas.microsoft.com/office/spreadsheetml/2009/9/ac");
            SheetDimension sheetDimension1 = new SheetDimension() { Reference = "A1:C24" };

            SheetViews sheetViews1 = new SheetViews();

            SheetView sheetView1 = new SheetView() { WorkbookViewId = (UInt32Value)0U };
            Selection selection1 = new Selection() { ActiveCell = "E23", SequenceOfReferences = new ListValue<StringValue>() { InnerText = "E23" } };

            sheetView1.Append(selection1);

            sheetViews1.Append(sheetView1);
            SheetFormatProperties sheetFormatProperties1 = new SheetFormatProperties() { DefaultRowHeight = 15D, DyDescent = 0.25D };

            Columns columns1 = new Columns();
            Column column1 = new Column() { Min = (UInt32Value)1U, Max = (UInt32Value)1U, Width = 27.28515625D, BestFit = true, CustomWidth = true };
            Column column2 = new Column() { Min = (UInt32Value)2U, Max = (UInt32Value)3U, Width = 19.5703125D, BestFit = true, CustomWidth = true };

            columns1.Append(column1);
            columns1.Append(column2);

            SheetData sheetData1 = new SheetData();

            Row row1 = new Row() { RowIndex = (UInt32Value)1U, Spans = new ListValue<StringValue>() { InnerText = "1:3" }, DyDescent = 0.25D };

            Cell cell1 = new Cell() { CellReference = "A1", StyleIndex = (UInt32Value)1U, DataType = CellValues.SharedString };
            CellValue cellValue1 = new CellValue();
            cellValue1.Text = "1";

            cell1.Append(cellValue1);

            Cell cell2 = new Cell() { CellReference = "B1", DataType = CellValues.SharedString };
            CellValue cellValue2 = new CellValue();
            cellValue2.Text = "61";

            cell2.Append(cellValue2);

            row1.Append(cell1);
            row1.Append(cell2);

            sheetData1.Append(row1);
            PageMargins pageMargins1 = new PageMargins() { Left = 0.7D, Right = 0.7D, Top = 0.75D, Bottom = 0.75D, Header = 0.3D, Footer = 0.3D };
            Drawing drawing1 = new Drawing() { Id = "rId2" };

            worksheet1.Append(sheetDimension1);
            worksheet1.Append(sheetViews1);
            worksheet1.Append(sheetFormatProperties1);
            worksheet1.Append(columns1);
            worksheet1.Append(sheetData1);
            worksheet1.Append(pageMargins1);
            worksheet1.Append(drawing1);

            worksheetPart1.Worksheet = worksheet1;
        }

        // Generates content of drawingsPart1.
        private void GenerateDrawingsPart1Content(DrawingsPart drawingsPart1)
        {
            Xdr.WorksheetDrawing worksheetDrawing1 = new Xdr.WorksheetDrawing();
            worksheetDrawing1.AddNamespaceDeclaration("xdr", "http://schemas.openxmlformats.org/drawingml/2006/spreadsheetDrawing");
            worksheetDrawing1.AddNamespaceDeclaration("a", "http://schemas.openxmlformats.org/drawingml/2006/main");

            Xdr.TwoCellAnchor twoCellAnchor1 = new Xdr.TwoCellAnchor();

            Xdr.FromMarker fromMarker1 = new Xdr.FromMarker();
            Xdr.ColumnId columnId1 = new Xdr.ColumnId();
            columnId1.Text = "0";
            Xdr.ColumnOffset columnOffset1 = new Xdr.ColumnOffset();
            columnOffset1.Text = "4761";
            Xdr.RowId rowId1 = new Xdr.RowId();
            rowId1.Text = "26";
            Xdr.RowOffset rowOffset1 = new Xdr.RowOffset();
            rowOffset1.Text = "9525";

            fromMarker1.Append(columnId1);
            fromMarker1.Append(columnOffset1);
            fromMarker1.Append(rowId1);
            fromMarker1.Append(rowOffset1);

            Xdr.ToMarker toMarker1 = new Xdr.ToMarker();
            Xdr.ColumnId columnId2 = new Xdr.ColumnId();
            columnId2.Text = "6";
            Xdr.ColumnOffset columnOffset2 = new Xdr.ColumnOffset();
            columnOffset2.Text = "609599";
            Xdr.RowId rowId2 = new Xdr.RowId();
            rowId2.Text = "52";
            Xdr.RowOffset rowOffset2 = new Xdr.RowOffset();
            rowOffset2.Text = "161925";

            toMarker1.Append(columnId2);
            toMarker1.Append(columnOffset2);
            toMarker1.Append(rowId2);
            toMarker1.Append(rowOffset2);

            Xdr.GraphicFrame graphicFrame1 = new Xdr.GraphicFrame() { Macro = "" };

            Xdr.NonVisualGraphicFrameProperties nonVisualGraphicFrameProperties1 = new Xdr.NonVisualGraphicFrameProperties();
            Xdr.NonVisualDrawingProperties nonVisualDrawingProperties1 = new Xdr.NonVisualDrawingProperties() { Id = (UInt32Value)2U, Name = "Chart 1" };
            Xdr.NonVisualGraphicFrameDrawingProperties nonVisualGraphicFrameDrawingProperties1 = new Xdr.NonVisualGraphicFrameDrawingProperties();

            nonVisualGraphicFrameProperties1.Append(nonVisualDrawingProperties1);
            nonVisualGraphicFrameProperties1.Append(nonVisualGraphicFrameDrawingProperties1);

            Xdr.Transform transform1 = new Xdr.Transform();
            A.Offset offset1 = new A.Offset() { X = 0L, Y = 0L };
            A.Extents extents1 = new A.Extents() { Cx = 0L, Cy = 0L };

            transform1.Append(offset1);
            transform1.Append(extents1);

            A.Graphic graphic1 = new A.Graphic();

            A.GraphicData graphicData1 = new A.GraphicData() { Uri = "http://schemas.openxmlformats.org/drawingml/2006/chart" };

            C.ChartReference chartReference1 = new C.ChartReference() { Id = "rId1" };
            chartReference1.AddNamespaceDeclaration("c", "http://schemas.openxmlformats.org/drawingml/2006/chart");
            chartReference1.AddNamespaceDeclaration("r", "http://schemas.openxmlformats.org/officeDocument/2006/relationships");

            graphicData1.Append(chartReference1);

            graphic1.Append(graphicData1);

            graphicFrame1.Append(nonVisualGraphicFrameProperties1);
            graphicFrame1.Append(transform1);
            graphicFrame1.Append(graphic1);
            Xdr.ClientData clientData1 = new Xdr.ClientData();

            twoCellAnchor1.Append(fromMarker1);
            twoCellAnchor1.Append(toMarker1);
            twoCellAnchor1.Append(graphicFrame1);
            twoCellAnchor1.Append(clientData1);

            worksheetDrawing1.Append(twoCellAnchor1);

            drawingsPart1.WorksheetDrawing = worksheetDrawing1;
        }

        // Generates content of chartPart1.
        private void GenerateChartPart1Content(ChartPart chartPart1)
        {
            C.ChartSpace chartSpace1 = new C.ChartSpace();
            chartSpace1.AddNamespaceDeclaration("c", "http://schemas.openxmlformats.org/drawingml/2006/chart");
            chartSpace1.AddNamespaceDeclaration("a", "http://schemas.openxmlformats.org/drawingml/2006/main");
            chartSpace1.AddNamespaceDeclaration("r", "http://schemas.openxmlformats.org/officeDocument/2006/relationships");
            C.Date1904 date19041 = new C.Date1904() { Val = false };
            C.EditingLanguage editingLanguage1 = new C.EditingLanguage() { Val = "en-US" };
            C.RoundedCorners roundedCorners1 = new C.RoundedCorners() { Val = false };

            AlternateContent alternateContent1 = new AlternateContent();
            alternateContent1.AddNamespaceDeclaration("mc", "http://schemas.openxmlformats.org/markup-compatibility/2006");

            AlternateContentChoice alternateContentChoice1 = new AlternateContentChoice() { Requires = "c14" };
            alternateContentChoice1.AddNamespaceDeclaration("c14", "http://schemas.microsoft.com/office/drawing/2007/8/2/chart");
            C14.Style style1 = new C14.Style() { Val = 102 };

            alternateContentChoice1.Append(style1);

            AlternateContentFallback alternateContentFallback1 = new AlternateContentFallback();
            C.Style style2 = new C.Style() { Val = 2 };

            alternateContentFallback1.Append(style2);

            alternateContent1.Append(alternateContentChoice1);
            alternateContent1.Append(alternateContentFallback1);

            C.PivotSource pivotSource1 = new C.PivotSource();
            C.PivotTableName pivotTableName1 = new C.PivotTableName();
            pivotTableName1.Text = "需求工作量按工时分析!PivotTable4";
            C.FormatId formatId1 = new C.FormatId() { Val = (UInt32Value)0U };

            pivotSource1.Append(pivotTableName1);
            pivotSource1.Append(formatId1);

            C.Chart chart1 = new C.Chart();

            C.Title title1 = new C.Title();

            C.ChartText chartText1 = new C.ChartText();

            C.RichText richText1 = new C.RichText();
            A.BodyProperties bodyProperties1 = new A.BodyProperties();
            A.ListStyle listStyle1 = new A.ListStyle();

            A.Paragraph paragraph1 = new A.Paragraph();

            A.ParagraphProperties paragraphProperties1 = new A.ParagraphProperties();
            A.DefaultRunProperties defaultRunProperties1 = new A.DefaultRunProperties();

            paragraphProperties1.Append(defaultRunProperties1);

            A.Run run1 = new A.Run();
            A.RunProperties runProperties1 = new A.RunProperties() { Language = "zh-CN", AlternativeLanguage = "en-US" };
            A.Text text1 = new A.Text();
            text1.Text = "需求工时分析";

            run1.Append(runProperties1);
            run1.Append(text1);
            A.EndParagraphRunProperties endParagraphRunProperties1 = new A.EndParagraphRunProperties() { Language = "en-US" };

            paragraph1.Append(paragraphProperties1);
            paragraph1.Append(run1);
            paragraph1.Append(endParagraphRunProperties1);

            richText1.Append(bodyProperties1);
            richText1.Append(listStyle1);
            richText1.Append(paragraph1);

            chartText1.Append(richText1);
            C.Layout layout1 = new C.Layout();
            C.Overlay overlay1 = new C.Overlay() { Val = false };

            title1.Append(chartText1);
            title1.Append(layout1);
            title1.Append(overlay1);
            C.AutoTitleDeleted autoTitleDeleted1 = new C.AutoTitleDeleted() { Val = false };

            C.PivotFormats pivotFormats1 = new C.PivotFormats();

            C.PivotFormat pivotFormat1 = new C.PivotFormat();
            C.Index index1 = new C.Index() { Val = (UInt32Value)0U };

            C.Marker marker1 = new C.Marker();
            C.Symbol symbol1 = new C.Symbol() { Val = C.MarkerStyleValues.None };

            marker1.Append(symbol1);

            pivotFormat1.Append(index1);
            pivotFormat1.Append(marker1);

            C.PivotFormat pivotFormat2 = new C.PivotFormat();
            C.Index index2 = new C.Index() { Val = (UInt32Value)1U };

            C.Marker marker2 = new C.Marker();
            C.Symbol symbol2 = new C.Symbol() { Val = C.MarkerStyleValues.None };

            marker2.Append(symbol2);

            pivotFormat2.Append(index2);
            pivotFormat2.Append(marker2);

            pivotFormats1.Append(pivotFormat1);
            pivotFormats1.Append(pivotFormat2);

            C.PlotArea plotArea1 = new C.PlotArea();
            C.Layout layout2 = new C.Layout();

            C.BarChart barChart1 = new C.BarChart();
            C.BarDirection barDirection1 = new C.BarDirection() { Val = C.BarDirectionValues.Column };
            C.BarGrouping barGrouping1 = new C.BarGrouping() { Val = C.BarGroupingValues.Stacked };
            C.VaryColors varyColors1 = new C.VaryColors() { Val = false };

            C.BarChartSeries barChartSeries1 = new C.BarChartSeries();
            C.Index index3 = new C.Index() { Val = (UInt32Value)0U };
            C.Order order1 = new C.Order() { Val = (UInt32Value)0U };

            C.SeriesText seriesText1 = new C.SeriesText();

            C.StringReference stringReference1 = new C.StringReference();
            C.Formula formula1 = new C.Formula();
            formula1.Text = "需求工作量按工时分析!$B$3";

            C.StringCache stringCache1 = new C.StringCache();
            C.PointCount pointCount1 = new C.PointCount() { Val = (UInt32Value)1U };

            C.StringPoint stringPoint1 = new C.StringPoint() { Index = (UInt32Value)0U };
            C.NumericValue numericValue1 = new C.NumericValue();
            numericValue1.Text = "已完成的工时总计";

            stringPoint1.Append(numericValue1);

            stringCache1.Append(pointCount1);
            stringCache1.Append(stringPoint1);

            stringReference1.Append(formula1);
            stringReference1.Append(stringCache1);

            seriesText1.Append(stringReference1);
            C.InvertIfNegative invertIfNegative1 = new C.InvertIfNegative() { Val = false };

            C.CategoryAxisData categoryAxisData1 = new C.CategoryAxisData();

            C.StringReference stringReference2 = new C.StringReference();
            C.Formula formula2 = new C.Formula();
            formula2.Text = "需求工作量按工时分析!$A$4:$A$24";

            C.StringCache stringCache2 = new C.StringCache();
            C.PointCount pointCount2 = new C.PointCount() { Val = (UInt32Value)20U };

            C.StringPoint stringPoint2 = new C.StringPoint() { Index = (UInt32Value)0U };
            C.NumericValue numericValue2 = new C.NumericValue();
            numericValue2.Text = "需求管理基本功能";

            stringPoint2.Append(numericValue2);

            C.StringPoint stringPoint3 = new C.StringPoint() { Index = (UInt32Value)1U };
            C.NumericValue numericValue3 = new C.NumericValue();
            numericValue3.Text = "整合配置管理系统";

            stringPoint3.Append(numericValue3);

            C.StringPoint stringPoint4 = new C.StringPoint() { Index = (UInt32Value)2U };
            C.NumericValue numericValue4 = new C.NumericValue();
            numericValue4.Text = "集成团队管理";

            stringPoint4.Append(numericValue4);

            C.StringPoint stringPoint5 = new C.StringPoint() { Index = (UInt32Value)3U };
            C.NumericValue numericValue5 = new C.NumericValue();
            numericValue5.Text = "需求管理中的进度跟踪";

            stringPoint5.Append(numericValue5);

            C.StringPoint stringPoint6 = new C.StringPoint() { Index = (UInt32Value)4U };
            C.NumericValue numericValue6 = new C.NumericValue();
            numericValue6.Text = "风险管理";

            stringPoint6.Append(numericValue6);

            C.StringPoint stringPoint7 = new C.StringPoint() { Index = (UInt32Value)5U };
            C.NumericValue numericValue7 = new C.NumericValue();
            numericValue7.Text = "全文检索";

            stringPoint7.Append(numericValue7);

            C.StringPoint stringPoint8 = new C.StringPoint() { Index = (UInt32Value)6U };
            C.NumericValue numericValue8 = new C.NumericValue();
            numericValue8.Text = "决策过程跟踪";

            stringPoint8.Append(numericValue8);

            C.StringPoint stringPoint9 = new C.StringPoint() { Index = (UInt32Value)7U };
            C.NumericValue numericValue9 = new C.NumericValue();
            numericValue9.Text = "统计功能";

            stringPoint9.Append(numericValue9);

            C.StringPoint stringPoint10 = new C.StringPoint() { Index = (UInt32Value)8U };
            C.NumericValue numericValue10 = new C.NumericValue();
            numericValue10.Text = "用户身份验证";

            stringPoint10.Append(numericValue10);

            C.StringPoint stringPoint11 = new C.StringPoint() { Index = (UInt32Value)9U };
            C.NumericValue numericValue11 = new C.NumericValue();
            numericValue11.Text = "任务标签";

            stringPoint11.Append(numericValue11);

            C.StringPoint stringPoint12 = new C.StringPoint() { Index = (UInt32Value)10U };
            C.NumericValue numericValue12 = new C.NumericValue();
            numericValue12.Text = "用户选择托管配置管理系统";

            stringPoint12.Append(numericValue12);

            C.StringPoint stringPoint13 = new C.StringPoint() { Index = (UInt32Value)11U };
            C.NumericValue numericValue13 = new C.NumericValue();
            numericValue13.Text = "权限管理";

            stringPoint13.Append(numericValue13);

            C.StringPoint stringPoint14 = new C.StringPoint() { Index = (UInt32Value)12U };
            C.NumericValue numericValue14 = new C.NumericValue();
            numericValue14.Text = "元素之间的相互转化";

            stringPoint14.Append(numericValue14);

            C.StringPoint stringPoint15 = new C.StringPoint() { Index = (UInt32Value)13U };
            C.NumericValue numericValue15 = new C.NumericValue();
            numericValue15.Text = "会议管理";

            stringPoint15.Append(numericValue15);

            C.StringPoint stringPoint16 = new C.StringPoint() { Index = (UInt32Value)14U };
            C.NumericValue numericValue16 = new C.NumericValue();
            numericValue16.Text = "流程管理";

            stringPoint16.Append(numericValue16);

            C.StringPoint stringPoint17 = new C.StringPoint() { Index = (UInt32Value)15U };
            C.NumericValue numericValue17 = new C.NumericValue();
            numericValue17.Text = "支持国际化";

            stringPoint17.Append(numericValue17);

            C.StringPoint stringPoint18 = new C.StringPoint() { Index = (UInt32Value)16U };
            C.NumericValue numericValue18 = new C.NumericValue();
            numericValue18.Text = "事故管理";

            stringPoint18.Append(numericValue18);

            C.StringPoint stringPoint19 = new C.StringPoint() { Index = (UInt32Value)17U };
            C.NumericValue numericValue19 = new C.NumericValue();
            numericValue19.Text = "团队时间管理";

            stringPoint19.Append(numericValue19);

            C.StringPoint stringPoint20 = new C.StringPoint() { Index = (UInt32Value)18U };
            C.NumericValue numericValue20 = new C.NumericValue();
            numericValue20.Text = "添加CAPTCHA支持";

            stringPoint20.Append(numericValue20);

            C.StringPoint stringPoint21 = new C.StringPoint() { Index = (UInt32Value)19U };
            C.NumericValue numericValue21 = new C.NumericValue();
            numericValue21.Text = "实现通用的分页显示功能";

            stringPoint21.Append(numericValue21);

            stringCache2.Append(pointCount2);
            stringCache2.Append(stringPoint2);
            stringCache2.Append(stringPoint3);
            stringCache2.Append(stringPoint4);
            stringCache2.Append(stringPoint5);
            stringCache2.Append(stringPoint6);
            stringCache2.Append(stringPoint7);
            stringCache2.Append(stringPoint8);
            stringCache2.Append(stringPoint9);
            stringCache2.Append(stringPoint10);
            stringCache2.Append(stringPoint11);
            stringCache2.Append(stringPoint12);
            stringCache2.Append(stringPoint13);
            stringCache2.Append(stringPoint14);
            stringCache2.Append(stringPoint15);
            stringCache2.Append(stringPoint16);
            stringCache2.Append(stringPoint17);
            stringCache2.Append(stringPoint18);
            stringCache2.Append(stringPoint19);
            stringCache2.Append(stringPoint20);
            stringCache2.Append(stringPoint21);

            stringReference2.Append(formula2);
            stringReference2.Append(stringCache2);

            categoryAxisData1.Append(stringReference2);

            C.Values values1 = new C.Values();

            C.NumberReference numberReference1 = new C.NumberReference();
            C.Formula formula3 = new C.Formula();
            formula3.Text = "需求工作量按工时分析!$B$4:$B$24";

            C.NumberingCache numberingCache1 = new C.NumberingCache();
            C.FormatCode formatCode1 = new C.FormatCode();
            formatCode1.Text = "General";
            C.PointCount pointCount3 = new C.PointCount() { Val = (UInt32Value)20U };

            C.NumericPoint numericPoint1 = new C.NumericPoint() { Index = (UInt32Value)0U };
            C.NumericValue numericValue22 = new C.NumericValue();
            numericValue22.Text = "361";

            numericPoint1.Append(numericValue22);

            C.NumericPoint numericPoint2 = new C.NumericPoint() { Index = (UInt32Value)1U };
            C.NumericValue numericValue23 = new C.NumericValue();
            numericValue23.Text = "130";

            numericPoint2.Append(numericValue23);

            C.NumericPoint numericPoint3 = new C.NumericPoint() { Index = (UInt32Value)2U };
            C.NumericValue numericValue24 = new C.NumericValue();
            numericValue24.Text = "0";

            numericPoint3.Append(numericValue24);

            C.NumericPoint numericPoint4 = new C.NumericPoint() { Index = (UInt32Value)3U };
            C.NumericValue numericValue25 = new C.NumericValue();
            numericValue25.Text = "0";

            numericPoint4.Append(numericValue25);

            C.NumericPoint numericPoint5 = new C.NumericPoint() { Index = (UInt32Value)4U };
            C.NumericValue numericValue26 = new C.NumericValue();
            numericValue26.Text = "0";

            numericPoint5.Append(numericValue26);

            C.NumericPoint numericPoint6 = new C.NumericPoint() { Index = (UInt32Value)5U };
            C.NumericValue numericValue27 = new C.NumericValue();
            numericValue27.Text = "0";

            numericPoint6.Append(numericValue27);

            C.NumericPoint numericPoint7 = new C.NumericPoint() { Index = (UInt32Value)6U };
            C.NumericValue numericValue28 = new C.NumericValue();
            numericValue28.Text = "0";

            numericPoint7.Append(numericValue28);

            C.NumericPoint numericPoint8 = new C.NumericPoint() { Index = (UInt32Value)7U };
            C.NumericValue numericValue29 = new C.NumericValue();
            numericValue29.Text = "0";

            numericPoint8.Append(numericValue29);

            C.NumericPoint numericPoint9 = new C.NumericPoint() { Index = (UInt32Value)8U };
            C.NumericValue numericValue30 = new C.NumericValue();
            numericValue30.Text = "0";

            numericPoint9.Append(numericValue30);

            C.NumericPoint numericPoint10 = new C.NumericPoint() { Index = (UInt32Value)9U };
            C.NumericValue numericValue31 = new C.NumericValue();
            numericValue31.Text = "0";

            numericPoint10.Append(numericValue31);

            C.NumericPoint numericPoint11 = new C.NumericPoint() { Index = (UInt32Value)10U };
            C.NumericValue numericValue32 = new C.NumericValue();
            numericValue32.Text = "0";

            numericPoint11.Append(numericValue32);

            C.NumericPoint numericPoint12 = new C.NumericPoint() { Index = (UInt32Value)11U };
            C.NumericValue numericValue33 = new C.NumericValue();
            numericValue33.Text = "0";

            numericPoint12.Append(numericValue33);

            C.NumericPoint numericPoint13 = new C.NumericPoint() { Index = (UInt32Value)12U };
            C.NumericValue numericValue34 = new C.NumericValue();
            numericValue34.Text = "0";

            numericPoint13.Append(numericValue34);

            C.NumericPoint numericPoint14 = new C.NumericPoint() { Index = (UInt32Value)13U };
            C.NumericValue numericValue35 = new C.NumericValue();
            numericValue35.Text = "0";

            numericPoint14.Append(numericValue35);

            C.NumericPoint numericPoint15 = new C.NumericPoint() { Index = (UInt32Value)14U };
            C.NumericValue numericValue36 = new C.NumericValue();
            numericValue36.Text = "0";

            numericPoint15.Append(numericValue36);

            C.NumericPoint numericPoint16 = new C.NumericPoint() { Index = (UInt32Value)15U };
            C.NumericValue numericValue37 = new C.NumericValue();
            numericValue37.Text = "0";

            numericPoint16.Append(numericValue37);

            C.NumericPoint numericPoint17 = new C.NumericPoint() { Index = (UInt32Value)16U };
            C.NumericValue numericValue38 = new C.NumericValue();
            numericValue38.Text = "0";

            numericPoint17.Append(numericValue38);

            C.NumericPoint numericPoint18 = new C.NumericPoint() { Index = (UInt32Value)17U };
            C.NumericValue numericValue39 = new C.NumericValue();
            numericValue39.Text = "0";

            numericPoint18.Append(numericValue39);

            C.NumericPoint numericPoint19 = new C.NumericPoint() { Index = (UInt32Value)18U };
            C.NumericValue numericValue40 = new C.NumericValue();
            numericValue40.Text = "0";

            numericPoint19.Append(numericValue40);

            C.NumericPoint numericPoint20 = new C.NumericPoint() { Index = (UInt32Value)19U };
            C.NumericValue numericValue41 = new C.NumericValue();
            numericValue41.Text = "0";

            numericPoint20.Append(numericValue41);

            numberingCache1.Append(formatCode1);
            numberingCache1.Append(pointCount3);
            numberingCache1.Append(numericPoint1);
            numberingCache1.Append(numericPoint2);
            numberingCache1.Append(numericPoint3);
            numberingCache1.Append(numericPoint4);
            numberingCache1.Append(numericPoint5);
            numberingCache1.Append(numericPoint6);
            numberingCache1.Append(numericPoint7);
            numberingCache1.Append(numericPoint8);
            numberingCache1.Append(numericPoint9);
            numberingCache1.Append(numericPoint10);
            numberingCache1.Append(numericPoint11);
            numberingCache1.Append(numericPoint12);
            numberingCache1.Append(numericPoint13);
            numberingCache1.Append(numericPoint14);
            numberingCache1.Append(numericPoint15);
            numberingCache1.Append(numericPoint16);
            numberingCache1.Append(numericPoint17);
            numberingCache1.Append(numericPoint18);
            numberingCache1.Append(numericPoint19);
            numberingCache1.Append(numericPoint20);

            numberReference1.Append(formula3);
            numberReference1.Append(numberingCache1);

            values1.Append(numberReference1);

            barChartSeries1.Append(index3);
            barChartSeries1.Append(order1);
            barChartSeries1.Append(seriesText1);
            barChartSeries1.Append(invertIfNegative1);
            barChartSeries1.Append(categoryAxisData1);
            barChartSeries1.Append(values1);

            C.BarChartSeries barChartSeries2 = new C.BarChartSeries();
            C.Index index4 = new C.Index() { Val = (UInt32Value)1U };
            C.Order order2 = new C.Order() { Val = (UInt32Value)1U };

            C.SeriesText seriesText2 = new C.SeriesText();

            C.StringReference stringReference3 = new C.StringReference();
            C.Formula formula4 = new C.Formula();
            formula4.Text = "需求工作量按工时分析!$C$3";

            C.StringCache stringCache3 = new C.StringCache();
            C.PointCount pointCount4 = new C.PointCount() { Val = (UInt32Value)1U };

            C.StringPoint stringPoint22 = new C.StringPoint() { Index = (UInt32Value)0U };
            C.NumericValue numericValue42 = new C.NumericValue();
            numericValue42.Text = "未完成的工时总计";

            stringPoint22.Append(numericValue42);

            stringCache3.Append(pointCount4);
            stringCache3.Append(stringPoint22);

            stringReference3.Append(formula4);
            stringReference3.Append(stringCache3);

            seriesText2.Append(stringReference3);
            C.InvertIfNegative invertIfNegative2 = new C.InvertIfNegative() { Val = false };

            C.CategoryAxisData categoryAxisData2 = new C.CategoryAxisData();

            C.StringReference stringReference4 = new C.StringReference();
            C.Formula formula5 = new C.Formula();
            formula5.Text = "需求工作量按工时分析!$A$4:$A$24";

            C.StringCache stringCache4 = new C.StringCache();
            C.PointCount pointCount5 = new C.PointCount() { Val = (UInt32Value)20U };

            C.StringPoint stringPoint23 = new C.StringPoint() { Index = (UInt32Value)0U };
            C.NumericValue numericValue43 = new C.NumericValue();
            numericValue43.Text = "需求管理基本功能";

            stringPoint23.Append(numericValue43);

            C.StringPoint stringPoint24 = new C.StringPoint() { Index = (UInt32Value)1U };
            C.NumericValue numericValue44 = new C.NumericValue();
            numericValue44.Text = "整合配置管理系统";

            stringPoint24.Append(numericValue44);

            C.StringPoint stringPoint25 = new C.StringPoint() { Index = (UInt32Value)2U };
            C.NumericValue numericValue45 = new C.NumericValue();
            numericValue45.Text = "集成团队管理";

            stringPoint25.Append(numericValue45);

            C.StringPoint stringPoint26 = new C.StringPoint() { Index = (UInt32Value)3U };
            C.NumericValue numericValue46 = new C.NumericValue();
            numericValue46.Text = "需求管理中的进度跟踪";

            stringPoint26.Append(numericValue46);

            C.StringPoint stringPoint27 = new C.StringPoint() { Index = (UInt32Value)4U };
            C.NumericValue numericValue47 = new C.NumericValue();
            numericValue47.Text = "风险管理";

            stringPoint27.Append(numericValue47);

            C.StringPoint stringPoint28 = new C.StringPoint() { Index = (UInt32Value)5U };
            C.NumericValue numericValue48 = new C.NumericValue();
            numericValue48.Text = "全文检索";

            stringPoint28.Append(numericValue48);

            C.StringPoint stringPoint29 = new C.StringPoint() { Index = (UInt32Value)6U };
            C.NumericValue numericValue49 = new C.NumericValue();
            numericValue49.Text = "决策过程跟踪";

            stringPoint29.Append(numericValue49);

            C.StringPoint stringPoint30 = new C.StringPoint() { Index = (UInt32Value)7U };
            C.NumericValue numericValue50 = new C.NumericValue();
            numericValue50.Text = "统计功能";

            stringPoint30.Append(numericValue50);

            C.StringPoint stringPoint31 = new C.StringPoint() { Index = (UInt32Value)8U };
            C.NumericValue numericValue51 = new C.NumericValue();
            numericValue51.Text = "用户身份验证";

            stringPoint31.Append(numericValue51);

            C.StringPoint stringPoint32 = new C.StringPoint() { Index = (UInt32Value)9U };
            C.NumericValue numericValue52 = new C.NumericValue();
            numericValue52.Text = "任务标签";

            stringPoint32.Append(numericValue52);

            C.StringPoint stringPoint33 = new C.StringPoint() { Index = (UInt32Value)10U };
            C.NumericValue numericValue53 = new C.NumericValue();
            numericValue53.Text = "用户选择托管配置管理系统";

            stringPoint33.Append(numericValue53);

            C.StringPoint stringPoint34 = new C.StringPoint() { Index = (UInt32Value)11U };
            C.NumericValue numericValue54 = new C.NumericValue();
            numericValue54.Text = "权限管理";

            stringPoint34.Append(numericValue54);

            C.StringPoint stringPoint35 = new C.StringPoint() { Index = (UInt32Value)12U };
            C.NumericValue numericValue55 = new C.NumericValue();
            numericValue55.Text = "元素之间的相互转化";

            stringPoint35.Append(numericValue55);

            C.StringPoint stringPoint36 = new C.StringPoint() { Index = (UInt32Value)13U };
            C.NumericValue numericValue56 = new C.NumericValue();
            numericValue56.Text = "会议管理";

            stringPoint36.Append(numericValue56);

            C.StringPoint stringPoint37 = new C.StringPoint() { Index = (UInt32Value)14U };
            C.NumericValue numericValue57 = new C.NumericValue();
            numericValue57.Text = "流程管理";

            stringPoint37.Append(numericValue57);

            C.StringPoint stringPoint38 = new C.StringPoint() { Index = (UInt32Value)15U };
            C.NumericValue numericValue58 = new C.NumericValue();
            numericValue58.Text = "支持国际化";

            stringPoint38.Append(numericValue58);

            C.StringPoint stringPoint39 = new C.StringPoint() { Index = (UInt32Value)16U };
            C.NumericValue numericValue59 = new C.NumericValue();
            numericValue59.Text = "事故管理";

            stringPoint39.Append(numericValue59);

            C.StringPoint stringPoint40 = new C.StringPoint() { Index = (UInt32Value)17U };
            C.NumericValue numericValue60 = new C.NumericValue();
            numericValue60.Text = "团队时间管理";

            stringPoint40.Append(numericValue60);

            C.StringPoint stringPoint41 = new C.StringPoint() { Index = (UInt32Value)18U };
            C.NumericValue numericValue61 = new C.NumericValue();
            numericValue61.Text = "添加CAPTCHA支持";

            stringPoint41.Append(numericValue61);

            C.StringPoint stringPoint42 = new C.StringPoint() { Index = (UInt32Value)19U };
            C.NumericValue numericValue62 = new C.NumericValue();
            numericValue62.Text = "实现通用的分页显示功能";

            stringPoint42.Append(numericValue62);

            stringCache4.Append(pointCount5);
            stringCache4.Append(stringPoint23);
            stringCache4.Append(stringPoint24);
            stringCache4.Append(stringPoint25);
            stringCache4.Append(stringPoint26);
            stringCache4.Append(stringPoint27);
            stringCache4.Append(stringPoint28);
            stringCache4.Append(stringPoint29);
            stringCache4.Append(stringPoint30);
            stringCache4.Append(stringPoint31);
            stringCache4.Append(stringPoint32);
            stringCache4.Append(stringPoint33);
            stringCache4.Append(stringPoint34);
            stringCache4.Append(stringPoint35);
            stringCache4.Append(stringPoint36);
            stringCache4.Append(stringPoint37);
            stringCache4.Append(stringPoint38);
            stringCache4.Append(stringPoint39);
            stringCache4.Append(stringPoint40);
            stringCache4.Append(stringPoint41);
            stringCache4.Append(stringPoint42);

            stringReference4.Append(formula5);
            stringReference4.Append(stringCache4);

            categoryAxisData2.Append(stringReference4);

            C.Values values2 = new C.Values();

            C.NumberReference numberReference2 = new C.NumberReference();
            C.Formula formula6 = new C.Formula();
            formula6.Text = "需求工作量按工时分析!$C$4:$C$24";

            C.NumberingCache numberingCache2 = new C.NumberingCache();
            C.FormatCode formatCode2 = new C.FormatCode();
            formatCode2.Text = "General";
            C.PointCount pointCount6 = new C.PointCount() { Val = (UInt32Value)20U };

            C.NumericPoint numericPoint21 = new C.NumericPoint() { Index = (UInt32Value)0U };
            C.NumericValue numericValue63 = new C.NumericValue();
            numericValue63.Text = "1225";

            numericPoint21.Append(numericValue63);

            C.NumericPoint numericPoint22 = new C.NumericPoint() { Index = (UInt32Value)1U };
            C.NumericValue numericValue64 = new C.NumericValue();
            numericValue64.Text = "370";

            numericPoint22.Append(numericValue64);

            C.NumericPoint numericPoint23 = new C.NumericPoint() { Index = (UInt32Value)2U };
            C.NumericValue numericValue65 = new C.NumericValue();
            numericValue65.Text = "360";

            numericPoint23.Append(numericValue65);

            C.NumericPoint numericPoint24 = new C.NumericPoint() { Index = (UInt32Value)3U };
            C.NumericValue numericValue66 = new C.NumericValue();
            numericValue66.Text = "288";

            numericPoint24.Append(numericValue66);

            C.NumericPoint numericPoint25 = new C.NumericPoint() { Index = (UInt32Value)4U };
            C.NumericValue numericValue67 = new C.NumericValue();
            numericValue67.Text = "240";

            numericPoint25.Append(numericValue67);

            C.NumericPoint numericPoint26 = new C.NumericPoint() { Index = (UInt32Value)5U };
            C.NumericValue numericValue68 = new C.NumericValue();
            numericValue68.Text = "240";

            numericPoint26.Append(numericValue68);

            C.NumericPoint numericPoint27 = new C.NumericPoint() { Index = (UInt32Value)6U };
            C.NumericValue numericValue69 = new C.NumericValue();
            numericValue69.Text = "219";

            numericPoint27.Append(numericValue69);

            C.NumericPoint numericPoint28 = new C.NumericPoint() { Index = (UInt32Value)7U };
            C.NumericValue numericValue70 = new C.NumericValue();
            numericValue70.Text = "195";

            numericPoint28.Append(numericValue70);

            C.NumericPoint numericPoint29 = new C.NumericPoint() { Index = (UInt32Value)8U };
            C.NumericValue numericValue71 = new C.NumericValue();
            numericValue71.Text = "168";

            numericPoint29.Append(numericValue71);

            C.NumericPoint numericPoint30 = new C.NumericPoint() { Index = (UInt32Value)9U };
            C.NumericValue numericValue72 = new C.NumericValue();
            numericValue72.Text = "144";

            numericPoint30.Append(numericValue72);

            C.NumericPoint numericPoint31 = new C.NumericPoint() { Index = (UInt32Value)10U };
            C.NumericValue numericValue73 = new C.NumericValue();
            numericValue73.Text = "144";

            numericPoint31.Append(numericValue73);

            C.NumericPoint numericPoint32 = new C.NumericPoint() { Index = (UInt32Value)11U };
            C.NumericValue numericValue74 = new C.NumericValue();
            numericValue74.Text = "120";

            numericPoint32.Append(numericValue74);

            C.NumericPoint numericPoint33 = new C.NumericPoint() { Index = (UInt32Value)12U };
            C.NumericValue numericValue75 = new C.NumericValue();
            numericValue75.Text = "96";

            numericPoint33.Append(numericValue75);

            C.NumericPoint numericPoint34 = new C.NumericPoint() { Index = (UInt32Value)13U };
            C.NumericValue numericValue76 = new C.NumericValue();
            numericValue76.Text = "72";

            numericPoint34.Append(numericValue76);

            C.NumericPoint numericPoint35 = new C.NumericPoint() { Index = (UInt32Value)14U };
            C.NumericValue numericValue77 = new C.NumericValue();
            numericValue77.Text = "48";

            numericPoint35.Append(numericValue77);

            C.NumericPoint numericPoint36 = new C.NumericPoint() { Index = (UInt32Value)15U };
            C.NumericValue numericValue78 = new C.NumericValue();
            numericValue78.Text = "24";

            numericPoint36.Append(numericValue78);

            C.NumericPoint numericPoint37 = new C.NumericPoint() { Index = (UInt32Value)16U };
            C.NumericValue numericValue79 = new C.NumericValue();
            numericValue79.Text = "0";

            numericPoint37.Append(numericValue79);

            C.NumericPoint numericPoint38 = new C.NumericPoint() { Index = (UInt32Value)17U };
            C.NumericValue numericValue80 = new C.NumericValue();
            numericValue80.Text = "0";

            numericPoint38.Append(numericValue80);

            C.NumericPoint numericPoint39 = new C.NumericPoint() { Index = (UInt32Value)18U };
            C.NumericValue numericValue81 = new C.NumericValue();
            numericValue81.Text = "0";

            numericPoint39.Append(numericValue81);

            C.NumericPoint numericPoint40 = new C.NumericPoint() { Index = (UInt32Value)19U };
            C.NumericValue numericValue82 = new C.NumericValue();
            numericValue82.Text = "0";

            numericPoint40.Append(numericValue82);

            numberingCache2.Append(formatCode2);
            numberingCache2.Append(pointCount6);
            numberingCache2.Append(numericPoint21);
            numberingCache2.Append(numericPoint22);
            numberingCache2.Append(numericPoint23);
            numberingCache2.Append(numericPoint24);
            numberingCache2.Append(numericPoint25);
            numberingCache2.Append(numericPoint26);
            numberingCache2.Append(numericPoint27);
            numberingCache2.Append(numericPoint28);
            numberingCache2.Append(numericPoint29);
            numberingCache2.Append(numericPoint30);
            numberingCache2.Append(numericPoint31);
            numberingCache2.Append(numericPoint32);
            numberingCache2.Append(numericPoint33);
            numberingCache2.Append(numericPoint34);
            numberingCache2.Append(numericPoint35);
            numberingCache2.Append(numericPoint36);
            numberingCache2.Append(numericPoint37);
            numberingCache2.Append(numericPoint38);
            numberingCache2.Append(numericPoint39);
            numberingCache2.Append(numericPoint40);

            numberReference2.Append(formula6);
            numberReference2.Append(numberingCache2);

            values2.Append(numberReference2);

            barChartSeries2.Append(index4);
            barChartSeries2.Append(order2);
            barChartSeries2.Append(seriesText2);
            barChartSeries2.Append(invertIfNegative2);
            barChartSeries2.Append(categoryAxisData2);
            barChartSeries2.Append(values2);

            C.DataLabels dataLabels1 = new C.DataLabels();
            C.ShowLegendKey showLegendKey1 = new C.ShowLegendKey() { Val = false };
            C.ShowValue showValue1 = new C.ShowValue() { Val = false };
            C.ShowCategoryName showCategoryName1 = new C.ShowCategoryName() { Val = false };
            C.ShowSeriesName showSeriesName1 = new C.ShowSeriesName() { Val = false };
            C.ShowPercent showPercent1 = new C.ShowPercent() { Val = false };
            C.ShowBubbleSize showBubbleSize1 = new C.ShowBubbleSize() { Val = false };

            dataLabels1.Append(showLegendKey1);
            dataLabels1.Append(showValue1);
            dataLabels1.Append(showCategoryName1);
            dataLabels1.Append(showSeriesName1);
            dataLabels1.Append(showPercent1);
            dataLabels1.Append(showBubbleSize1);
            C.GapWidth gapWidth1 = new C.GapWidth() { Val = (UInt16Value)75U };
            C.Overlap overlap1 = new C.Overlap() { Val = 100 };
            C.AxisId axisId1 = new C.AxisId() { Val = (UInt32Value)219337856U };
            C.AxisId axisId2 = new C.AxisId() { Val = (UInt32Value)219339392U };

            barChart1.Append(barDirection1);
            barChart1.Append(barGrouping1);
            barChart1.Append(varyColors1);
            barChart1.Append(barChartSeries1);
            barChart1.Append(barChartSeries2);
            barChart1.Append(dataLabels1);
            barChart1.Append(gapWidth1);
            barChart1.Append(overlap1);
            barChart1.Append(axisId1);
            barChart1.Append(axisId2);

            C.CategoryAxis categoryAxis1 = new C.CategoryAxis();
            C.AxisId axisId3 = new C.AxisId() { Val = (UInt32Value)219337856U };

            C.Scaling scaling1 = new C.Scaling();
            C.Orientation orientation1 = new C.Orientation() { Val = C.OrientationValues.MinMax };

            scaling1.Append(orientation1);
            C.Delete delete1 = new C.Delete() { Val = false };
            C.AxisPosition axisPosition1 = new C.AxisPosition() { Val = C.AxisPositionValues.Bottom };
            C.MajorTickMark majorTickMark1 = new C.MajorTickMark() { Val = C.TickMarkValues.None };
            C.MinorTickMark minorTickMark1 = new C.MinorTickMark() { Val = C.TickMarkValues.None };
            C.TickLabelPosition tickLabelPosition1 = new C.TickLabelPosition() { Val = C.TickLabelPositionValues.NextTo };
            C.CrossingAxis crossingAxis1 = new C.CrossingAxis() { Val = (UInt32Value)219339392U };
            C.Crosses crosses1 = new C.Crosses() { Val = C.CrossesValues.AutoZero };
            C.AutoLabeled autoLabeled1 = new C.AutoLabeled() { Val = true };
            C.LabelAlignment labelAlignment1 = new C.LabelAlignment() { Val = C.LabelAlignmentValues.Center };
            C.LabelOffset labelOffset1 = new C.LabelOffset() { Val = (UInt16Value)100U };
            C.NoMultiLevelLabels noMultiLevelLabels1 = new C.NoMultiLevelLabels() { Val = false };

            categoryAxis1.Append(axisId3);
            categoryAxis1.Append(scaling1);
            categoryAxis1.Append(delete1);
            categoryAxis1.Append(axisPosition1);
            categoryAxis1.Append(majorTickMark1);
            categoryAxis1.Append(minorTickMark1);
            categoryAxis1.Append(tickLabelPosition1);
            categoryAxis1.Append(crossingAxis1);
            categoryAxis1.Append(crosses1);
            categoryAxis1.Append(autoLabeled1);
            categoryAxis1.Append(labelAlignment1);
            categoryAxis1.Append(labelOffset1);
            categoryAxis1.Append(noMultiLevelLabels1);

            C.ValueAxis valueAxis1 = new C.ValueAxis();
            C.AxisId axisId4 = new C.AxisId() { Val = (UInt32Value)219339392U };

            C.Scaling scaling2 = new C.Scaling();
            C.Orientation orientation2 = new C.Orientation() { Val = C.OrientationValues.MinMax };

            scaling2.Append(orientation2);
            C.Delete delete2 = new C.Delete() { Val = false };
            C.AxisPosition axisPosition2 = new C.AxisPosition() { Val = C.AxisPositionValues.Left };
            C.MajorGridlines majorGridlines1 = new C.MajorGridlines();
            C.NumberingFormat numberingFormat1 = new C.NumberingFormat() { FormatCode = "General", SourceLinked = true };
            C.MajorTickMark majorTickMark2 = new C.MajorTickMark() { Val = C.TickMarkValues.None };
            C.MinorTickMark minorTickMark2 = new C.MinorTickMark() { Val = C.TickMarkValues.None };
            C.TickLabelPosition tickLabelPosition2 = new C.TickLabelPosition() { Val = C.TickLabelPositionValues.NextTo };

            C.ChartShapeProperties chartShapeProperties1 = new C.ChartShapeProperties();

            A.Outline outline1 = new A.Outline() { Width = 9525 };
            A.NoFill noFill1 = new A.NoFill();

            outline1.Append(noFill1);

            chartShapeProperties1.Append(outline1);
            C.CrossingAxis crossingAxis2 = new C.CrossingAxis() { Val = (UInt32Value)219337856U };
            C.Crosses crosses2 = new C.Crosses() { Val = C.CrossesValues.AutoZero };
            C.CrossBetween crossBetween1 = new C.CrossBetween() { Val = C.CrossBetweenValues.Between };

            valueAxis1.Append(axisId4);
            valueAxis1.Append(scaling2);
            valueAxis1.Append(delete2);
            valueAxis1.Append(axisPosition2);
            valueAxis1.Append(majorGridlines1);
            valueAxis1.Append(numberingFormat1);
            valueAxis1.Append(majorTickMark2);
            valueAxis1.Append(minorTickMark2);
            valueAxis1.Append(tickLabelPosition2);
            valueAxis1.Append(chartShapeProperties1);
            valueAxis1.Append(crossingAxis2);
            valueAxis1.Append(crosses2);
            valueAxis1.Append(crossBetween1);

            plotArea1.Append(layout2);
            plotArea1.Append(barChart1);
            plotArea1.Append(categoryAxis1);
            plotArea1.Append(valueAxis1);

            C.Legend legend1 = new C.Legend();
            C.LegendPosition legendPosition1 = new C.LegendPosition() { Val = C.LegendPositionValues.Bottom };
            C.Layout layout3 = new C.Layout();
            C.Overlay overlay2 = new C.Overlay() { Val = false };

            legend1.Append(legendPosition1);
            legend1.Append(layout3);
            legend1.Append(overlay2);
            C.PlotVisibleOnly plotVisibleOnly1 = new C.PlotVisibleOnly() { Val = true };
            C.DisplayBlanksAs displayBlanksAs1 = new C.DisplayBlanksAs() { Val = C.DisplayBlanksAsValues.Gap };
            C.ShowDataLabelsOverMaximum showDataLabelsOverMaximum1 = new C.ShowDataLabelsOverMaximum() { Val = false };

            chart1.Append(title1);
            chart1.Append(autoTitleDeleted1);
            chart1.Append(pivotFormats1);
            chart1.Append(plotArea1);
            chart1.Append(legend1);
            chart1.Append(plotVisibleOnly1);
            chart1.Append(displayBlanksAs1);
            chart1.Append(showDataLabelsOverMaximum1);

            C.PrintSettings printSettings1 = new C.PrintSettings();
            C.HeaderFooter headerFooter1 = new C.HeaderFooter();
            C.PageMargins pageMargins2 = new C.PageMargins() { Left = 0.7D, Right = 0.7D, Top = 0.75D, Bottom = 0.75D, Header = 0.3D, Footer = 0.3D };
            C.PageSetup pageSetup1 = new C.PageSetup();

            printSettings1.Append(headerFooter1);
            printSettings1.Append(pageMargins2);
            printSettings1.Append(pageSetup1);

            C.ChartSpaceExtensionList chartSpaceExtensionList1 = new C.ChartSpaceExtensionList();

            C.ChartSpaceExtension chartSpaceExtension1 = new C.ChartSpaceExtension() { Uri = "{781A3756-C4B2-4CAC-9D66-4F8BD8637D16}" };
            chartSpaceExtension1.AddNamespaceDeclaration("c14", "http://schemas.microsoft.com/office/drawing/2007/8/2/chart");

            C14.PivotOptions pivotOptions1 = new C14.PivotOptions();
            C14.DropZoneFilter dropZoneFilter1 = new C14.DropZoneFilter() { Val = true };
            C14.DropZoneCategories dropZoneCategories1 = new C14.DropZoneCategories() { Val = true };
            C14.DropZoneData dropZoneData1 = new C14.DropZoneData() { Val = true };
            C14.DropZoneSeries dropZoneSeries1 = new C14.DropZoneSeries() { Val = true };
            C14.DropZonesVisible dropZonesVisible1 = new C14.DropZonesVisible() { Val = true };

            pivotOptions1.Append(dropZoneFilter1);
            pivotOptions1.Append(dropZoneCategories1);
            pivotOptions1.Append(dropZoneData1);
            pivotOptions1.Append(dropZoneSeries1);
            pivotOptions1.Append(dropZonesVisible1);

            chartSpaceExtension1.Append(pivotOptions1);

            chartSpaceExtensionList1.Append(chartSpaceExtension1);

            chartSpace1.Append(date19041);
            chartSpace1.Append(editingLanguage1);
            chartSpace1.Append(roundedCorners1);
            chartSpace1.Append(alternateContent1);
            chartSpace1.Append(pivotSource1);
            chartSpace1.Append(chart1);
            chartSpace1.Append(printSettings1);
            chartSpace1.Append(chartSpaceExtensionList1);

            chartPart1.ChartSpace = chartSpace1;
        }

        // Generates content of pivotTablePart1.
        private void GeneratePivotTablePart1Content(PivotTablePart pivotTablePart1)
        {
            PivotTableDefinition pivotTableDefinition1 = new PivotTableDefinition() { Name = "PivotTable4", CacheId = (UInt32Value)7U, ApplyNumberFormats = false, ApplyBorderFormats = false, ApplyFontFormats = false, ApplyPatternFormats = false, ApplyAlignmentFormats = false, ApplyWidthHeightFormats = true, DataCaption = "Values", UpdatedVersion = 4, MinRefreshableVersion = 3, UseAutoFormatting = true, ItemPrintTitles = true, CreatedVersion = 4, Indent = (UInt32Value)0U, Outline = true, OutlineData = true, MultipleFieldFilters = false, ChartFormat = (UInt32Value)1U };
            Location location1 = new Location() { Reference = "A3:C24", FirstHeaderRow = (UInt32Value)0U, FirstDataRow = (UInt32Value)1U, FirstDataColumn = (UInt32Value)1U, RowPageCount = (UInt32Value)1U, ColumnsPerPage = (UInt32Value)1U };

            PivotFields pivotFields1 = new PivotFields() { Count = (UInt32Value)12U };

            PivotField pivotField1 = new PivotField() { Axis = PivotTableAxisValues.AxisRow, ShowAll = false, SortType = FieldSortValues.Descending };

            Items items1 = new Items() { Count = (UInt32Value)21U };
            Item item1 = new Item() { Index = (UInt32Value)1U };
            Item item2 = new Item() { Index = (UInt32Value)0U };
            Item item3 = new Item() { Index = (UInt32Value)11U };
            Item item4 = new Item() { Index = (UInt32Value)10U };
            Item item5 = new Item() { Index = (UInt32Value)9U };
            Item item6 = new Item() { Index = (UInt32Value)15U };
            Item item7 = new Item() { Index = (UInt32Value)2U };
            Item item8 = new Item() { Index = (UInt32Value)18U };
            Item item9 = new Item() { Index = (UInt32Value)19U };
            Item item10 = new Item() { Index = (UInt32Value)3U };
            Item item11 = new Item() { Index = (UInt32Value)17U };
            Item item12 = new Item() { Index = (UInt32Value)6U };
            Item item13 = new Item() { Index = (UInt32Value)7U };
            Item item14 = new Item() { Index = (UInt32Value)8U };
            Item item15 = new Item() { Index = (UInt32Value)5U };
            Item item16 = new Item() { Index = (UInt32Value)14U };
            Item item17 = new Item() { Index = (UInt32Value)4U };
            Item item18 = new Item() { Index = (UInt32Value)13U };
            Item item19 = new Item() { Index = (UInt32Value)12U };
            Item item20 = new Item() { Index = (UInt32Value)16U };
            Item item21 = new Item() { ItemType = ItemValues.Default };

            items1.Append(item1);
            items1.Append(item2);
            items1.Append(item3);
            items1.Append(item4);
            items1.Append(item5);
            items1.Append(item6);
            items1.Append(item7);
            items1.Append(item8);
            items1.Append(item9);
            items1.Append(item10);
            items1.Append(item11);
            items1.Append(item12);
            items1.Append(item13);
            items1.Append(item14);
            items1.Append(item15);
            items1.Append(item16);
            items1.Append(item17);
            items1.Append(item18);
            items1.Append(item19);
            items1.Append(item20);
            items1.Append(item21);

            AutoSortScope autoSortScope1 = new AutoSortScope();

            PivotArea pivotArea1 = new PivotArea() { DataOnly = false, Outline = false, FieldPosition = (UInt32Value)0U };

            PivotAreaReferences pivotAreaReferences1 = new PivotAreaReferences() { Count = (UInt32Value)1U };

            PivotAreaReference pivotAreaReference1 = new PivotAreaReference() { Field = (UInt32Value)4294967294U, Count = (UInt32Value)1U, Selected = false };
            FieldItem fieldItem1 = new FieldItem() { Val = (UInt32Value)1U };

            pivotAreaReference1.Append(fieldItem1);

            pivotAreaReferences1.Append(pivotAreaReference1);

            pivotArea1.Append(pivotAreaReferences1);

            autoSortScope1.Append(pivotArea1);

            pivotField1.Append(items1);
            pivotField1.Append(autoSortScope1);

            PivotField pivotField2 = new PivotField() { Axis = PivotTableAxisValues.AxisPage, ShowAll = false };

            Items items2 = new Items() { Count = (UInt32Value)2U };
            Item item22 = new Item() { Index = (UInt32Value)0U };
            Item item23 = new Item() { ItemType = ItemValues.Default };

            items2.Append(item22);
            items2.Append(item23);

            pivotField2.Append(items2);
            PivotField pivotField3 = new PivotField() { ShowAll = false };
            PivotField pivotField4 = new PivotField() { ShowAll = false };
            PivotField pivotField5 = new PivotField() { ShowAll = false };
            PivotField pivotField6 = new PivotField() { ShowAll = false };
            PivotField pivotField7 = new PivotField() { DataField = true, ShowAll = false };
            PivotField pivotField8 = new PivotField() { DataField = true, ShowAll = false };
            PivotField pivotField9 = new PivotField() { ShowAll = false };
            PivotField pivotField10 = new PivotField() { ShowAll = false };
            PivotField pivotField11 = new PivotField() { ShowAll = false };
            PivotField pivotField12 = new PivotField() { ShowAll = false };

            pivotFields1.Append(pivotField1);
            pivotFields1.Append(pivotField2);
            pivotFields1.Append(pivotField3);
            pivotFields1.Append(pivotField4);
            pivotFields1.Append(pivotField5);
            pivotFields1.Append(pivotField6);
            pivotFields1.Append(pivotField7);
            pivotFields1.Append(pivotField8);
            pivotFields1.Append(pivotField9);
            pivotFields1.Append(pivotField10);
            pivotFields1.Append(pivotField11);
            pivotFields1.Append(pivotField12);

            RowFields rowFields1 = new RowFields() { Count = (UInt32Value)1U };
            Field field1 = new Field() { Index = 0 };

            rowFields1.Append(field1);

            RowItems rowItems1 = new RowItems() { Count = (UInt32Value)21U };

            RowItem rowItem1 = new RowItem();
            MemberPropertyIndex memberPropertyIndex1 = new MemberPropertyIndex() { Val = 18 };

            rowItem1.Append(memberPropertyIndex1);

            RowItem rowItem2 = new RowItem();
            MemberPropertyIndex memberPropertyIndex2 = new MemberPropertyIndex() { Val = 9 };

            rowItem2.Append(memberPropertyIndex2);

            RowItem rowItem3 = new RowItem();
            MemberPropertyIndex memberPropertyIndex3 = new MemberPropertyIndex() { Val = 16 };

            rowItem3.Append(memberPropertyIndex3);

            RowItem rowItem4 = new RowItem();
            MemberPropertyIndex memberPropertyIndex4 = new MemberPropertyIndex() { Val = 17 };

            rowItem4.Append(memberPropertyIndex4);

            RowItem rowItem5 = new RowItem();
            MemberPropertyIndex memberPropertyIndex5 = new MemberPropertyIndex() { Val = 19 };

            rowItem5.Append(memberPropertyIndex5);

            RowItem rowItem6 = new RowItem();
            MemberPropertyIndex memberPropertyIndex6 = new MemberPropertyIndex() { Val = 4 };

            rowItem6.Append(memberPropertyIndex6);

            RowItem rowItem7 = new RowItem();
            MemberPropertyIndex memberPropertyIndex7 = new MemberPropertyIndex() { Val = 5 };

            rowItem7.Append(memberPropertyIndex7);

            RowItem rowItem8 = new RowItem();
            MemberPropertyIndex memberPropertyIndex8 = new MemberPropertyIndex() { Val = 15 };

            rowItem8.Append(memberPropertyIndex8);

            RowItem rowItem9 = new RowItem();
            MemberPropertyIndex memberPropertyIndex9 = new MemberPropertyIndex() { Val = 13 };

            rowItem9.Append(memberPropertyIndex9);

            RowItem rowItem10 = new RowItem();
            MemberPropertyIndex memberPropertyIndex10 = new MemberPropertyIndex() { Val = 1 };

            rowItem10.Append(memberPropertyIndex10);

            RowItem rowItem11 = new RowItem();
            MemberPropertyIndex memberPropertyIndex11 = new MemberPropertyIndex() { Val = 14 };

            rowItem11.Append(memberPropertyIndex11);

            RowItem rowItem12 = new RowItem();
            MemberPropertyIndex memberPropertyIndex12 = new MemberPropertyIndex() { Val = 10 };

            rowItem12.Append(memberPropertyIndex12);

            RowItem rowItem13 = new RowItem();
            MemberPropertyIndex memberPropertyIndex13 = new MemberPropertyIndex() { Val = 3 };

            rowItem13.Append(memberPropertyIndex13);

            RowItem rowItem14 = new RowItem();
            MemberPropertyIndex memberPropertyIndex14 = new MemberPropertyIndex() { Val = 2 };

            rowItem14.Append(memberPropertyIndex14);

            RowItem rowItem15 = new RowItem();
            MemberPropertyIndex memberPropertyIndex15 = new MemberPropertyIndex() { Val = 11 };

            rowItem15.Append(memberPropertyIndex15);

            RowItem rowItem16 = new RowItem();
            MemberPropertyIndex memberPropertyIndex16 = new MemberPropertyIndex() { Val = 8 };

            rowItem16.Append(memberPropertyIndex16);

            RowItem rowItem17 = new RowItem();
            MemberPropertyIndex memberPropertyIndex17 = new MemberPropertyIndex();

            rowItem17.Append(memberPropertyIndex17);

            RowItem rowItem18 = new RowItem();
            MemberPropertyIndex memberPropertyIndex18 = new MemberPropertyIndex() { Val = 6 };

            rowItem18.Append(memberPropertyIndex18);

            RowItem rowItem19 = new RowItem();
            MemberPropertyIndex memberPropertyIndex19 = new MemberPropertyIndex() { Val = 12 };

            rowItem19.Append(memberPropertyIndex19);

            RowItem rowItem20 = new RowItem();
            MemberPropertyIndex memberPropertyIndex20 = new MemberPropertyIndex() { Val = 7 };

            rowItem20.Append(memberPropertyIndex20);

            RowItem rowItem21 = new RowItem() { ItemType = ItemValues.Grand };
            MemberPropertyIndex memberPropertyIndex21 = new MemberPropertyIndex();

            rowItem21.Append(memberPropertyIndex21);

            rowItems1.Append(rowItem1);
            rowItems1.Append(rowItem2);
            rowItems1.Append(rowItem3);
            rowItems1.Append(rowItem4);
            rowItems1.Append(rowItem5);
            rowItems1.Append(rowItem6);
            rowItems1.Append(rowItem7);
            rowItems1.Append(rowItem8);
            rowItems1.Append(rowItem9);
            rowItems1.Append(rowItem10);
            rowItems1.Append(rowItem11);
            rowItems1.Append(rowItem12);
            rowItems1.Append(rowItem13);
            rowItems1.Append(rowItem14);
            rowItems1.Append(rowItem15);
            rowItems1.Append(rowItem16);
            rowItems1.Append(rowItem17);
            rowItems1.Append(rowItem18);
            rowItems1.Append(rowItem19);
            rowItems1.Append(rowItem20);
            rowItems1.Append(rowItem21);

            ColumnFields columnFields1 = new ColumnFields() { Count = (UInt32Value)1U };
            Field field2 = new Field() { Index = -2 };

            columnFields1.Append(field2);

            ColumnItems columnItems1 = new ColumnItems() { Count = (UInt32Value)2U };

            RowItem rowItem22 = new RowItem();
            MemberPropertyIndex memberPropertyIndex22 = new MemberPropertyIndex();

            rowItem22.Append(memberPropertyIndex22);

            RowItem rowItem23 = new RowItem() { Index = (UInt32Value)1U };
            MemberPropertyIndex memberPropertyIndex23 = new MemberPropertyIndex() { Val = 1 };

            rowItem23.Append(memberPropertyIndex23);

            columnItems1.Append(rowItem22);
            columnItems1.Append(rowItem23);

            PageFields pageFields1 = new PageFields() { Count = (UInt32Value)1U };
            PageField pageField1 = new PageField() { Field = 1, Hierarchy = -1 };

            pageFields1.Append(pageField1);

            DataFields dataFields1 = new DataFields() { Count = (UInt32Value)2U };
            DataField dataField1 = new DataField() { Name = "已完成的工时总计", Field = (UInt32Value)6U, BaseField = 0, BaseItem = (UInt32Value)9U };
            DataField dataField2 = new DataField() { Name = "未完成的工时总计", Field = (UInt32Value)7U, BaseField = 0, BaseItem = (UInt32Value)9U };

            dataFields1.Append(dataField1);
            dataFields1.Append(dataField2);

            ChartFormats chartFormats1 = new ChartFormats() { Count = (UInt32Value)2U };

            ChartFormat chartFormat1 = new ChartFormat() { Chart = (UInt32Value)0U, Format = (UInt32Value)0U, Series = true };

            PivotArea pivotArea2 = new PivotArea() { Type = PivotAreaValues.Data, Outline = false, FieldPosition = (UInt32Value)0U };

            PivotAreaReferences pivotAreaReferences2 = new PivotAreaReferences() { Count = (UInt32Value)1U };

            PivotAreaReference pivotAreaReference2 = new PivotAreaReference() { Field = (UInt32Value)4294967294U, Count = (UInt32Value)1U, Selected = false };
            FieldItem fieldItem2 = new FieldItem() { Val = (UInt32Value)0U };

            pivotAreaReference2.Append(fieldItem2);

            pivotAreaReferences2.Append(pivotAreaReference2);

            pivotArea2.Append(pivotAreaReferences2);

            chartFormat1.Append(pivotArea2);

            ChartFormat chartFormat2 = new ChartFormat() { Chart = (UInt32Value)0U, Format = (UInt32Value)1U, Series = true };

            PivotArea pivotArea3 = new PivotArea() { Type = PivotAreaValues.Data, Outline = false, FieldPosition = (UInt32Value)0U };

            PivotAreaReferences pivotAreaReferences3 = new PivotAreaReferences() { Count = (UInt32Value)1U };

            PivotAreaReference pivotAreaReference3 = new PivotAreaReference() { Field = (UInt32Value)4294967294U, Count = (UInt32Value)1U, Selected = false };
            FieldItem fieldItem3 = new FieldItem() { Val = (UInt32Value)1U };

            pivotAreaReference3.Append(fieldItem3);

            pivotAreaReferences3.Append(pivotAreaReference3);

            pivotArea3.Append(pivotAreaReferences3);

            chartFormat2.Append(pivotArea3);

            chartFormats1.Append(chartFormat1);
            chartFormats1.Append(chartFormat2);
            PivotTableStyle pivotTableStyle1 = new PivotTableStyle() { Name = "PivotStyleLight16", ShowRowHeaders = true, ShowColumnHeaders = true, ShowRowStripes = false, ShowColumnStripes = false, ShowLastColumn = true };

            PivotTableDefinitionExtensionList pivotTableDefinitionExtensionList1 = new PivotTableDefinitionExtensionList();

            PivotTableDefinitionExtension pivotTableDefinitionExtension1 = new PivotTableDefinitionExtension() { Uri = "{962EF5D1-5CA2-4c93-8EF4-DBF5C05439D2}" };
            pivotTableDefinitionExtension1.AddNamespaceDeclaration("x14", "http://schemas.microsoft.com/office/spreadsheetml/2009/9/main");

            X14.PivotTableDefinition pivotTableDefinition2 = new X14.PivotTableDefinition() { HideValuesRow = true };
            pivotTableDefinition2.AddNamespaceDeclaration("xm", "http://schemas.microsoft.com/office/excel/2006/main");

            pivotTableDefinitionExtension1.Append(pivotTableDefinition2);

            pivotTableDefinitionExtensionList1.Append(pivotTableDefinitionExtension1);

            pivotTableDefinition1.Append(location1);
            pivotTableDefinition1.Append(pivotFields1);
            pivotTableDefinition1.Append(rowFields1);
            pivotTableDefinition1.Append(rowItems1);
            pivotTableDefinition1.Append(columnFields1);
            pivotTableDefinition1.Append(columnItems1);
            pivotTableDefinition1.Append(pageFields1);
            pivotTableDefinition1.Append(dataFields1);
            pivotTableDefinition1.Append(chartFormats1);
            pivotTableDefinition1.Append(pivotTableStyle1);
            pivotTableDefinition1.Append(pivotTableDefinitionExtensionList1);

            pivotTablePart1.PivotTableDefinition = pivotTableDefinition1;
        }

        // Generates content of pivotTableCacheDefinitionPart1.
        private void GeneratePivotTableCacheDefinitionPart1Content(PivotTableCacheDefinitionPart pivotTableCacheDefinitionPart1)
        {
            PivotCacheDefinition pivotCacheDefinition1 = new PivotCacheDefinition() { Id = "rId1", RefreshedBy = "MSPD", RefreshedDate = 40652.734881828706D, CreatedVersion = 4, RefreshedVersion = 4, MinRefreshableVersion = 3, RecordCount = (UInt32Value)24U };
            pivotCacheDefinition1.AddNamespaceDeclaration("r", "http://schemas.openxmlformats.org/officeDocument/2006/relationships");

            CacheSource cacheSource1 = new CacheSource() { Type = SourceValues.Worksheet };
            WorksheetSource worksheetSource1 = new WorksheetSource() { Reference = "A1:L25", Sheet = ConnectionName };

            cacheSource1.Append(worksheetSource1);

            CacheFields cacheFields1 = new CacheFields() { Count = (UInt32Value)12U };

            CacheField cacheField1 = new CacheField() { Name = "Requirement", NumberFormatId = (UInt32Value)0U };

            SharedItems sharedItems1 = new SharedItems() { Count = (UInt32Value)20U };
            StringItem stringItem1 = new StringItem() { Val = "任务标签" };
            StringItem stringItem2 = new StringItem() { Val = "事故管理" };
            StringItem stringItem3 = new StringItem() { Val = "团队时间管理" };
            StringItem stringItem4 = new StringItem() { Val = "整合配置管理系统" };
            StringItem stringItem5 = new StringItem() { Val = "集成团队管理" };
            StringItem stringItem6 = new StringItem() { Val = "用户选择托管配置管理系统" };
            StringItem stringItem7 = new StringItem() { Val = "流程管理" };
            StringItem stringItem8 = new StringItem() { Val = "添加CAPTCHA支持" };
            StringItem stringItem9 = new StringItem() { Val = "用户身份验证" };
            StringItem stringItem10 = new StringItem() { Val = "全文检索" };
            StringItem stringItem11 = new StringItem() { Val = "元素之间的相互转化" };
            StringItem stringItem12 = new StringItem() { Val = "会议管理" };
            StringItem stringItem13 = new StringItem() { Val = "需求管理基本功能" };
            StringItem stringItem14 = new StringItem() { Val = "需求管理中的进度跟踪" };
            StringItem stringItem15 = new StringItem() { Val = "统计功能" };
            StringItem stringItem16 = new StringItem() { Val = "决策过程跟踪" };
            StringItem stringItem17 = new StringItem() { Val = "风险管理" };
            StringItem stringItem18 = new StringItem() { Val = "权限管理" };
            StringItem stringItem19 = new StringItem() { Val = "实现通用的分页显示功能" };
            StringItem stringItem20 = new StringItem() { Val = "支持国际化" };

            sharedItems1.Append(stringItem1);
            sharedItems1.Append(stringItem2);
            sharedItems1.Append(stringItem3);
            sharedItems1.Append(stringItem4);
            sharedItems1.Append(stringItem5);
            sharedItems1.Append(stringItem6);
            sharedItems1.Append(stringItem7);
            sharedItems1.Append(stringItem8);
            sharedItems1.Append(stringItem9);
            sharedItems1.Append(stringItem10);
            sharedItems1.Append(stringItem11);
            sharedItems1.Append(stringItem12);
            sharedItems1.Append(stringItem13);
            sharedItems1.Append(stringItem14);
            sharedItems1.Append(stringItem15);
            sharedItems1.Append(stringItem16);
            sharedItems1.Append(stringItem17);
            sharedItems1.Append(stringItem18);
            sharedItems1.Append(stringItem19);
            sharedItems1.Append(stringItem20);

            cacheField1.Append(sharedItems1);

            CacheField cacheField2 = new CacheField() { Name = "需求级别", NumberFormatId = (UInt32Value)0U };

            SharedItems sharedItems2 = new SharedItems() { ContainsSemiMixedTypes = false, ContainsString = false, ContainsNumber = true, ContainsInteger = true, MinValue = 0D, MaxValue = 0D, Count = (UInt32Value)1U };
            NumberItem numberItem1 = new NumberItem() { Val = 0D };

            sharedItems2.Append(numberItem1);

            cacheField2.Append(sharedItems2);

            CacheField cacheField3 = new CacheField() { Name = "未完成的工作项目数目", NumberFormatId = (UInt32Value)0U };
            SharedItems sharedItems3 = new SharedItems() { ContainsSemiMixedTypes = false, ContainsString = false, ContainsNumber = true, ContainsInteger = true, MinValue = 0D, MaxValue = 12D };

            cacheField3.Append(sharedItems3);

            CacheField cacheField4 = new CacheField() { Name = "已完成的工作项目数目", NumberFormatId = (UInt32Value)0U };
            SharedItems sharedItems4 = new SharedItems() { ContainsSemiMixedTypes = false, ContainsString = false, ContainsNumber = true, ContainsInteger = true, MinValue = 0D, MaxValue = 6D };

            cacheField4.Append(sharedItems4);

            CacheField cacheField5 = new CacheField() { Name = "未完成的任务数目", NumberFormatId = (UInt32Value)0U };
            SharedItems sharedItems5 = new SharedItems() { ContainsSemiMixedTypes = false, ContainsString = false, ContainsNumber = true, ContainsInteger = true, MinValue = 0D, MaxValue = 10D };

            cacheField5.Append(sharedItems5);

            CacheField cacheField6 = new CacheField() { Name = "已完成的任务数目", NumberFormatId = (UInt32Value)0U };
            SharedItems sharedItems6 = new SharedItems() { ContainsSemiMixedTypes = false, ContainsString = false, ContainsNumber = true, ContainsInteger = true, MinValue = 0D, MaxValue = 5D };

            cacheField6.Append(sharedItems6);

            CacheField cacheField7 = new CacheField() { Name = "已完成的工时", NumberFormatId = (UInt32Value)0U };
            SharedItems sharedItems7 = new SharedItems() { ContainsSemiMixedTypes = false, ContainsString = false, ContainsNumber = true, ContainsInteger = true, MinValue = 0D, MaxValue = 361D };

            cacheField7.Append(sharedItems7);

            CacheField cacheField8 = new CacheField() { Name = "未完成的工时", NumberFormatId = (UInt32Value)0U };
            SharedItems sharedItems8 = new SharedItems() { ContainsSemiMixedTypes = false, ContainsString = false, ContainsNumber = true, ContainsInteger = true, MinValue = 0D, MaxValue = 1225D };

            cacheField8.Append(sharedItems8);

            CacheField cacheField9 = new CacheField() { Name = "挣值", NumberFormatId = (UInt32Value)0U };
            SharedItems sharedItems9 = new SharedItems() { ContainsSemiMixedTypes = false, ContainsString = false, ContainsNumber = true, ContainsInteger = true, MinValue = 0D, MaxValue = 0D };

            cacheField9.Append(sharedItems9);

            CacheField cacheField10 = new CacheField() { Name = "总值", NumberFormatId = (UInt32Value)0U };
            SharedItems sharedItems10 = new SharedItems() { ContainsSemiMixedTypes = false, ContainsString = false, ContainsNumber = true, ContainsInteger = true, MinValue = 0D, MaxValue = 30000D };

            cacheField10.Append(sharedItems10);

            CacheField cacheField11 = new CacheField() { Name = "未完成的需求数目", NumberFormatId = (UInt32Value)0U };
            SharedItems sharedItems11 = new SharedItems() { ContainsSemiMixedTypes = false, ContainsString = false, ContainsNumber = true, ContainsInteger = true, MinValue = 0D, MaxValue = 2D };

            cacheField11.Append(sharedItems11);

            CacheField cacheField12 = new CacheField() { Name = "已完成的需求数目", NumberFormatId = (UInt32Value)0U };
            SharedItems sharedItems12 = new SharedItems() { ContainsSemiMixedTypes = false, ContainsString = false, ContainsNumber = true, ContainsInteger = true, MinValue = 0D, MaxValue = 1D };

            cacheField12.Append(sharedItems12);

            cacheFields1.Append(cacheField1);
            cacheFields1.Append(cacheField2);
            cacheFields1.Append(cacheField3);
            cacheFields1.Append(cacheField4);
            cacheFields1.Append(cacheField5);
            cacheFields1.Append(cacheField6);
            cacheFields1.Append(cacheField7);
            cacheFields1.Append(cacheField8);
            cacheFields1.Append(cacheField9);
            cacheFields1.Append(cacheField10);
            cacheFields1.Append(cacheField11);
            cacheFields1.Append(cacheField12);

            PivotCacheDefinitionExtensionList pivotCacheDefinitionExtensionList1 = new PivotCacheDefinitionExtensionList();

            PivotCacheDefinitionExtension pivotCacheDefinitionExtension1 = new PivotCacheDefinitionExtension() { Uri = "{725AE2AE-9491-48be-B2B4-4EB974FC3084}" };
            pivotCacheDefinitionExtension1.AddNamespaceDeclaration("x14", "http://schemas.microsoft.com/office/spreadsheetml/2009/9/main");
            X14.PivotCacheDefinition pivotCacheDefinition2 = new X14.PivotCacheDefinition();

            pivotCacheDefinitionExtension1.Append(pivotCacheDefinition2);

            pivotCacheDefinitionExtensionList1.Append(pivotCacheDefinitionExtension1);

            pivotCacheDefinition1.Append(cacheSource1);
            pivotCacheDefinition1.Append(cacheFields1);
            pivotCacheDefinition1.Append(pivotCacheDefinitionExtensionList1);

            pivotTableCacheDefinitionPart1.PivotCacheDefinition = pivotCacheDefinition1;
        }

        // Generates content of pivotTableCacheRecordsPart1.
        private void GeneratePivotTableCacheRecordsPart1Content(PivotTableCacheRecordsPart pivotTableCacheRecordsPart1)
        {
            PivotCacheRecords pivotCacheRecords1 = new PivotCacheRecords() { Count = (UInt32Value)24U };
            pivotCacheRecords1.AddNamespaceDeclaration("r", "http://schemas.openxmlformats.org/officeDocument/2006/relationships");

            PivotCacheRecord pivotCacheRecord1 = new PivotCacheRecord();
            FieldItem fieldItem4 = new FieldItem() { Val = (UInt32Value)0U };
            FieldItem fieldItem5 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem2 = new NumberItem() { Val = 4D };
            NumberItem numberItem3 = new NumberItem() { Val = 0D };
            NumberItem numberItem4 = new NumberItem() { Val = 4D };
            NumberItem numberItem5 = new NumberItem() { Val = 0D };
            NumberItem numberItem6 = new NumberItem() { Val = 0D };
            NumberItem numberItem7 = new NumberItem() { Val = 144D };
            NumberItem numberItem8 = new NumberItem() { Val = 0D };
            NumberItem numberItem9 = new NumberItem() { Val = 3000D };
            NumberItem numberItem10 = new NumberItem() { Val = 0D };
            NumberItem numberItem11 = new NumberItem() { Val = 0D };

            pivotCacheRecord1.Append(fieldItem4);
            pivotCacheRecord1.Append(fieldItem5);
            pivotCacheRecord1.Append(numberItem2);
            pivotCacheRecord1.Append(numberItem3);
            pivotCacheRecord1.Append(numberItem4);
            pivotCacheRecord1.Append(numberItem5);
            pivotCacheRecord1.Append(numberItem6);
            pivotCacheRecord1.Append(numberItem7);
            pivotCacheRecord1.Append(numberItem8);
            pivotCacheRecord1.Append(numberItem9);
            pivotCacheRecord1.Append(numberItem10);
            pivotCacheRecord1.Append(numberItem11);

            PivotCacheRecord pivotCacheRecord2 = new PivotCacheRecord();
            FieldItem fieldItem6 = new FieldItem() { Val = (UInt32Value)1U };
            FieldItem fieldItem7 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem12 = new NumberItem() { Val = 0D };
            NumberItem numberItem13 = new NumberItem() { Val = 0D };
            NumberItem numberItem14 = new NumberItem() { Val = 0D };
            NumberItem numberItem15 = new NumberItem() { Val = 0D };
            NumberItem numberItem16 = new NumberItem() { Val = 0D };
            NumberItem numberItem17 = new NumberItem() { Val = 0D };
            NumberItem numberItem18 = new NumberItem() { Val = 0D };
            NumberItem numberItem19 = new NumberItem() { Val = 0D };
            NumberItem numberItem20 = new NumberItem() { Val = 0D };
            NumberItem numberItem21 = new NumberItem() { Val = 0D };

            pivotCacheRecord2.Append(fieldItem6);
            pivotCacheRecord2.Append(fieldItem7);
            pivotCacheRecord2.Append(numberItem12);
            pivotCacheRecord2.Append(numberItem13);
            pivotCacheRecord2.Append(numberItem14);
            pivotCacheRecord2.Append(numberItem15);
            pivotCacheRecord2.Append(numberItem16);
            pivotCacheRecord2.Append(numberItem17);
            pivotCacheRecord2.Append(numberItem18);
            pivotCacheRecord2.Append(numberItem19);
            pivotCacheRecord2.Append(numberItem20);
            pivotCacheRecord2.Append(numberItem21);

            PivotCacheRecord pivotCacheRecord3 = new PivotCacheRecord();
            FieldItem fieldItem8 = new FieldItem() { Val = (UInt32Value)2U };
            FieldItem fieldItem9 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem22 = new NumberItem() { Val = 0D };
            NumberItem numberItem23 = new NumberItem() { Val = 0D };
            NumberItem numberItem24 = new NumberItem() { Val = 0D };
            NumberItem numberItem25 = new NumberItem() { Val = 0D };
            NumberItem numberItem26 = new NumberItem() { Val = 0D };
            NumberItem numberItem27 = new NumberItem() { Val = 0D };
            NumberItem numberItem28 = new NumberItem() { Val = 0D };
            NumberItem numberItem29 = new NumberItem() { Val = 0D };
            NumberItem numberItem30 = new NumberItem() { Val = 0D };
            NumberItem numberItem31 = new NumberItem() { Val = 0D };

            pivotCacheRecord3.Append(fieldItem8);
            pivotCacheRecord3.Append(fieldItem9);
            pivotCacheRecord3.Append(numberItem22);
            pivotCacheRecord3.Append(numberItem23);
            pivotCacheRecord3.Append(numberItem24);
            pivotCacheRecord3.Append(numberItem25);
            pivotCacheRecord3.Append(numberItem26);
            pivotCacheRecord3.Append(numberItem27);
            pivotCacheRecord3.Append(numberItem28);
            pivotCacheRecord3.Append(numberItem29);
            pivotCacheRecord3.Append(numberItem30);
            pivotCacheRecord3.Append(numberItem31);

            PivotCacheRecord pivotCacheRecord4 = new PivotCacheRecord();
            FieldItem fieldItem10 = new FieldItem() { Val = (UInt32Value)3U };
            FieldItem fieldItem11 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem32 = new NumberItem() { Val = 0D };
            NumberItem numberItem33 = new NumberItem() { Val = 6D };
            NumberItem numberItem34 = new NumberItem() { Val = 0D };
            NumberItem numberItem35 = new NumberItem() { Val = 5D };
            NumberItem numberItem36 = new NumberItem() { Val = 130D };
            NumberItem numberItem37 = new NumberItem() { Val = 130D };
            NumberItem numberItem38 = new NumberItem() { Val = 0D };
            NumberItem numberItem39 = new NumberItem() { Val = 10000D };
            NumberItem numberItem40 = new NumberItem() { Val = 0D };
            NumberItem numberItem41 = new NumberItem() { Val = 1D };

            pivotCacheRecord4.Append(fieldItem10);
            pivotCacheRecord4.Append(fieldItem11);
            pivotCacheRecord4.Append(numberItem32);
            pivotCacheRecord4.Append(numberItem33);
            pivotCacheRecord4.Append(numberItem34);
            pivotCacheRecord4.Append(numberItem35);
            pivotCacheRecord4.Append(numberItem36);
            pivotCacheRecord4.Append(numberItem37);
            pivotCacheRecord4.Append(numberItem38);
            pivotCacheRecord4.Append(numberItem39);
            pivotCacheRecord4.Append(numberItem40);
            pivotCacheRecord4.Append(numberItem41);

            PivotCacheRecord pivotCacheRecord5 = new PivotCacheRecord();
            FieldItem fieldItem12 = new FieldItem() { Val = (UInt32Value)4U };
            FieldItem fieldItem13 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem42 = new NumberItem() { Val = 2D };
            NumberItem numberItem43 = new NumberItem() { Val = 0D };
            NumberItem numberItem44 = new NumberItem() { Val = 2D };
            NumberItem numberItem45 = new NumberItem() { Val = 0D };
            NumberItem numberItem46 = new NumberItem() { Val = 0D };
            NumberItem numberItem47 = new NumberItem() { Val = 72D };
            NumberItem numberItem48 = new NumberItem() { Val = 0D };
            NumberItem numberItem49 = new NumberItem() { Val = 3000D };
            NumberItem numberItem50 = new NumberItem() { Val = 0D };
            NumberItem numberItem51 = new NumberItem() { Val = 0D };

            pivotCacheRecord5.Append(fieldItem12);
            pivotCacheRecord5.Append(fieldItem13);
            pivotCacheRecord5.Append(numberItem42);
            pivotCacheRecord5.Append(numberItem43);
            pivotCacheRecord5.Append(numberItem44);
            pivotCacheRecord5.Append(numberItem45);
            pivotCacheRecord5.Append(numberItem46);
            pivotCacheRecord5.Append(numberItem47);
            pivotCacheRecord5.Append(numberItem48);
            pivotCacheRecord5.Append(numberItem49);
            pivotCacheRecord5.Append(numberItem50);
            pivotCacheRecord5.Append(numberItem51);

            PivotCacheRecord pivotCacheRecord6 = new PivotCacheRecord();
            FieldItem fieldItem14 = new FieldItem() { Val = (UInt32Value)5U };
            FieldItem fieldItem15 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem52 = new NumberItem() { Val = 5D };
            NumberItem numberItem53 = new NumberItem() { Val = 0D };
            NumberItem numberItem54 = new NumberItem() { Val = 4D };
            NumberItem numberItem55 = new NumberItem() { Val = 0D };
            NumberItem numberItem56 = new NumberItem() { Val = 0D };
            NumberItem numberItem57 = new NumberItem() { Val = 144D };
            NumberItem numberItem58 = new NumberItem() { Val = 0D };
            NumberItem numberItem59 = new NumberItem() { Val = 0D };
            NumberItem numberItem60 = new NumberItem() { Val = 1D };
            NumberItem numberItem61 = new NumberItem() { Val = 0D };

            pivotCacheRecord6.Append(fieldItem14);
            pivotCacheRecord6.Append(fieldItem15);
            pivotCacheRecord6.Append(numberItem52);
            pivotCacheRecord6.Append(numberItem53);
            pivotCacheRecord6.Append(numberItem54);
            pivotCacheRecord6.Append(numberItem55);
            pivotCacheRecord6.Append(numberItem56);
            pivotCacheRecord6.Append(numberItem57);
            pivotCacheRecord6.Append(numberItem58);
            pivotCacheRecord6.Append(numberItem59);
            pivotCacheRecord6.Append(numberItem60);
            pivotCacheRecord6.Append(numberItem61);

            PivotCacheRecord pivotCacheRecord7 = new PivotCacheRecord();
            FieldItem fieldItem16 = new FieldItem() { Val = (UInt32Value)6U };
            FieldItem fieldItem17 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem62 = new NumberItem() { Val = 2D };
            NumberItem numberItem63 = new NumberItem() { Val = 0D };
            NumberItem numberItem64 = new NumberItem() { Val = 2D };
            NumberItem numberItem65 = new NumberItem() { Val = 0D };
            NumberItem numberItem66 = new NumberItem() { Val = 0D };
            NumberItem numberItem67 = new NumberItem() { Val = 48D };
            NumberItem numberItem68 = new NumberItem() { Val = 0D };
            NumberItem numberItem69 = new NumberItem() { Val = 5000D };
            NumberItem numberItem70 = new NumberItem() { Val = 0D };
            NumberItem numberItem71 = new NumberItem() { Val = 0D };

            pivotCacheRecord7.Append(fieldItem16);
            pivotCacheRecord7.Append(fieldItem17);
            pivotCacheRecord7.Append(numberItem62);
            pivotCacheRecord7.Append(numberItem63);
            pivotCacheRecord7.Append(numberItem64);
            pivotCacheRecord7.Append(numberItem65);
            pivotCacheRecord7.Append(numberItem66);
            pivotCacheRecord7.Append(numberItem67);
            pivotCacheRecord7.Append(numberItem68);
            pivotCacheRecord7.Append(numberItem69);
            pivotCacheRecord7.Append(numberItem70);
            pivotCacheRecord7.Append(numberItem71);

            PivotCacheRecord pivotCacheRecord8 = new PivotCacheRecord();
            FieldItem fieldItem18 = new FieldItem() { Val = (UInt32Value)7U };
            FieldItem fieldItem19 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem72 = new NumberItem() { Val = 0D };
            NumberItem numberItem73 = new NumberItem() { Val = 0D };
            NumberItem numberItem74 = new NumberItem() { Val = 0D };
            NumberItem numberItem75 = new NumberItem() { Val = 0D };
            NumberItem numberItem76 = new NumberItem() { Val = 0D };
            NumberItem numberItem77 = new NumberItem() { Val = 0D };
            NumberItem numberItem78 = new NumberItem() { Val = 0D };
            NumberItem numberItem79 = new NumberItem() { Val = 1000D };
            NumberItem numberItem80 = new NumberItem() { Val = 0D };
            NumberItem numberItem81 = new NumberItem() { Val = 0D };

            pivotCacheRecord8.Append(fieldItem18);
            pivotCacheRecord8.Append(fieldItem19);
            pivotCacheRecord8.Append(numberItem72);
            pivotCacheRecord8.Append(numberItem73);
            pivotCacheRecord8.Append(numberItem74);
            pivotCacheRecord8.Append(numberItem75);
            pivotCacheRecord8.Append(numberItem76);
            pivotCacheRecord8.Append(numberItem77);
            pivotCacheRecord8.Append(numberItem78);
            pivotCacheRecord8.Append(numberItem79);
            pivotCacheRecord8.Append(numberItem80);
            pivotCacheRecord8.Append(numberItem81);

            PivotCacheRecord pivotCacheRecord9 = new PivotCacheRecord();
            FieldItem fieldItem20 = new FieldItem() { Val = (UInt32Value)8U };
            FieldItem fieldItem21 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem82 = new NumberItem() { Val = 2D };
            NumberItem numberItem83 = new NumberItem() { Val = 0D };
            NumberItem numberItem84 = new NumberItem() { Val = 2D };
            NumberItem numberItem85 = new NumberItem() { Val = 0D };
            NumberItem numberItem86 = new NumberItem() { Val = 0D };
            NumberItem numberItem87 = new NumberItem() { Val = 72D };
            NumberItem numberItem88 = new NumberItem() { Val = 0D };
            NumberItem numberItem89 = new NumberItem() { Val = 10000D };
            NumberItem numberItem90 = new NumberItem() { Val = 0D };
            NumberItem numberItem91 = new NumberItem() { Val = 0D };

            pivotCacheRecord9.Append(fieldItem20);
            pivotCacheRecord9.Append(fieldItem21);
            pivotCacheRecord9.Append(numberItem82);
            pivotCacheRecord9.Append(numberItem83);
            pivotCacheRecord9.Append(numberItem84);
            pivotCacheRecord9.Append(numberItem85);
            pivotCacheRecord9.Append(numberItem86);
            pivotCacheRecord9.Append(numberItem87);
            pivotCacheRecord9.Append(numberItem88);
            pivotCacheRecord9.Append(numberItem89);
            pivotCacheRecord9.Append(numberItem90);
            pivotCacheRecord9.Append(numberItem91);

            PivotCacheRecord pivotCacheRecord10 = new PivotCacheRecord();
            FieldItem fieldItem22 = new FieldItem() { Val = (UInt32Value)9U };
            FieldItem fieldItem23 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem92 = new NumberItem() { Val = 9D };
            NumberItem numberItem93 = new NumberItem() { Val = 0D };
            NumberItem numberItem94 = new NumberItem() { Val = 7D };
            NumberItem numberItem95 = new NumberItem() { Val = 0D };
            NumberItem numberItem96 = new NumberItem() { Val = 0D };
            NumberItem numberItem97 = new NumberItem() { Val = 216D };
            NumberItem numberItem98 = new NumberItem() { Val = 0D };
            NumberItem numberItem99 = new NumberItem() { Val = 10000D };
            NumberItem numberItem100 = new NumberItem() { Val = 2D };
            NumberItem numberItem101 = new NumberItem() { Val = 0D };

            pivotCacheRecord10.Append(fieldItem22);
            pivotCacheRecord10.Append(fieldItem23);
            pivotCacheRecord10.Append(numberItem92);
            pivotCacheRecord10.Append(numberItem93);
            pivotCacheRecord10.Append(numberItem94);
            pivotCacheRecord10.Append(numberItem95);
            pivotCacheRecord10.Append(numberItem96);
            pivotCacheRecord10.Append(numberItem97);
            pivotCacheRecord10.Append(numberItem98);
            pivotCacheRecord10.Append(numberItem99);
            pivotCacheRecord10.Append(numberItem100);
            pivotCacheRecord10.Append(numberItem101);

            PivotCacheRecord pivotCacheRecord11 = new PivotCacheRecord();
            FieldItem fieldItem24 = new FieldItem() { Val = (UInt32Value)10U };
            FieldItem fieldItem25 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem102 = new NumberItem() { Val = 3D };
            NumberItem numberItem103 = new NumberItem() { Val = 0D };
            NumberItem numberItem104 = new NumberItem() { Val = 3D };
            NumberItem numberItem105 = new NumberItem() { Val = 0D };
            NumberItem numberItem106 = new NumberItem() { Val = 0D };
            NumberItem numberItem107 = new NumberItem() { Val = 96D };
            NumberItem numberItem108 = new NumberItem() { Val = 0D };
            NumberItem numberItem109 = new NumberItem() { Val = 0D };
            NumberItem numberItem110 = new NumberItem() { Val = 0D };
            NumberItem numberItem111 = new NumberItem() { Val = 0D };

            pivotCacheRecord11.Append(fieldItem24);
            pivotCacheRecord11.Append(fieldItem25);
            pivotCacheRecord11.Append(numberItem102);
            pivotCacheRecord11.Append(numberItem103);
            pivotCacheRecord11.Append(numberItem104);
            pivotCacheRecord11.Append(numberItem105);
            pivotCacheRecord11.Append(numberItem106);
            pivotCacheRecord11.Append(numberItem107);
            pivotCacheRecord11.Append(numberItem108);
            pivotCacheRecord11.Append(numberItem109);
            pivotCacheRecord11.Append(numberItem110);
            pivotCacheRecord11.Append(numberItem111);

            PivotCacheRecord pivotCacheRecord12 = new PivotCacheRecord();
            FieldItem fieldItem26 = new FieldItem() { Val = (UInt32Value)8U };
            FieldItem fieldItem27 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem112 = new NumberItem() { Val = 3D };
            NumberItem numberItem113 = new NumberItem() { Val = 0D };
            NumberItem numberItem114 = new NumberItem() { Val = 3D };
            NumberItem numberItem115 = new NumberItem() { Val = 0D };
            NumberItem numberItem116 = new NumberItem() { Val = 0D };
            NumberItem numberItem117 = new NumberItem() { Val = 96D };
            NumberItem numberItem118 = new NumberItem() { Val = 0D };
            NumberItem numberItem119 = new NumberItem() { Val = 2000D };
            NumberItem numberItem120 = new NumberItem() { Val = 0D };
            NumberItem numberItem121 = new NumberItem() { Val = 0D };

            pivotCacheRecord12.Append(fieldItem26);
            pivotCacheRecord12.Append(fieldItem27);
            pivotCacheRecord12.Append(numberItem112);
            pivotCacheRecord12.Append(numberItem113);
            pivotCacheRecord12.Append(numberItem114);
            pivotCacheRecord12.Append(numberItem115);
            pivotCacheRecord12.Append(numberItem116);
            pivotCacheRecord12.Append(numberItem117);
            pivotCacheRecord12.Append(numberItem118);
            pivotCacheRecord12.Append(numberItem119);
            pivotCacheRecord12.Append(numberItem120);
            pivotCacheRecord12.Append(numberItem121);

            PivotCacheRecord pivotCacheRecord13 = new PivotCacheRecord();
            FieldItem fieldItem28 = new FieldItem() { Val = (UInt32Value)4U };
            FieldItem fieldItem29 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem122 = new NumberItem() { Val = 10D };
            NumberItem numberItem123 = new NumberItem() { Val = 0D };
            NumberItem numberItem124 = new NumberItem() { Val = 9D };
            NumberItem numberItem125 = new NumberItem() { Val = 0D };
            NumberItem numberItem126 = new NumberItem() { Val = 0D };
            NumberItem numberItem127 = new NumberItem() { Val = 288D };
            NumberItem numberItem128 = new NumberItem() { Val = 0D };
            NumberItem numberItem129 = new NumberItem() { Val = 10000D };
            NumberItem numberItem130 = new NumberItem() { Val = 1D };
            NumberItem numberItem131 = new NumberItem() { Val = 0D };

            pivotCacheRecord13.Append(fieldItem28);
            pivotCacheRecord13.Append(fieldItem29);
            pivotCacheRecord13.Append(numberItem122);
            pivotCacheRecord13.Append(numberItem123);
            pivotCacheRecord13.Append(numberItem124);
            pivotCacheRecord13.Append(numberItem125);
            pivotCacheRecord13.Append(numberItem126);
            pivotCacheRecord13.Append(numberItem127);
            pivotCacheRecord13.Append(numberItem128);
            pivotCacheRecord13.Append(numberItem129);
            pivotCacheRecord13.Append(numberItem130);
            pivotCacheRecord13.Append(numberItem131);

            PivotCacheRecord pivotCacheRecord14 = new PivotCacheRecord();
            FieldItem fieldItem30 = new FieldItem() { Val = (UInt32Value)9U };
            FieldItem fieldItem31 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem132 = new NumberItem() { Val = 1D };
            NumberItem numberItem133 = new NumberItem() { Val = 0D };
            NumberItem numberItem134 = new NumberItem() { Val = 1D };
            NumberItem numberItem135 = new NumberItem() { Val = 0D };
            NumberItem numberItem136 = new NumberItem() { Val = 0D };
            NumberItem numberItem137 = new NumberItem() { Val = 24D };
            NumberItem numberItem138 = new NumberItem() { Val = 0D };
            NumberItem numberItem139 = new NumberItem() { Val = 1000D };
            NumberItem numberItem140 = new NumberItem() { Val = 0D };
            NumberItem numberItem141 = new NumberItem() { Val = 0D };

            pivotCacheRecord14.Append(fieldItem30);
            pivotCacheRecord14.Append(fieldItem31);
            pivotCacheRecord14.Append(numberItem132);
            pivotCacheRecord14.Append(numberItem133);
            pivotCacheRecord14.Append(numberItem134);
            pivotCacheRecord14.Append(numberItem135);
            pivotCacheRecord14.Append(numberItem136);
            pivotCacheRecord14.Append(numberItem137);
            pivotCacheRecord14.Append(numberItem138);
            pivotCacheRecord14.Append(numberItem139);
            pivotCacheRecord14.Append(numberItem140);
            pivotCacheRecord14.Append(numberItem141);

            PivotCacheRecord pivotCacheRecord15 = new PivotCacheRecord();
            FieldItem fieldItem32 = new FieldItem() { Val = (UInt32Value)11U };
            FieldItem fieldItem33 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem142 = new NumberItem() { Val = 2D };
            NumberItem numberItem143 = new NumberItem() { Val = 0D };
            NumberItem numberItem144 = new NumberItem() { Val = 2D };
            NumberItem numberItem145 = new NumberItem() { Val = 0D };
            NumberItem numberItem146 = new NumberItem() { Val = 0D };
            NumberItem numberItem147 = new NumberItem() { Val = 72D };
            NumberItem numberItem148 = new NumberItem() { Val = 0D };
            NumberItem numberItem149 = new NumberItem() { Val = 0D };
            NumberItem numberItem150 = new NumberItem() { Val = 0D };
            NumberItem numberItem151 = new NumberItem() { Val = 0D };

            pivotCacheRecord15.Append(fieldItem32);
            pivotCacheRecord15.Append(fieldItem33);
            pivotCacheRecord15.Append(numberItem142);
            pivotCacheRecord15.Append(numberItem143);
            pivotCacheRecord15.Append(numberItem144);
            pivotCacheRecord15.Append(numberItem145);
            pivotCacheRecord15.Append(numberItem146);
            pivotCacheRecord15.Append(numberItem147);
            pivotCacheRecord15.Append(numberItem148);
            pivotCacheRecord15.Append(numberItem149);
            pivotCacheRecord15.Append(numberItem150);
            pivotCacheRecord15.Append(numberItem151);

            PivotCacheRecord pivotCacheRecord16 = new PivotCacheRecord();
            FieldItem fieldItem34 = new FieldItem() { Val = (UInt32Value)12U };
            FieldItem fieldItem35 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem152 = new NumberItem() { Val = 12D };
            NumberItem numberItem153 = new NumberItem() { Val = 3D };
            NumberItem numberItem154 = new NumberItem() { Val = 10D };
            NumberItem numberItem155 = new NumberItem() { Val = 3D };
            NumberItem numberItem156 = new NumberItem() { Val = 361D };
            NumberItem numberItem157 = new NumberItem() { Val = 1225D };
            NumberItem numberItem158 = new NumberItem() { Val = 0D };
            NumberItem numberItem159 = new NumberItem() { Val = 10000D };
            NumberItem numberItem160 = new NumberItem() { Val = 2D };
            NumberItem numberItem161 = new NumberItem() { Val = 0D };

            pivotCacheRecord16.Append(fieldItem34);
            pivotCacheRecord16.Append(fieldItem35);
            pivotCacheRecord16.Append(numberItem152);
            pivotCacheRecord16.Append(numberItem153);
            pivotCacheRecord16.Append(numberItem154);
            pivotCacheRecord16.Append(numberItem155);
            pivotCacheRecord16.Append(numberItem156);
            pivotCacheRecord16.Append(numberItem157);
            pivotCacheRecord16.Append(numberItem158);
            pivotCacheRecord16.Append(numberItem159);
            pivotCacheRecord16.Append(numberItem160);
            pivotCacheRecord16.Append(numberItem161);

            PivotCacheRecord pivotCacheRecord17 = new PivotCacheRecord();
            FieldItem fieldItem36 = new FieldItem() { Val = (UInt32Value)13U };
            FieldItem fieldItem37 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem162 = new NumberItem() { Val = 10D };
            NumberItem numberItem163 = new NumberItem() { Val = 0D };
            NumberItem numberItem164 = new NumberItem() { Val = 9D };
            NumberItem numberItem165 = new NumberItem() { Val = 0D };
            NumberItem numberItem166 = new NumberItem() { Val = 0D };
            NumberItem numberItem167 = new NumberItem() { Val = 288D };
            NumberItem numberItem168 = new NumberItem() { Val = 0D };
            NumberItem numberItem169 = new NumberItem() { Val = 10000D };
            NumberItem numberItem170 = new NumberItem() { Val = 1D };
            NumberItem numberItem171 = new NumberItem() { Val = 0D };

            pivotCacheRecord17.Append(fieldItem36);
            pivotCacheRecord17.Append(fieldItem37);
            pivotCacheRecord17.Append(numberItem162);
            pivotCacheRecord17.Append(numberItem163);
            pivotCacheRecord17.Append(numberItem164);
            pivotCacheRecord17.Append(numberItem165);
            pivotCacheRecord17.Append(numberItem166);
            pivotCacheRecord17.Append(numberItem167);
            pivotCacheRecord17.Append(numberItem168);
            pivotCacheRecord17.Append(numberItem169);
            pivotCacheRecord17.Append(numberItem170);
            pivotCacheRecord17.Append(numberItem171);

            PivotCacheRecord pivotCacheRecord18 = new PivotCacheRecord();
            FieldItem fieldItem38 = new FieldItem() { Val = (UInt32Value)14U };
            FieldItem fieldItem39 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem172 = new NumberItem() { Val = 3D };
            NumberItem numberItem173 = new NumberItem() { Val = 0D };
            NumberItem numberItem174 = new NumberItem() { Val = 3D };
            NumberItem numberItem175 = new NumberItem() { Val = 0D };
            NumberItem numberItem176 = new NumberItem() { Val = 0D };
            NumberItem numberItem177 = new NumberItem() { Val = 195D };
            NumberItem numberItem178 = new NumberItem() { Val = 0D };
            NumberItem numberItem179 = new NumberItem() { Val = 30000D };
            NumberItem numberItem180 = new NumberItem() { Val = 0D };
            NumberItem numberItem181 = new NumberItem() { Val = 0D };

            pivotCacheRecord18.Append(fieldItem38);
            pivotCacheRecord18.Append(fieldItem39);
            pivotCacheRecord18.Append(numberItem172);
            pivotCacheRecord18.Append(numberItem173);
            pivotCacheRecord18.Append(numberItem174);
            pivotCacheRecord18.Append(numberItem175);
            pivotCacheRecord18.Append(numberItem176);
            pivotCacheRecord18.Append(numberItem177);
            pivotCacheRecord18.Append(numberItem178);
            pivotCacheRecord18.Append(numberItem179);
            pivotCacheRecord18.Append(numberItem180);
            pivotCacheRecord18.Append(numberItem181);

            PivotCacheRecord pivotCacheRecord19 = new PivotCacheRecord();
            FieldItem fieldItem40 = new FieldItem() { Val = (UInt32Value)3U };
            FieldItem fieldItem41 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem182 = new NumberItem() { Val = 10D };
            NumberItem numberItem183 = new NumberItem() { Val = 0D };
            NumberItem numberItem184 = new NumberItem() { Val = 8D };
            NumberItem numberItem185 = new NumberItem() { Val = 0D };
            NumberItem numberItem186 = new NumberItem() { Val = 0D };
            NumberItem numberItem187 = new NumberItem() { Val = 240D };
            NumberItem numberItem188 = new NumberItem() { Val = 0D };
            NumberItem numberItem189 = new NumberItem() { Val = 20000D };
            NumberItem numberItem190 = new NumberItem() { Val = 2D };
            NumberItem numberItem191 = new NumberItem() { Val = 0D };

            pivotCacheRecord19.Append(fieldItem40);
            pivotCacheRecord19.Append(fieldItem41);
            pivotCacheRecord19.Append(numberItem182);
            pivotCacheRecord19.Append(numberItem183);
            pivotCacheRecord19.Append(numberItem184);
            pivotCacheRecord19.Append(numberItem185);
            pivotCacheRecord19.Append(numberItem186);
            pivotCacheRecord19.Append(numberItem187);
            pivotCacheRecord19.Append(numberItem188);
            pivotCacheRecord19.Append(numberItem189);
            pivotCacheRecord19.Append(numberItem190);
            pivotCacheRecord19.Append(numberItem191);

            PivotCacheRecord pivotCacheRecord20 = new PivotCacheRecord();
            FieldItem fieldItem42 = new FieldItem() { Val = (UInt32Value)15U };
            FieldItem fieldItem43 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem192 = new NumberItem() { Val = 7D };
            NumberItem numberItem193 = new NumberItem() { Val = 0D };
            NumberItem numberItem194 = new NumberItem() { Val = 7D };
            NumberItem numberItem195 = new NumberItem() { Val = 0D };
            NumberItem numberItem196 = new NumberItem() { Val = 0D };
            NumberItem numberItem197 = new NumberItem() { Val = 219D };
            NumberItem numberItem198 = new NumberItem() { Val = 0D };
            NumberItem numberItem199 = new NumberItem() { Val = 20000D };
            NumberItem numberItem200 = new NumberItem() { Val = 0D };
            NumberItem numberItem201 = new NumberItem() { Val = 0D };

            pivotCacheRecord20.Append(fieldItem42);
            pivotCacheRecord20.Append(fieldItem43);
            pivotCacheRecord20.Append(numberItem192);
            pivotCacheRecord20.Append(numberItem193);
            pivotCacheRecord20.Append(numberItem194);
            pivotCacheRecord20.Append(numberItem195);
            pivotCacheRecord20.Append(numberItem196);
            pivotCacheRecord20.Append(numberItem197);
            pivotCacheRecord20.Append(numberItem198);
            pivotCacheRecord20.Append(numberItem199);
            pivotCacheRecord20.Append(numberItem200);
            pivotCacheRecord20.Append(numberItem201);

            PivotCacheRecord pivotCacheRecord21 = new PivotCacheRecord();
            FieldItem fieldItem44 = new FieldItem() { Val = (UInt32Value)16U };
            FieldItem fieldItem45 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem202 = new NumberItem() { Val = 7D };
            NumberItem numberItem203 = new NumberItem() { Val = 0D };
            NumberItem numberItem204 = new NumberItem() { Val = 7D };
            NumberItem numberItem205 = new NumberItem() { Val = 0D };
            NumberItem numberItem206 = new NumberItem() { Val = 0D };
            NumberItem numberItem207 = new NumberItem() { Val = 240D };
            NumberItem numberItem208 = new NumberItem() { Val = 0D };
            NumberItem numberItem209 = new NumberItem() { Val = 5000D };
            NumberItem numberItem210 = new NumberItem() { Val = 0D };
            NumberItem numberItem211 = new NumberItem() { Val = 0D };

            pivotCacheRecord21.Append(fieldItem44);
            pivotCacheRecord21.Append(fieldItem45);
            pivotCacheRecord21.Append(numberItem202);
            pivotCacheRecord21.Append(numberItem203);
            pivotCacheRecord21.Append(numberItem204);
            pivotCacheRecord21.Append(numberItem205);
            pivotCacheRecord21.Append(numberItem206);
            pivotCacheRecord21.Append(numberItem207);
            pivotCacheRecord21.Append(numberItem208);
            pivotCacheRecord21.Append(numberItem209);
            pivotCacheRecord21.Append(numberItem210);
            pivotCacheRecord21.Append(numberItem211);

            PivotCacheRecord pivotCacheRecord22 = new PivotCacheRecord();
            FieldItem fieldItem46 = new FieldItem() { Val = (UInt32Value)17U };
            FieldItem fieldItem47 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem212 = new NumberItem() { Val = 4D };
            NumberItem numberItem213 = new NumberItem() { Val = 0D };
            NumberItem numberItem214 = new NumberItem() { Val = 4D };
            NumberItem numberItem215 = new NumberItem() { Val = 0D };
            NumberItem numberItem216 = new NumberItem() { Val = 0D };
            NumberItem numberItem217 = new NumberItem() { Val = 120D };
            NumberItem numberItem218 = new NumberItem() { Val = 0D };
            NumberItem numberItem219 = new NumberItem() { Val = 10000D };
            NumberItem numberItem220 = new NumberItem() { Val = 0D };
            NumberItem numberItem221 = new NumberItem() { Val = 0D };

            pivotCacheRecord22.Append(fieldItem46);
            pivotCacheRecord22.Append(fieldItem47);
            pivotCacheRecord22.Append(numberItem212);
            pivotCacheRecord22.Append(numberItem213);
            pivotCacheRecord22.Append(numberItem214);
            pivotCacheRecord22.Append(numberItem215);
            pivotCacheRecord22.Append(numberItem216);
            pivotCacheRecord22.Append(numberItem217);
            pivotCacheRecord22.Append(numberItem218);
            pivotCacheRecord22.Append(numberItem219);
            pivotCacheRecord22.Append(numberItem220);
            pivotCacheRecord22.Append(numberItem221);

            PivotCacheRecord pivotCacheRecord23 = new PivotCacheRecord();
            FieldItem fieldItem48 = new FieldItem() { Val = (UInt32Value)18U };
            FieldItem fieldItem49 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem222 = new NumberItem() { Val = 0D };
            NumberItem numberItem223 = new NumberItem() { Val = 0D };
            NumberItem numberItem224 = new NumberItem() { Val = 0D };
            NumberItem numberItem225 = new NumberItem() { Val = 0D };
            NumberItem numberItem226 = new NumberItem() { Val = 0D };
            NumberItem numberItem227 = new NumberItem() { Val = 0D };
            NumberItem numberItem228 = new NumberItem() { Val = 0D };
            NumberItem numberItem229 = new NumberItem() { Val = 0D };
            NumberItem numberItem230 = new NumberItem() { Val = 0D };
            NumberItem numberItem231 = new NumberItem() { Val = 0D };

            pivotCacheRecord23.Append(fieldItem48);
            pivotCacheRecord23.Append(fieldItem49);
            pivotCacheRecord23.Append(numberItem222);
            pivotCacheRecord23.Append(numberItem223);
            pivotCacheRecord23.Append(numberItem224);
            pivotCacheRecord23.Append(numberItem225);
            pivotCacheRecord23.Append(numberItem226);
            pivotCacheRecord23.Append(numberItem227);
            pivotCacheRecord23.Append(numberItem228);
            pivotCacheRecord23.Append(numberItem229);
            pivotCacheRecord23.Append(numberItem230);
            pivotCacheRecord23.Append(numberItem231);

            PivotCacheRecord pivotCacheRecord24 = new PivotCacheRecord();
            FieldItem fieldItem50 = new FieldItem() { Val = (UInt32Value)19U };
            FieldItem fieldItem51 = new FieldItem() { Val = (UInt32Value)0U };
            NumberItem numberItem232 = new NumberItem() { Val = 1D };
            NumberItem numberItem233 = new NumberItem() { Val = 0D };
            NumberItem numberItem234 = new NumberItem() { Val = 1D };
            NumberItem numberItem235 = new NumberItem() { Val = 0D };
            NumberItem numberItem236 = new NumberItem() { Val = 0D };
            NumberItem numberItem237 = new NumberItem() { Val = 24D };
            NumberItem numberItem238 = new NumberItem() { Val = 0D };
            NumberItem numberItem239 = new NumberItem() { Val = 4000D };
            NumberItem numberItem240 = new NumberItem() { Val = 0D };
            NumberItem numberItem241 = new NumberItem() { Val = 0D };

            pivotCacheRecord24.Append(fieldItem50);
            pivotCacheRecord24.Append(fieldItem51);
            pivotCacheRecord24.Append(numberItem232);
            pivotCacheRecord24.Append(numberItem233);
            pivotCacheRecord24.Append(numberItem234);
            pivotCacheRecord24.Append(numberItem235);
            pivotCacheRecord24.Append(numberItem236);
            pivotCacheRecord24.Append(numberItem237);
            pivotCacheRecord24.Append(numberItem238);
            pivotCacheRecord24.Append(numberItem239);
            pivotCacheRecord24.Append(numberItem240);
            pivotCacheRecord24.Append(numberItem241);

            pivotCacheRecords1.Append(pivotCacheRecord1);
            pivotCacheRecords1.Append(pivotCacheRecord2);
            pivotCacheRecords1.Append(pivotCacheRecord3);
            pivotCacheRecords1.Append(pivotCacheRecord4);
            pivotCacheRecords1.Append(pivotCacheRecord5);
            pivotCacheRecords1.Append(pivotCacheRecord6);
            pivotCacheRecords1.Append(pivotCacheRecord7);
            pivotCacheRecords1.Append(pivotCacheRecord8);
            pivotCacheRecords1.Append(pivotCacheRecord9);
            pivotCacheRecords1.Append(pivotCacheRecord10);
            pivotCacheRecords1.Append(pivotCacheRecord11);
            pivotCacheRecords1.Append(pivotCacheRecord12);
            pivotCacheRecords1.Append(pivotCacheRecord13);
            pivotCacheRecords1.Append(pivotCacheRecord14);
            pivotCacheRecords1.Append(pivotCacheRecord15);
            pivotCacheRecords1.Append(pivotCacheRecord16);
            pivotCacheRecords1.Append(pivotCacheRecord17);
            pivotCacheRecords1.Append(pivotCacheRecord18);
            pivotCacheRecords1.Append(pivotCacheRecord19);
            pivotCacheRecords1.Append(pivotCacheRecord20);
            pivotCacheRecords1.Append(pivotCacheRecord21);
            pivotCacheRecords1.Append(pivotCacheRecord22);
            pivotCacheRecords1.Append(pivotCacheRecord23);
            pivotCacheRecords1.Append(pivotCacheRecord24);

            pivotTableCacheRecordsPart1.PivotCacheRecords = pivotCacheRecords1;
        }

        // Generates content of worksheetPart2.
        private void GenerateWorksheetPart2Content(WorksheetPart worksheetPart2)
        {
            Worksheet worksheet2 = new Worksheet() { MCAttributes = new MarkupCompatibilityAttributes() { Ignorable = "x14ac" } };
            worksheet2.AddNamespaceDeclaration("r", "http://schemas.openxmlformats.org/officeDocument/2006/relationships");
            worksheet2.AddNamespaceDeclaration("mc", "http://schemas.openxmlformats.org/markup-compatibility/2006");
            worksheet2.AddNamespaceDeclaration("x14ac", "http://schemas.microsoft.com/office/spreadsheetml/2009/9/ac");
            SheetDimension sheetDimension2 = new SheetDimension() { Reference = "A1:C24" };

            SheetViews sheetViews2 = new SheetViews();

            SheetView sheetView2 = new SheetView() { WorkbookViewId = (UInt32Value)0U };
            Selection selection2 = new Selection() { ActiveCell = "E55", SequenceOfReferences = new ListValue<StringValue>() { InnerText = "E55" } };

            sheetView2.Append(selection2);

            sheetViews2.Append(sheetView2);
            SheetFormatProperties sheetFormatProperties2 = new SheetFormatProperties() { DefaultRowHeight = 15D, DyDescent = 0.25D };

            Columns columns2 = new Columns();
            Column column3 = new Column() { Min = (UInt32Value)1U, Max = (UInt32Value)1U, Width = 27.28515625D, BestFit = true, CustomWidth = true };
            Column column4 = new Column() { Min = (UInt32Value)2U, Max = (UInt32Value)2U, Width = 10.140625D, BestFit = true, CustomWidth = true };
            Column column5 = new Column() { Min = (UInt32Value)3U, Max = (UInt32Value)3U, Width = 19.5703125D, BestFit = true, CustomWidth = true };

            columns2.Append(column3);
            columns2.Append(column4);
            columns2.Append(column5);

            SheetData sheetData2 = new SheetData();

            Row row24 = new Row() { RowIndex = (UInt32Value)1U, Spans = new ListValue<StringValue>() { InnerText = "1:3" }, DyDescent = 0.25D };

            Cell cell69 = new Cell() { CellReference = "A1", StyleIndex = (UInt32Value)1U, DataType = CellValues.SharedString };
            CellValue cellValue69 = new CellValue();
            cellValue69.Text = "1";

            cell69.Append(cellValue69);

            Cell cell70 = new Cell() { CellReference = "B1", DataType = CellValues.SharedString };
            CellValue cellValue70 = new CellValue();
            cellValue70.Text = "61";

            cell70.Append(cellValue70);

            row24.Append(cell69);
            row24.Append(cell70);

            sheetData2.Append(row24);
            PageMargins pageMargins3 = new PageMargins() { Left = 0.7D, Right = 0.7D, Top = 0.75D, Bottom = 0.75D, Header = 0.3D, Footer = 0.3D };
            Drawing drawing2 = new Drawing() { Id = "rId2" };

            worksheet2.Append(sheetDimension2);
            worksheet2.Append(sheetViews2);
            worksheet2.Append(sheetFormatProperties2);
            worksheet2.Append(columns2);
            worksheet2.Append(sheetData2);
            worksheet2.Append(pageMargins3);
            worksheet2.Append(drawing2);

            worksheetPart2.Worksheet = worksheet2;
        }

        // Generates content of drawingsPart2.
        private void GenerateDrawingsPart2Content(DrawingsPart drawingsPart2)
        {
            Xdr.WorksheetDrawing worksheetDrawing2 = new Xdr.WorksheetDrawing();
            worksheetDrawing2.AddNamespaceDeclaration("xdr", "http://schemas.openxmlformats.org/drawingml/2006/spreadsheetDrawing");
            worksheetDrawing2.AddNamespaceDeclaration("a", "http://schemas.openxmlformats.org/drawingml/2006/main");

            Xdr.TwoCellAnchor twoCellAnchor2 = new Xdr.TwoCellAnchor();

            Xdr.FromMarker fromMarker2 = new Xdr.FromMarker();
            Xdr.ColumnId columnId3 = new Xdr.ColumnId();
            columnId3.Text = "0";
            Xdr.ColumnOffset columnOffset3 = new Xdr.ColumnOffset();
            columnOffset3.Text = "0";
            Xdr.RowId rowId3 = new Xdr.RowId();
            rowId3.Text = "26";
            Xdr.RowOffset rowOffset3 = new Xdr.RowOffset();
            rowOffset3.Text = "9525";

            fromMarker2.Append(columnId3);
            fromMarker2.Append(columnOffset3);
            fromMarker2.Append(rowId3);
            fromMarker2.Append(rowOffset3);

            Xdr.ToMarker toMarker2 = new Xdr.ToMarker();
            Xdr.ColumnId columnId4 = new Xdr.ColumnId();
            columnId4.Text = "8";
            Xdr.ColumnOffset columnOffset4 = new Xdr.ColumnOffset();
            columnOffset4.Text = "19049";
            Xdr.RowId rowId4 = new Xdr.RowId();
            rowId4.Text = "50";
            Xdr.RowOffset rowOffset4 = new Xdr.RowOffset();
            rowOffset4.Text = "85725";

            toMarker2.Append(columnId4);
            toMarker2.Append(columnOffset4);
            toMarker2.Append(rowId4);
            toMarker2.Append(rowOffset4);

            Xdr.GraphicFrame graphicFrame2 = new Xdr.GraphicFrame() { Macro = "" };

            Xdr.NonVisualGraphicFrameProperties nonVisualGraphicFrameProperties2 = new Xdr.NonVisualGraphicFrameProperties();
            Xdr.NonVisualDrawingProperties nonVisualDrawingProperties2 = new Xdr.NonVisualDrawingProperties() { Id = (UInt32Value)2U, Name = "Chart 1" };
            Xdr.NonVisualGraphicFrameDrawingProperties nonVisualGraphicFrameDrawingProperties2 = new Xdr.NonVisualGraphicFrameDrawingProperties();

            nonVisualGraphicFrameProperties2.Append(nonVisualDrawingProperties2);
            nonVisualGraphicFrameProperties2.Append(nonVisualGraphicFrameDrawingProperties2);

            Xdr.Transform transform2 = new Xdr.Transform();
            A.Offset offset2 = new A.Offset() { X = 0L, Y = 0L };
            A.Extents extents2 = new A.Extents() { Cx = 0L, Cy = 0L };

            transform2.Append(offset2);
            transform2.Append(extents2);

            A.Graphic graphic2 = new A.Graphic();

            A.GraphicData graphicData2 = new A.GraphicData() { Uri = "http://schemas.openxmlformats.org/drawingml/2006/chart" };

            C.ChartReference chartReference2 = new C.ChartReference() { Id = "rId1" };
            chartReference2.AddNamespaceDeclaration("c", "http://schemas.openxmlformats.org/drawingml/2006/chart");
            chartReference2.AddNamespaceDeclaration("r", "http://schemas.openxmlformats.org/officeDocument/2006/relationships");

            graphicData2.Append(chartReference2);

            graphic2.Append(graphicData2);

            graphicFrame2.Append(nonVisualGraphicFrameProperties2);
            graphicFrame2.Append(transform2);
            graphicFrame2.Append(graphic2);
            Xdr.ClientData clientData2 = new Xdr.ClientData();

            twoCellAnchor2.Append(fromMarker2);
            twoCellAnchor2.Append(toMarker2);
            twoCellAnchor2.Append(graphicFrame2);
            twoCellAnchor2.Append(clientData2);

            worksheetDrawing2.Append(twoCellAnchor2);

            drawingsPart2.WorksheetDrawing = worksheetDrawing2;
        }

        // Generates content of chartPart2.
        private void GenerateChartPart2Content(ChartPart chartPart2)
        {
            C.ChartSpace chartSpace2 = new C.ChartSpace();
            chartSpace2.AddNamespaceDeclaration("c", "http://schemas.openxmlformats.org/drawingml/2006/chart");
            chartSpace2.AddNamespaceDeclaration("a", "http://schemas.openxmlformats.org/drawingml/2006/main");
            chartSpace2.AddNamespaceDeclaration("r", "http://schemas.openxmlformats.org/officeDocument/2006/relationships");
            C.Date1904 date19042 = new C.Date1904() { Val = false };
            C.EditingLanguage editingLanguage2 = new C.EditingLanguage() { Val = "en-US" };
            C.RoundedCorners roundedCorners2 = new C.RoundedCorners() { Val = false };

            AlternateContent alternateContent2 = new AlternateContent();
            alternateContent2.AddNamespaceDeclaration("mc", "http://schemas.openxmlformats.org/markup-compatibility/2006");

            AlternateContentChoice alternateContentChoice2 = new AlternateContentChoice() { Requires = "c14" };
            alternateContentChoice2.AddNamespaceDeclaration("c14", "http://schemas.microsoft.com/office/drawing/2007/8/2/chart");
            C14.Style style3 = new C14.Style() { Val = 102 };

            alternateContentChoice2.Append(style3);

            AlternateContentFallback alternateContentFallback2 = new AlternateContentFallback();
            C.Style style4 = new C.Style() { Val = 2 };

            alternateContentFallback2.Append(style4);

            alternateContent2.Append(alternateContentChoice2);
            alternateContent2.Append(alternateContentFallback2);

            C.PivotSource pivotSource2 = new C.PivotSource();
            C.PivotTableName pivotTableName2 = new C.PivotTableName();
            pivotTableName2.Text = "需求商业价值分析!PivotTable3";
            C.FormatId formatId2 = new C.FormatId() { Val = (UInt32Value)0U };

            pivotSource2.Append(pivotTableName2);
            pivotSource2.Append(formatId2);

            C.Chart chart2 = new C.Chart();

            C.Title title2 = new C.Title();

            C.ChartText chartText2 = new C.ChartText();

            C.RichText richText2 = new C.RichText();
            A.BodyProperties bodyProperties2 = new A.BodyProperties();
            A.ListStyle listStyle2 = new A.ListStyle();

            A.Paragraph paragraph2 = new A.Paragraph();

            A.ParagraphProperties paragraphProperties2 = new A.ParagraphProperties();
            A.DefaultRunProperties defaultRunProperties2 = new A.DefaultRunProperties();

            paragraphProperties2.Append(defaultRunProperties2);

            A.Run run2 = new A.Run();
            A.RunProperties runProperties2 = new A.RunProperties() { Language = "zh-CN", AlternativeLanguage = "en-US" };
            A.Text text2 = new A.Text();
            text2.Text = "需求商业价值分析";

            run2.Append(runProperties2);
            run2.Append(text2);
            A.EndParagraphRunProperties endParagraphRunProperties2 = new A.EndParagraphRunProperties() { Language = "en-US" };

            paragraph2.Append(paragraphProperties2);
            paragraph2.Append(run2);
            paragraph2.Append(endParagraphRunProperties2);

            richText2.Append(bodyProperties2);
            richText2.Append(listStyle2);
            richText2.Append(paragraph2);

            chartText2.Append(richText2);
            C.Layout layout4 = new C.Layout();
            C.Overlay overlay3 = new C.Overlay() { Val = false };

            title2.Append(chartText2);
            title2.Append(layout4);
            title2.Append(overlay3);
            C.AutoTitleDeleted autoTitleDeleted2 = new C.AutoTitleDeleted() { Val = false };

            C.PivotFormats pivotFormats2 = new C.PivotFormats();

            C.PivotFormat pivotFormat3 = new C.PivotFormat();
            C.Index index5 = new C.Index() { Val = (UInt32Value)0U };

            C.Marker marker3 = new C.Marker();
            C.Symbol symbol3 = new C.Symbol() { Val = C.MarkerStyleValues.None };

            marker3.Append(symbol3);

            pivotFormat3.Append(index5);
            pivotFormat3.Append(marker3);

            C.PivotFormat pivotFormat4 = new C.PivotFormat();
            C.Index index6 = new C.Index() { Val = (UInt32Value)1U };

            C.Marker marker4 = new C.Marker();
            C.Symbol symbol4 = new C.Symbol() { Val = C.MarkerStyleValues.None };

            marker4.Append(symbol4);

            pivotFormat4.Append(index6);
            pivotFormat4.Append(marker4);

            pivotFormats2.Append(pivotFormat3);
            pivotFormats2.Append(pivotFormat4);

            C.PlotArea plotArea2 = new C.PlotArea();
            C.Layout layout5 = new C.Layout();

            C.BarChart barChart2 = new C.BarChart();
            C.BarDirection barDirection2 = new C.BarDirection() { Val = C.BarDirectionValues.Column };
            C.BarGrouping barGrouping2 = new C.BarGrouping() { Val = C.BarGroupingValues.Clustered };
            C.VaryColors varyColors2 = new C.VaryColors() { Val = false };

            C.BarChartSeries barChartSeries3 = new C.BarChartSeries();
            C.Index index7 = new C.Index() { Val = (UInt32Value)0U };
            C.Order order3 = new C.Order() { Val = (UInt32Value)0U };

            C.SeriesText seriesText3 = new C.SeriesText();

            C.StringReference stringReference5 = new C.StringReference();
            C.Formula formula7 = new C.Formula();
            formula7.Text = "需求商业价值分析!$B$3";

            C.StringCache stringCache5 = new C.StringCache();
            C.PointCount pointCount7 = new C.PointCount() { Val = (UInt32Value)1U };

            C.StringPoint stringPoint43 = new C.StringPoint() { Index = (UInt32Value)0U };
            C.NumericValue numericValue83 = new C.NumericValue();
            numericValue83.Text = "挣值总计";

            stringPoint43.Append(numericValue83);

            stringCache5.Append(pointCount7);
            stringCache5.Append(stringPoint43);

            stringReference5.Append(formula7);
            stringReference5.Append(stringCache5);

            seriesText3.Append(stringReference5);
            C.InvertIfNegative invertIfNegative3 = new C.InvertIfNegative() { Val = false };

            C.CategoryAxisData categoryAxisData3 = new C.CategoryAxisData();

            C.StringReference stringReference6 = new C.StringReference();
            C.Formula formula8 = new C.Formula();
            formula8.Text = "需求商业价值分析!$A$4:$A$24";

            C.StringCache stringCache6 = new C.StringCache();
            C.PointCount pointCount8 = new C.PointCount() { Val = (UInt32Value)20U };

            C.StringPoint stringPoint44 = new C.StringPoint() { Index = (UInt32Value)0U };
            C.NumericValue numericValue84 = new C.NumericValue();
            numericValue84.Text = "统计功能";

            stringPoint44.Append(numericValue84);

            C.StringPoint stringPoint45 = new C.StringPoint() { Index = (UInt32Value)1U };
            C.NumericValue numericValue85 = new C.NumericValue();
            numericValue85.Text = "整合配置管理系统";

            stringPoint45.Append(numericValue85);

            C.StringPoint stringPoint46 = new C.StringPoint() { Index = (UInt32Value)2U };
            C.NumericValue numericValue86 = new C.NumericValue();
            numericValue86.Text = "决策过程跟踪";

            stringPoint46.Append(numericValue86);

            C.StringPoint stringPoint47 = new C.StringPoint() { Index = (UInt32Value)3U };
            C.NumericValue numericValue87 = new C.NumericValue();
            numericValue87.Text = "集成团队管理";

            stringPoint47.Append(numericValue87);

            C.StringPoint stringPoint48 = new C.StringPoint() { Index = (UInt32Value)4U };
            C.NumericValue numericValue88 = new C.NumericValue();
            numericValue88.Text = "用户身份验证";

            stringPoint48.Append(numericValue88);

            C.StringPoint stringPoint49 = new C.StringPoint() { Index = (UInt32Value)5U };
            C.NumericValue numericValue89 = new C.NumericValue();
            numericValue89.Text = "全文检索";

            stringPoint49.Append(numericValue89);

            C.StringPoint stringPoint50 = new C.StringPoint() { Index = (UInt32Value)6U };
            C.NumericValue numericValue90 = new C.NumericValue();
            numericValue90.Text = "需求管理中的进度跟踪";

            stringPoint50.Append(numericValue90);

            C.StringPoint stringPoint51 = new C.StringPoint() { Index = (UInt32Value)7U };
            C.NumericValue numericValue91 = new C.NumericValue();
            numericValue91.Text = "需求管理基本功能";

            stringPoint51.Append(numericValue91);

            C.StringPoint stringPoint52 = new C.StringPoint() { Index = (UInt32Value)8U };
            C.NumericValue numericValue92 = new C.NumericValue();
            numericValue92.Text = "权限管理";

            stringPoint52.Append(numericValue92);

            C.StringPoint stringPoint53 = new C.StringPoint() { Index = (UInt32Value)9U };
            C.NumericValue numericValue93 = new C.NumericValue();
            numericValue93.Text = "流程管理";

            stringPoint53.Append(numericValue93);

            C.StringPoint stringPoint54 = new C.StringPoint() { Index = (UInt32Value)10U };
            C.NumericValue numericValue94 = new C.NumericValue();
            numericValue94.Text = "风险管理";

            stringPoint54.Append(numericValue94);

            C.StringPoint stringPoint55 = new C.StringPoint() { Index = (UInt32Value)11U };
            C.NumericValue numericValue95 = new C.NumericValue();
            numericValue95.Text = "支持国际化";

            stringPoint55.Append(numericValue95);

            C.StringPoint stringPoint56 = new C.StringPoint() { Index = (UInt32Value)12U };
            C.NumericValue numericValue96 = new C.NumericValue();
            numericValue96.Text = "任务标签";

            stringPoint56.Append(numericValue96);

            C.StringPoint stringPoint57 = new C.StringPoint() { Index = (UInt32Value)13U };
            C.NumericValue numericValue97 = new C.NumericValue();
            numericValue97.Text = "添加CAPTCHA支持";

            stringPoint57.Append(numericValue97);

            C.StringPoint stringPoint58 = new C.StringPoint() { Index = (UInt32Value)14U };
            C.NumericValue numericValue98 = new C.NumericValue();
            numericValue98.Text = "团队时间管理";

            stringPoint58.Append(numericValue98);

            C.StringPoint stringPoint59 = new C.StringPoint() { Index = (UInt32Value)15U };
            C.NumericValue numericValue99 = new C.NumericValue();
            numericValue99.Text = "实现通用的分页显示功能";

            stringPoint59.Append(numericValue99);

            C.StringPoint stringPoint60 = new C.StringPoint() { Index = (UInt32Value)16U };
            C.NumericValue numericValue100 = new C.NumericValue();
            numericValue100.Text = "会议管理";

            stringPoint60.Append(numericValue100);

            C.StringPoint stringPoint61 = new C.StringPoint() { Index = (UInt32Value)17U };
            C.NumericValue numericValue101 = new C.NumericValue();
            numericValue101.Text = "事故管理";

            stringPoint61.Append(numericValue101);

            C.StringPoint stringPoint62 = new C.StringPoint() { Index = (UInt32Value)18U };
            C.NumericValue numericValue102 = new C.NumericValue();
            numericValue102.Text = "元素之间的相互转化";

            stringPoint62.Append(numericValue102);

            C.StringPoint stringPoint63 = new C.StringPoint() { Index = (UInt32Value)19U };
            C.NumericValue numericValue103 = new C.NumericValue();
            numericValue103.Text = "用户选择托管配置管理系统";

            stringPoint63.Append(numericValue103);

            stringCache6.Append(pointCount8);
            stringCache6.Append(stringPoint44);
            stringCache6.Append(stringPoint45);
            stringCache6.Append(stringPoint46);
            stringCache6.Append(stringPoint47);
            stringCache6.Append(stringPoint48);
            stringCache6.Append(stringPoint49);
            stringCache6.Append(stringPoint50);
            stringCache6.Append(stringPoint51);
            stringCache6.Append(stringPoint52);
            stringCache6.Append(stringPoint53);
            stringCache6.Append(stringPoint54);
            stringCache6.Append(stringPoint55);
            stringCache6.Append(stringPoint56);
            stringCache6.Append(stringPoint57);
            stringCache6.Append(stringPoint58);
            stringCache6.Append(stringPoint59);
            stringCache6.Append(stringPoint60);
            stringCache6.Append(stringPoint61);
            stringCache6.Append(stringPoint62);
            stringCache6.Append(stringPoint63);

            stringReference6.Append(formula8);
            stringReference6.Append(stringCache6);

            categoryAxisData3.Append(stringReference6);

            C.Values values3 = new C.Values();

            C.NumberReference numberReference3 = new C.NumberReference();
            C.Formula formula9 = new C.Formula();
            formula9.Text = "需求商业价值分析!$B$4:$B$24";

            C.NumberingCache numberingCache3 = new C.NumberingCache();
            C.FormatCode formatCode3 = new C.FormatCode();
            formatCode3.Text = "General";
            C.PointCount pointCount9 = new C.PointCount() { Val = (UInt32Value)20U };

            C.NumericPoint numericPoint41 = new C.NumericPoint() { Index = (UInt32Value)0U };
            C.NumericValue numericValue104 = new C.NumericValue();
            numericValue104.Text = "0";

            numericPoint41.Append(numericValue104);

            C.NumericPoint numericPoint42 = new C.NumericPoint() { Index = (UInt32Value)1U };
            C.NumericValue numericValue105 = new C.NumericValue();
            numericValue105.Text = "0";

            numericPoint42.Append(numericValue105);

            C.NumericPoint numericPoint43 = new C.NumericPoint() { Index = (UInt32Value)2U };
            C.NumericValue numericValue106 = new C.NumericValue();
            numericValue106.Text = "0";

            numericPoint43.Append(numericValue106);

            C.NumericPoint numericPoint44 = new C.NumericPoint() { Index = (UInt32Value)3U };
            C.NumericValue numericValue107 = new C.NumericValue();
            numericValue107.Text = "0";

            numericPoint44.Append(numericValue107);

            C.NumericPoint numericPoint45 = new C.NumericPoint() { Index = (UInt32Value)4U };
            C.NumericValue numericValue108 = new C.NumericValue();
            numericValue108.Text = "0";

            numericPoint45.Append(numericValue108);

            C.NumericPoint numericPoint46 = new C.NumericPoint() { Index = (UInt32Value)5U };
            C.NumericValue numericValue109 = new C.NumericValue();
            numericValue109.Text = "0";

            numericPoint46.Append(numericValue109);

            C.NumericPoint numericPoint47 = new C.NumericPoint() { Index = (UInt32Value)6U };
            C.NumericValue numericValue110 = new C.NumericValue();
            numericValue110.Text = "0";

            numericPoint47.Append(numericValue110);

            C.NumericPoint numericPoint48 = new C.NumericPoint() { Index = (UInt32Value)7U };
            C.NumericValue numericValue111 = new C.NumericValue();
            numericValue111.Text = "0";

            numericPoint48.Append(numericValue111);

            C.NumericPoint numericPoint49 = new C.NumericPoint() { Index = (UInt32Value)8U };
            C.NumericValue numericValue112 = new C.NumericValue();
            numericValue112.Text = "0";

            numericPoint49.Append(numericValue112);

            C.NumericPoint numericPoint50 = new C.NumericPoint() { Index = (UInt32Value)9U };
            C.NumericValue numericValue113 = new C.NumericValue();
            numericValue113.Text = "0";

            numericPoint50.Append(numericValue113);

            C.NumericPoint numericPoint51 = new C.NumericPoint() { Index = (UInt32Value)10U };
            C.NumericValue numericValue114 = new C.NumericValue();
            numericValue114.Text = "0";

            numericPoint51.Append(numericValue114);

            C.NumericPoint numericPoint52 = new C.NumericPoint() { Index = (UInt32Value)11U };
            C.NumericValue numericValue115 = new C.NumericValue();
            numericValue115.Text = "0";

            numericPoint52.Append(numericValue115);

            C.NumericPoint numericPoint53 = new C.NumericPoint() { Index = (UInt32Value)12U };
            C.NumericValue numericValue116 = new C.NumericValue();
            numericValue116.Text = "0";

            numericPoint53.Append(numericValue116);

            C.NumericPoint numericPoint54 = new C.NumericPoint() { Index = (UInt32Value)13U };
            C.NumericValue numericValue117 = new C.NumericValue();
            numericValue117.Text = "0";

            numericPoint54.Append(numericValue117);

            C.NumericPoint numericPoint55 = new C.NumericPoint() { Index = (UInt32Value)14U };
            C.NumericValue numericValue118 = new C.NumericValue();
            numericValue118.Text = "0";

            numericPoint55.Append(numericValue118);

            C.NumericPoint numericPoint56 = new C.NumericPoint() { Index = (UInt32Value)15U };
            C.NumericValue numericValue119 = new C.NumericValue();
            numericValue119.Text = "0";

            numericPoint56.Append(numericValue119);

            C.NumericPoint numericPoint57 = new C.NumericPoint() { Index = (UInt32Value)16U };
            C.NumericValue numericValue120 = new C.NumericValue();
            numericValue120.Text = "0";

            numericPoint57.Append(numericValue120);

            C.NumericPoint numericPoint58 = new C.NumericPoint() { Index = (UInt32Value)17U };
            C.NumericValue numericValue121 = new C.NumericValue();
            numericValue121.Text = "0";

            numericPoint58.Append(numericValue121);

            C.NumericPoint numericPoint59 = new C.NumericPoint() { Index = (UInt32Value)18U };
            C.NumericValue numericValue122 = new C.NumericValue();
            numericValue122.Text = "0";

            numericPoint59.Append(numericValue122);

            C.NumericPoint numericPoint60 = new C.NumericPoint() { Index = (UInt32Value)19U };
            C.NumericValue numericValue123 = new C.NumericValue();
            numericValue123.Text = "0";

            numericPoint60.Append(numericValue123);

            numberingCache3.Append(formatCode3);
            numberingCache3.Append(pointCount9);
            numberingCache3.Append(numericPoint41);
            numberingCache3.Append(numericPoint42);
            numberingCache3.Append(numericPoint43);
            numberingCache3.Append(numericPoint44);
            numberingCache3.Append(numericPoint45);
            numberingCache3.Append(numericPoint46);
            numberingCache3.Append(numericPoint47);
            numberingCache3.Append(numericPoint48);
            numberingCache3.Append(numericPoint49);
            numberingCache3.Append(numericPoint50);
            numberingCache3.Append(numericPoint51);
            numberingCache3.Append(numericPoint52);
            numberingCache3.Append(numericPoint53);
            numberingCache3.Append(numericPoint54);
            numberingCache3.Append(numericPoint55);
            numberingCache3.Append(numericPoint56);
            numberingCache3.Append(numericPoint57);
            numberingCache3.Append(numericPoint58);
            numberingCache3.Append(numericPoint59);
            numberingCache3.Append(numericPoint60);

            numberReference3.Append(formula9);
            numberReference3.Append(numberingCache3);

            values3.Append(numberReference3);

            barChartSeries3.Append(index7);
            barChartSeries3.Append(order3);
            barChartSeries3.Append(seriesText3);
            barChartSeries3.Append(invertIfNegative3);
            barChartSeries3.Append(categoryAxisData3);
            barChartSeries3.Append(values3);

            C.BarChartSeries barChartSeries4 = new C.BarChartSeries();
            C.Index index8 = new C.Index() { Val = (UInt32Value)1U };
            C.Order order4 = new C.Order() { Val = (UInt32Value)1U };

            C.SeriesText seriesText4 = new C.SeriesText();

            C.StringReference stringReference7 = new C.StringReference();
            C.Formula formula10 = new C.Formula();
            formula10.Text = "需求商业价值分析!$C$3";

            C.StringCache stringCache7 = new C.StringCache();
            C.PointCount pointCount10 = new C.PointCount() { Val = (UInt32Value)1U };

            C.StringPoint stringPoint64 = new C.StringPoint() { Index = (UInt32Value)0U };
            C.NumericValue numericValue124 = new C.NumericValue();
            numericValue124.Text = "需求商业价值总计";

            stringPoint64.Append(numericValue124);

            stringCache7.Append(pointCount10);
            stringCache7.Append(stringPoint64);

            stringReference7.Append(formula10);
            stringReference7.Append(stringCache7);

            seriesText4.Append(stringReference7);
            C.InvertIfNegative invertIfNegative4 = new C.InvertIfNegative() { Val = false };

            C.CategoryAxisData categoryAxisData4 = new C.CategoryAxisData();

            C.StringReference stringReference8 = new C.StringReference();
            C.Formula formula11 = new C.Formula();
            formula11.Text = "需求商业价值分析!$A$4:$A$24";

            C.StringCache stringCache8 = new C.StringCache();
            C.PointCount pointCount11 = new C.PointCount() { Val = (UInt32Value)20U };

            C.StringPoint stringPoint65 = new C.StringPoint() { Index = (UInt32Value)0U };
            C.NumericValue numericValue125 = new C.NumericValue();
            numericValue125.Text = "统计功能";

            stringPoint65.Append(numericValue125);

            C.StringPoint stringPoint66 = new C.StringPoint() { Index = (UInt32Value)1U };
            C.NumericValue numericValue126 = new C.NumericValue();
            numericValue126.Text = "整合配置管理系统";

            stringPoint66.Append(numericValue126);

            C.StringPoint stringPoint67 = new C.StringPoint() { Index = (UInt32Value)2U };
            C.NumericValue numericValue127 = new C.NumericValue();
            numericValue127.Text = "决策过程跟踪";

            stringPoint67.Append(numericValue127);

            C.StringPoint stringPoint68 = new C.StringPoint() { Index = (UInt32Value)3U };
            C.NumericValue numericValue128 = new C.NumericValue();
            numericValue128.Text = "集成团队管理";

            stringPoint68.Append(numericValue128);

            C.StringPoint stringPoint69 = new C.StringPoint() { Index = (UInt32Value)4U };
            C.NumericValue numericValue129 = new C.NumericValue();
            numericValue129.Text = "用户身份验证";

            stringPoint69.Append(numericValue129);

            C.StringPoint stringPoint70 = new C.StringPoint() { Index = (UInt32Value)5U };
            C.NumericValue numericValue130 = new C.NumericValue();
            numericValue130.Text = "全文检索";

            stringPoint70.Append(numericValue130);

            C.StringPoint stringPoint71 = new C.StringPoint() { Index = (UInt32Value)6U };
            C.NumericValue numericValue131 = new C.NumericValue();
            numericValue131.Text = "需求管理中的进度跟踪";

            stringPoint71.Append(numericValue131);

            C.StringPoint stringPoint72 = new C.StringPoint() { Index = (UInt32Value)7U };
            C.NumericValue numericValue132 = new C.NumericValue();
            numericValue132.Text = "需求管理基本功能";

            stringPoint72.Append(numericValue132);

            C.StringPoint stringPoint73 = new C.StringPoint() { Index = (UInt32Value)8U };
            C.NumericValue numericValue133 = new C.NumericValue();
            numericValue133.Text = "权限管理";

            stringPoint73.Append(numericValue133);

            C.StringPoint stringPoint74 = new C.StringPoint() { Index = (UInt32Value)9U };
            C.NumericValue numericValue134 = new C.NumericValue();
            numericValue134.Text = "流程管理";

            stringPoint74.Append(numericValue134);

            C.StringPoint stringPoint75 = new C.StringPoint() { Index = (UInt32Value)10U };
            C.NumericValue numericValue135 = new C.NumericValue();
            numericValue135.Text = "风险管理";

            stringPoint75.Append(numericValue135);

            C.StringPoint stringPoint76 = new C.StringPoint() { Index = (UInt32Value)11U };
            C.NumericValue numericValue136 = new C.NumericValue();
            numericValue136.Text = "支持国际化";

            stringPoint76.Append(numericValue136);

            C.StringPoint stringPoint77 = new C.StringPoint() { Index = (UInt32Value)12U };
            C.NumericValue numericValue137 = new C.NumericValue();
            numericValue137.Text = "任务标签";

            stringPoint77.Append(numericValue137);

            C.StringPoint stringPoint78 = new C.StringPoint() { Index = (UInt32Value)13U };
            C.NumericValue numericValue138 = new C.NumericValue();
            numericValue138.Text = "添加CAPTCHA支持";

            stringPoint78.Append(numericValue138);

            C.StringPoint stringPoint79 = new C.StringPoint() { Index = (UInt32Value)14U };
            C.NumericValue numericValue139 = new C.NumericValue();
            numericValue139.Text = "团队时间管理";

            stringPoint79.Append(numericValue139);

            C.StringPoint stringPoint80 = new C.StringPoint() { Index = (UInt32Value)15U };
            C.NumericValue numericValue140 = new C.NumericValue();
            numericValue140.Text = "实现通用的分页显示功能";

            stringPoint80.Append(numericValue140);

            C.StringPoint stringPoint81 = new C.StringPoint() { Index = (UInt32Value)16U };
            C.NumericValue numericValue141 = new C.NumericValue();
            numericValue141.Text = "会议管理";

            stringPoint81.Append(numericValue141);

            C.StringPoint stringPoint82 = new C.StringPoint() { Index = (UInt32Value)17U };
            C.NumericValue numericValue142 = new C.NumericValue();
            numericValue142.Text = "事故管理";

            stringPoint82.Append(numericValue142);

            C.StringPoint stringPoint83 = new C.StringPoint() { Index = (UInt32Value)18U };
            C.NumericValue numericValue143 = new C.NumericValue();
            numericValue143.Text = "元素之间的相互转化";

            stringPoint83.Append(numericValue143);

            C.StringPoint stringPoint84 = new C.StringPoint() { Index = (UInt32Value)19U };
            C.NumericValue numericValue144 = new C.NumericValue();
            numericValue144.Text = "用户选择托管配置管理系统";

            stringPoint84.Append(numericValue144);

            stringCache8.Append(pointCount11);
            stringCache8.Append(stringPoint65);
            stringCache8.Append(stringPoint66);
            stringCache8.Append(stringPoint67);
            stringCache8.Append(stringPoint68);
            stringCache8.Append(stringPoint69);
            stringCache8.Append(stringPoint70);
            stringCache8.Append(stringPoint71);
            stringCache8.Append(stringPoint72);
            stringCache8.Append(stringPoint73);
            stringCache8.Append(stringPoint74);
            stringCache8.Append(stringPoint75);
            stringCache8.Append(stringPoint76);
            stringCache8.Append(stringPoint77);
            stringCache8.Append(stringPoint78);
            stringCache8.Append(stringPoint79);
            stringCache8.Append(stringPoint80);
            stringCache8.Append(stringPoint81);
            stringCache8.Append(stringPoint82);
            stringCache8.Append(stringPoint83);
            stringCache8.Append(stringPoint84);

            stringReference8.Append(formula11);
            stringReference8.Append(stringCache8);

            categoryAxisData4.Append(stringReference8);

            C.Values values4 = new C.Values();

            C.NumberReference numberReference4 = new C.NumberReference();
            C.Formula formula12 = new C.Formula();
            formula12.Text = "需求商业价值分析!$C$4:$C$24";

            C.NumberingCache numberingCache4 = new C.NumberingCache();
            C.FormatCode formatCode4 = new C.FormatCode();
            formatCode4.Text = "General";
            C.PointCount pointCount12 = new C.PointCount() { Val = (UInt32Value)20U };

            C.NumericPoint numericPoint61 = new C.NumericPoint() { Index = (UInt32Value)0U };
            C.NumericValue numericValue145 = new C.NumericValue();
            numericValue145.Text = "30000";

            numericPoint61.Append(numericValue145);

            C.NumericPoint numericPoint62 = new C.NumericPoint() { Index = (UInt32Value)1U };
            C.NumericValue numericValue146 = new C.NumericValue();
            numericValue146.Text = "30000";

            numericPoint62.Append(numericValue146);

            C.NumericPoint numericPoint63 = new C.NumericPoint() { Index = (UInt32Value)2U };
            C.NumericValue numericValue147 = new C.NumericValue();
            numericValue147.Text = "20000";

            numericPoint63.Append(numericValue147);

            C.NumericPoint numericPoint64 = new C.NumericPoint() { Index = (UInt32Value)3U };
            C.NumericValue numericValue148 = new C.NumericValue();
            numericValue148.Text = "13000";

            numericPoint64.Append(numericValue148);

            C.NumericPoint numericPoint65 = new C.NumericPoint() { Index = (UInt32Value)4U };
            C.NumericValue numericValue149 = new C.NumericValue();
            numericValue149.Text = "12000";

            numericPoint65.Append(numericValue149);

            C.NumericPoint numericPoint66 = new C.NumericPoint() { Index = (UInt32Value)5U };
            C.NumericValue numericValue150 = new C.NumericValue();
            numericValue150.Text = "11000";

            numericPoint66.Append(numericValue150);

            C.NumericPoint numericPoint67 = new C.NumericPoint() { Index = (UInt32Value)6U };
            C.NumericValue numericValue151 = new C.NumericValue();
            numericValue151.Text = "10000";

            numericPoint67.Append(numericValue151);

            C.NumericPoint numericPoint68 = new C.NumericPoint() { Index = (UInt32Value)7U };
            C.NumericValue numericValue152 = new C.NumericValue();
            numericValue152.Text = "10000";

            numericPoint68.Append(numericValue152);

            C.NumericPoint numericPoint69 = new C.NumericPoint() { Index = (UInt32Value)8U };
            C.NumericValue numericValue153 = new C.NumericValue();
            numericValue153.Text = "10000";

            numericPoint69.Append(numericValue153);

            C.NumericPoint numericPoint70 = new C.NumericPoint() { Index = (UInt32Value)9U };
            C.NumericValue numericValue154 = new C.NumericValue();
            numericValue154.Text = "5000";

            numericPoint70.Append(numericValue154);

            C.NumericPoint numericPoint71 = new C.NumericPoint() { Index = (UInt32Value)10U };
            C.NumericValue numericValue155 = new C.NumericValue();
            numericValue155.Text = "5000";

            numericPoint71.Append(numericValue155);

            C.NumericPoint numericPoint72 = new C.NumericPoint() { Index = (UInt32Value)11U };
            C.NumericValue numericValue156 = new C.NumericValue();
            numericValue156.Text = "4000";

            numericPoint72.Append(numericValue156);

            C.NumericPoint numericPoint73 = new C.NumericPoint() { Index = (UInt32Value)12U };
            C.NumericValue numericValue157 = new C.NumericValue();
            numericValue157.Text = "3000";

            numericPoint73.Append(numericValue157);

            C.NumericPoint numericPoint74 = new C.NumericPoint() { Index = (UInt32Value)13U };
            C.NumericValue numericValue158 = new C.NumericValue();
            numericValue158.Text = "1000";

            numericPoint74.Append(numericValue158);

            C.NumericPoint numericPoint75 = new C.NumericPoint() { Index = (UInt32Value)14U };
            C.NumericValue numericValue159 = new C.NumericValue();
            numericValue159.Text = "0";

            numericPoint75.Append(numericValue159);

            C.NumericPoint numericPoint76 = new C.NumericPoint() { Index = (UInt32Value)15U };
            C.NumericValue numericValue160 = new C.NumericValue();
            numericValue160.Text = "0";

            numericPoint76.Append(numericValue160);

            C.NumericPoint numericPoint77 = new C.NumericPoint() { Index = (UInt32Value)16U };
            C.NumericValue numericValue161 = new C.NumericValue();
            numericValue161.Text = "0";

            numericPoint77.Append(numericValue161);

            C.NumericPoint numericPoint78 = new C.NumericPoint() { Index = (UInt32Value)17U };
            C.NumericValue numericValue162 = new C.NumericValue();
            numericValue162.Text = "0";

            numericPoint78.Append(numericValue162);

            C.NumericPoint numericPoint79 = new C.NumericPoint() { Index = (UInt32Value)18U };
            C.NumericValue numericValue163 = new C.NumericValue();
            numericValue163.Text = "0";

            numericPoint79.Append(numericValue163);

            C.NumericPoint numericPoint80 = new C.NumericPoint() { Index = (UInt32Value)19U };
            C.NumericValue numericValue164 = new C.NumericValue();
            numericValue164.Text = "0";

            numericPoint80.Append(numericValue164);

            numberingCache4.Append(formatCode4);
            numberingCache4.Append(pointCount12);
            numberingCache4.Append(numericPoint61);
            numberingCache4.Append(numericPoint62);
            numberingCache4.Append(numericPoint63);
            numberingCache4.Append(numericPoint64);
            numberingCache4.Append(numericPoint65);
            numberingCache4.Append(numericPoint66);
            numberingCache4.Append(numericPoint67);
            numberingCache4.Append(numericPoint68);
            numberingCache4.Append(numericPoint69);
            numberingCache4.Append(numericPoint70);
            numberingCache4.Append(numericPoint71);
            numberingCache4.Append(numericPoint72);
            numberingCache4.Append(numericPoint73);
            numberingCache4.Append(numericPoint74);
            numberingCache4.Append(numericPoint75);
            numberingCache4.Append(numericPoint76);
            numberingCache4.Append(numericPoint77);
            numberingCache4.Append(numericPoint78);
            numberingCache4.Append(numericPoint79);
            numberingCache4.Append(numericPoint80);

            numberReference4.Append(formula12);
            numberReference4.Append(numberingCache4);

            values4.Append(numberReference4);

            barChartSeries4.Append(index8);
            barChartSeries4.Append(order4);
            barChartSeries4.Append(seriesText4);
            barChartSeries4.Append(invertIfNegative4);
            barChartSeries4.Append(categoryAxisData4);
            barChartSeries4.Append(values4);

            C.DataLabels dataLabels2 = new C.DataLabels();
            C.ShowLegendKey showLegendKey2 = new C.ShowLegendKey() { Val = false };
            C.ShowValue showValue2 = new C.ShowValue() { Val = false };
            C.ShowCategoryName showCategoryName2 = new C.ShowCategoryName() { Val = false };
            C.ShowSeriesName showSeriesName2 = new C.ShowSeriesName() { Val = false };
            C.ShowPercent showPercent2 = new C.ShowPercent() { Val = false };
            C.ShowBubbleSize showBubbleSize2 = new C.ShowBubbleSize() { Val = false };

            dataLabels2.Append(showLegendKey2);
            dataLabels2.Append(showValue2);
            dataLabels2.Append(showCategoryName2);
            dataLabels2.Append(showSeriesName2);
            dataLabels2.Append(showPercent2);
            dataLabels2.Append(showBubbleSize2);
            C.GapWidth gapWidth2 = new C.GapWidth() { Val = (UInt16Value)75U };
            C.Overlap overlap2 = new C.Overlap() { Val = -25 };
            C.AxisId axisId5 = new C.AxisId() { Val = (UInt32Value)279044480U };
            C.AxisId axisId6 = new C.AxisId() { Val = (UInt32Value)279046016U };

            barChart2.Append(barDirection2);
            barChart2.Append(barGrouping2);
            barChart2.Append(varyColors2);
            barChart2.Append(barChartSeries3);
            barChart2.Append(barChartSeries4);
            barChart2.Append(dataLabels2);
            barChart2.Append(gapWidth2);
            barChart2.Append(overlap2);
            barChart2.Append(axisId5);
            barChart2.Append(axisId6);

            C.CategoryAxis categoryAxis2 = new C.CategoryAxis();
            C.AxisId axisId7 = new C.AxisId() { Val = (UInt32Value)279044480U };

            C.Scaling scaling3 = new C.Scaling();
            C.Orientation orientation3 = new C.Orientation() { Val = C.OrientationValues.MinMax };

            scaling3.Append(orientation3);
            C.Delete delete3 = new C.Delete() { Val = false };
            C.AxisPosition axisPosition3 = new C.AxisPosition() { Val = C.AxisPositionValues.Bottom };
            C.MajorTickMark majorTickMark3 = new C.MajorTickMark() { Val = C.TickMarkValues.None };
            C.MinorTickMark minorTickMark3 = new C.MinorTickMark() { Val = C.TickMarkValues.None };
            C.TickLabelPosition tickLabelPosition3 = new C.TickLabelPosition() { Val = C.TickLabelPositionValues.NextTo };
            C.CrossingAxis crossingAxis3 = new C.CrossingAxis() { Val = (UInt32Value)279046016U };
            C.Crosses crosses3 = new C.Crosses() { Val = C.CrossesValues.AutoZero };
            C.AutoLabeled autoLabeled2 = new C.AutoLabeled() { Val = true };
            C.LabelAlignment labelAlignment2 = new C.LabelAlignment() { Val = C.LabelAlignmentValues.Center };
            C.LabelOffset labelOffset2 = new C.LabelOffset() { Val = (UInt16Value)100U };
            C.NoMultiLevelLabels noMultiLevelLabels2 = new C.NoMultiLevelLabels() { Val = false };

            categoryAxis2.Append(axisId7);
            categoryAxis2.Append(scaling3);
            categoryAxis2.Append(delete3);
            categoryAxis2.Append(axisPosition3);
            categoryAxis2.Append(majorTickMark3);
            categoryAxis2.Append(minorTickMark3);
            categoryAxis2.Append(tickLabelPosition3);
            categoryAxis2.Append(crossingAxis3);
            categoryAxis2.Append(crosses3);
            categoryAxis2.Append(autoLabeled2);
            categoryAxis2.Append(labelAlignment2);
            categoryAxis2.Append(labelOffset2);
            categoryAxis2.Append(noMultiLevelLabels2);

            C.ValueAxis valueAxis2 = new C.ValueAxis();
            C.AxisId axisId8 = new C.AxisId() { Val = (UInt32Value)279046016U };

            C.Scaling scaling4 = new C.Scaling();
            C.Orientation orientation4 = new C.Orientation() { Val = C.OrientationValues.MinMax };

            scaling4.Append(orientation4);
            C.Delete delete4 = new C.Delete() { Val = false };
            C.AxisPosition axisPosition4 = new C.AxisPosition() { Val = C.AxisPositionValues.Left };
            C.MajorGridlines majorGridlines2 = new C.MajorGridlines();
            C.NumberingFormat numberingFormat2 = new C.NumberingFormat() { FormatCode = "General", SourceLinked = true };
            C.MajorTickMark majorTickMark4 = new C.MajorTickMark() { Val = C.TickMarkValues.None };
            C.MinorTickMark minorTickMark4 = new C.MinorTickMark() { Val = C.TickMarkValues.None };
            C.TickLabelPosition tickLabelPosition4 = new C.TickLabelPosition() { Val = C.TickLabelPositionValues.NextTo };

            C.ChartShapeProperties chartShapeProperties2 = new C.ChartShapeProperties();

            A.Outline outline2 = new A.Outline() { Width = 9525 };
            A.NoFill noFill2 = new A.NoFill();

            outline2.Append(noFill2);

            chartShapeProperties2.Append(outline2);
            C.CrossingAxis crossingAxis4 = new C.CrossingAxis() { Val = (UInt32Value)279044480U };
            C.Crosses crosses4 = new C.Crosses() { Val = C.CrossesValues.AutoZero };
            C.CrossBetween crossBetween2 = new C.CrossBetween() { Val = C.CrossBetweenValues.Between };

            valueAxis2.Append(axisId8);
            valueAxis2.Append(scaling4);
            valueAxis2.Append(delete4);
            valueAxis2.Append(axisPosition4);
            valueAxis2.Append(majorGridlines2);
            valueAxis2.Append(numberingFormat2);
            valueAxis2.Append(majorTickMark4);
            valueAxis2.Append(minorTickMark4);
            valueAxis2.Append(tickLabelPosition4);
            valueAxis2.Append(chartShapeProperties2);
            valueAxis2.Append(crossingAxis4);
            valueAxis2.Append(crosses4);
            valueAxis2.Append(crossBetween2);

            plotArea2.Append(layout5);
            plotArea2.Append(barChart2);
            plotArea2.Append(categoryAxis2);
            plotArea2.Append(valueAxis2);

            C.Legend legend2 = new C.Legend();
            C.LegendPosition legendPosition2 = new C.LegendPosition() { Val = C.LegendPositionValues.Bottom };
            C.Layout layout6 = new C.Layout();
            C.Overlay overlay4 = new C.Overlay() { Val = false };

            legend2.Append(legendPosition2);
            legend2.Append(layout6);
            legend2.Append(overlay4);
            C.PlotVisibleOnly plotVisibleOnly2 = new C.PlotVisibleOnly() { Val = true };
            C.DisplayBlanksAs displayBlanksAs2 = new C.DisplayBlanksAs() { Val = C.DisplayBlanksAsValues.Gap };
            C.ShowDataLabelsOverMaximum showDataLabelsOverMaximum2 = new C.ShowDataLabelsOverMaximum() { Val = false };

            chart2.Append(title2);
            chart2.Append(autoTitleDeleted2);
            chart2.Append(pivotFormats2);
            chart2.Append(plotArea2);
            chart2.Append(legend2);
            chart2.Append(plotVisibleOnly2);
            chart2.Append(displayBlanksAs2);
            chart2.Append(showDataLabelsOverMaximum2);

            C.PrintSettings printSettings2 = new C.PrintSettings();
            C.HeaderFooter headerFooter2 = new C.HeaderFooter();
            C.PageMargins pageMargins4 = new C.PageMargins() { Left = 0.7D, Right = 0.7D, Top = 0.75D, Bottom = 0.75D, Header = 0.3D, Footer = 0.3D };
            C.PageSetup pageSetup2 = new C.PageSetup();

            printSettings2.Append(headerFooter2);
            printSettings2.Append(pageMargins4);
            printSettings2.Append(pageSetup2);

            C.ChartSpaceExtensionList chartSpaceExtensionList2 = new C.ChartSpaceExtensionList();

            C.ChartSpaceExtension chartSpaceExtension2 = new C.ChartSpaceExtension() { Uri = "{781A3756-C4B2-4CAC-9D66-4F8BD8637D16}" };
            chartSpaceExtension2.AddNamespaceDeclaration("c14", "http://schemas.microsoft.com/office/drawing/2007/8/2/chart");

            C14.PivotOptions pivotOptions2 = new C14.PivotOptions();
            C14.DropZoneFilter dropZoneFilter2 = new C14.DropZoneFilter() { Val = true };
            C14.DropZoneCategories dropZoneCategories2 = new C14.DropZoneCategories() { Val = true };
            C14.DropZoneData dropZoneData2 = new C14.DropZoneData() { Val = true };
            C14.DropZoneSeries dropZoneSeries2 = new C14.DropZoneSeries() { Val = true };
            C14.DropZonesVisible dropZonesVisible2 = new C14.DropZonesVisible() { Val = true };

            pivotOptions2.Append(dropZoneFilter2);
            pivotOptions2.Append(dropZoneCategories2);
            pivotOptions2.Append(dropZoneData2);
            pivotOptions2.Append(dropZoneSeries2);
            pivotOptions2.Append(dropZonesVisible2);

            chartSpaceExtension2.Append(pivotOptions2);

            chartSpaceExtensionList2.Append(chartSpaceExtension2);

            chartSpace2.Append(date19042);
            chartSpace2.Append(editingLanguage2);
            chartSpace2.Append(roundedCorners2);
            chartSpace2.Append(alternateContent2);
            chartSpace2.Append(pivotSource2);
            chartSpace2.Append(chart2);
            chartSpace2.Append(printSettings2);
            chartSpace2.Append(chartSpaceExtensionList2);

            chartPart2.ChartSpace = chartSpace2;
        }

        // Generates content of pivotTablePart2.
        private void GeneratePivotTablePart2Content(PivotTablePart pivotTablePart2)
        {
            PivotTableDefinition pivotTableDefinition3 = new PivotTableDefinition() { Name = "PivotTable3", CacheId = (UInt32Value)7U, ApplyNumberFormats = false, ApplyBorderFormats = false, ApplyFontFormats = false, ApplyPatternFormats = false, ApplyAlignmentFormats = false, ApplyWidthHeightFormats = true, DataCaption = "Values", UpdatedVersion = 4, MinRefreshableVersion = 3, UseAutoFormatting = true, ItemPrintTitles = true, CreatedVersion = 4, Indent = (UInt32Value)0U, Outline = true, OutlineData = true, MultipleFieldFilters = false, ChartFormat = (UInt32Value)1U };
            Location location2 = new Location() { Reference = "A3:C24", FirstHeaderRow = (UInt32Value)0U, FirstDataRow = (UInt32Value)1U, FirstDataColumn = (UInt32Value)1U, RowPageCount = (UInt32Value)1U, ColumnsPerPage = (UInt32Value)1U };

            PivotFields pivotFields2 = new PivotFields() { Count = (UInt32Value)12U };

            PivotField pivotField13 = new PivotField() { Axis = PivotTableAxisValues.AxisRow, ShowAll = false, SortType = FieldSortValues.Descending };

            Items items3 = new Items() { Count = (UInt32Value)21U };
            Item item24 = new Item() { Index = (UInt32Value)1U };
            Item item25 = new Item() { Index = (UInt32Value)0U };
            Item item26 = new Item() { Index = (UInt32Value)11U };
            Item item27 = new Item() { Index = (UInt32Value)10U };
            Item item28 = new Item() { Index = (UInt32Value)9U };
            Item item29 = new Item() { Index = (UInt32Value)15U };
            Item item30 = new Item() { Index = (UInt32Value)2U };
            Item item31 = new Item() { Index = (UInt32Value)18U };
            Item item32 = new Item() { Index = (UInt32Value)19U };
            Item item33 = new Item() { Index = (UInt32Value)3U };
            Item item34 = new Item() { Index = (UInt32Value)17U };
            Item item35 = new Item() { Index = (UInt32Value)6U };
            Item item36 = new Item() { Index = (UInt32Value)7U };
            Item item37 = new Item() { Index = (UInt32Value)8U };
            Item item38 = new Item() { Index = (UInt32Value)5U };
            Item item39 = new Item() { Index = (UInt32Value)14U };
            Item item40 = new Item() { Index = (UInt32Value)4U };
            Item item41 = new Item() { Index = (UInt32Value)13U };
            Item item42 = new Item() { Index = (UInt32Value)12U };
            Item item43 = new Item() { Index = (UInt32Value)16U };
            Item item44 = new Item() { ItemType = ItemValues.Default };

            items3.Append(item24);
            items3.Append(item25);
            items3.Append(item26);
            items3.Append(item27);
            items3.Append(item28);
            items3.Append(item29);
            items3.Append(item30);
            items3.Append(item31);
            items3.Append(item32);
            items3.Append(item33);
            items3.Append(item34);
            items3.Append(item35);
            items3.Append(item36);
            items3.Append(item37);
            items3.Append(item38);
            items3.Append(item39);
            items3.Append(item40);
            items3.Append(item41);
            items3.Append(item42);
            items3.Append(item43);
            items3.Append(item44);

            AutoSortScope autoSortScope2 = new AutoSortScope();

            PivotArea pivotArea4 = new PivotArea() { DataOnly = false, Outline = false, FieldPosition = (UInt32Value)0U };

            PivotAreaReferences pivotAreaReferences4 = new PivotAreaReferences() { Count = (UInt32Value)1U };

            PivotAreaReference pivotAreaReference4 = new PivotAreaReference() { Field = (UInt32Value)4294967294U, Count = (UInt32Value)1U, Selected = false };
            FieldItem fieldItem52 = new FieldItem() { Val = (UInt32Value)1U };

            pivotAreaReference4.Append(fieldItem52);

            pivotAreaReferences4.Append(pivotAreaReference4);

            pivotArea4.Append(pivotAreaReferences4);

            autoSortScope2.Append(pivotArea4);

            pivotField13.Append(items3);
            pivotField13.Append(autoSortScope2);

            PivotField pivotField14 = new PivotField() { Axis = PivotTableAxisValues.AxisPage, ShowAll = false };

            Items items4 = new Items() { Count = (UInt32Value)2U };
            Item item45 = new Item() { Index = (UInt32Value)0U };
            Item item46 = new Item() { ItemType = ItemValues.Default };

            items4.Append(item45);
            items4.Append(item46);

            pivotField14.Append(items4);
            PivotField pivotField15 = new PivotField() { ShowAll = false };
            PivotField pivotField16 = new PivotField() { ShowAll = false };
            PivotField pivotField17 = new PivotField() { ShowAll = false };
            PivotField pivotField18 = new PivotField() { ShowAll = false };
            PivotField pivotField19 = new PivotField() { ShowAll = false };
            PivotField pivotField20 = new PivotField() { ShowAll = false };
            PivotField pivotField21 = new PivotField() { DataField = true, ShowAll = false };
            PivotField pivotField22 = new PivotField() { DataField = true, ShowAll = false };
            PivotField pivotField23 = new PivotField() { ShowAll = false };
            PivotField pivotField24 = new PivotField() { ShowAll = false };

            pivotFields2.Append(pivotField13);
            pivotFields2.Append(pivotField14);
            pivotFields2.Append(pivotField15);
            pivotFields2.Append(pivotField16);
            pivotFields2.Append(pivotField17);
            pivotFields2.Append(pivotField18);
            pivotFields2.Append(pivotField19);
            pivotFields2.Append(pivotField20);
            pivotFields2.Append(pivotField21);
            pivotFields2.Append(pivotField22);
            pivotFields2.Append(pivotField23);
            pivotFields2.Append(pivotField24);

            RowFields rowFields2 = new RowFields() { Count = (UInt32Value)1U };
            Field field3 = new Field() { Index = 0 };

            rowFields2.Append(field3);

            RowItems rowItems2 = new RowItems() { Count = (UInt32Value)21U };

            RowItem rowItem24 = new RowItem();
            MemberPropertyIndex memberPropertyIndex24 = new MemberPropertyIndex() { Val = 15 };

            rowItem24.Append(memberPropertyIndex24);

            RowItem rowItem25 = new RowItem();
            MemberPropertyIndex memberPropertyIndex25 = new MemberPropertyIndex() { Val = 9 };

            rowItem25.Append(memberPropertyIndex25);

            RowItem rowItem26 = new RowItem();
            MemberPropertyIndex memberPropertyIndex26 = new MemberPropertyIndex() { Val = 5 };

            rowItem26.Append(memberPropertyIndex26);

            RowItem rowItem27 = new RowItem();
            MemberPropertyIndex memberPropertyIndex27 = new MemberPropertyIndex() { Val = 16 };

            rowItem27.Append(memberPropertyIndex27);

            RowItem rowItem28 = new RowItem();
            MemberPropertyIndex memberPropertyIndex28 = new MemberPropertyIndex() { Val = 13 };

            rowItem28.Append(memberPropertyIndex28);

            RowItem rowItem29 = new RowItem();
            MemberPropertyIndex memberPropertyIndex29 = new MemberPropertyIndex() { Val = 4 };

            rowItem29.Append(memberPropertyIndex29);

            RowItem rowItem30 = new RowItem();
            MemberPropertyIndex memberPropertyIndex30 = new MemberPropertyIndex() { Val = 17 };

            rowItem30.Append(memberPropertyIndex30);

            RowItem rowItem31 = new RowItem();
            MemberPropertyIndex memberPropertyIndex31 = new MemberPropertyIndex() { Val = 18 };

            rowItem31.Append(memberPropertyIndex31);

            RowItem rowItem32 = new RowItem();
            MemberPropertyIndex memberPropertyIndex32 = new MemberPropertyIndex() { Val = 10 };

            rowItem32.Append(memberPropertyIndex32);

            RowItem rowItem33 = new RowItem();
            MemberPropertyIndex memberPropertyIndex33 = new MemberPropertyIndex() { Val = 11 };

            rowItem33.Append(memberPropertyIndex33);

            RowItem rowItem34 = new RowItem();
            MemberPropertyIndex memberPropertyIndex34 = new MemberPropertyIndex() { Val = 19 };

            rowItem34.Append(memberPropertyIndex34);

            RowItem rowItem35 = new RowItem();
            MemberPropertyIndex memberPropertyIndex35 = new MemberPropertyIndex() { Val = 8 };

            rowItem35.Append(memberPropertyIndex35);

            RowItem rowItem36 = new RowItem();
            MemberPropertyIndex memberPropertyIndex36 = new MemberPropertyIndex() { Val = 1 };

            rowItem36.Append(memberPropertyIndex36);

            RowItem rowItem37 = new RowItem();
            MemberPropertyIndex memberPropertyIndex37 = new MemberPropertyIndex() { Val = 12 };

            rowItem37.Append(memberPropertyIndex37);

            RowItem rowItem38 = new RowItem();
            MemberPropertyIndex memberPropertyIndex38 = new MemberPropertyIndex() { Val = 6 };

            rowItem38.Append(memberPropertyIndex38);

            RowItem rowItem39 = new RowItem();
            MemberPropertyIndex memberPropertyIndex39 = new MemberPropertyIndex() { Val = 7 };

            rowItem39.Append(memberPropertyIndex39);

            RowItem rowItem40 = new RowItem();
            MemberPropertyIndex memberPropertyIndex40 = new MemberPropertyIndex() { Val = 2 };

            rowItem40.Append(memberPropertyIndex40);

            RowItem rowItem41 = new RowItem();
            MemberPropertyIndex memberPropertyIndex41 = new MemberPropertyIndex();

            rowItem41.Append(memberPropertyIndex41);

            RowItem rowItem42 = new RowItem();
            MemberPropertyIndex memberPropertyIndex42 = new MemberPropertyIndex() { Val = 3 };

            rowItem42.Append(memberPropertyIndex42);

            RowItem rowItem43 = new RowItem();
            MemberPropertyIndex memberPropertyIndex43 = new MemberPropertyIndex() { Val = 14 };

            rowItem43.Append(memberPropertyIndex43);

            RowItem rowItem44 = new RowItem() { ItemType = ItemValues.Grand };
            MemberPropertyIndex memberPropertyIndex44 = new MemberPropertyIndex();

            rowItem44.Append(memberPropertyIndex44);

            rowItems2.Append(rowItem24);
            rowItems2.Append(rowItem25);
            rowItems2.Append(rowItem26);
            rowItems2.Append(rowItem27);
            rowItems2.Append(rowItem28);
            rowItems2.Append(rowItem29);
            rowItems2.Append(rowItem30);
            rowItems2.Append(rowItem31);
            rowItems2.Append(rowItem32);
            rowItems2.Append(rowItem33);
            rowItems2.Append(rowItem34);
            rowItems2.Append(rowItem35);
            rowItems2.Append(rowItem36);
            rowItems2.Append(rowItem37);
            rowItems2.Append(rowItem38);
            rowItems2.Append(rowItem39);
            rowItems2.Append(rowItem40);
            rowItems2.Append(rowItem41);
            rowItems2.Append(rowItem42);
            rowItems2.Append(rowItem43);
            rowItems2.Append(rowItem44);

            ColumnFields columnFields2 = new ColumnFields() { Count = (UInt32Value)1U };
            Field field4 = new Field() { Index = -2 };

            columnFields2.Append(field4);

            ColumnItems columnItems2 = new ColumnItems() { Count = (UInt32Value)2U };

            RowItem rowItem45 = new RowItem();
            MemberPropertyIndex memberPropertyIndex45 = new MemberPropertyIndex();

            rowItem45.Append(memberPropertyIndex45);

            RowItem rowItem46 = new RowItem() { Index = (UInt32Value)1U };
            MemberPropertyIndex memberPropertyIndex46 = new MemberPropertyIndex() { Val = 1 };

            rowItem46.Append(memberPropertyIndex46);

            columnItems2.Append(rowItem45);
            columnItems2.Append(rowItem46);

            PageFields pageFields2 = new PageFields() { Count = (UInt32Value)1U };
            PageField pageField2 = new PageField() { Field = 1, Hierarchy = -1 };

            pageFields2.Append(pageField2);

            DataFields dataFields2 = new DataFields() { Count = (UInt32Value)2U };
            DataField dataField3 = new DataField() { Name = "挣值总计", Field = (UInt32Value)8U, BaseField = 0, BaseItem = (UInt32Value)0U };
            DataField dataField4 = new DataField() { Name = "需求商业价值总计", Field = (UInt32Value)9U, BaseField = 0, BaseItem = (UInt32Value)0U };

            dataFields2.Append(dataField3);
            dataFields2.Append(dataField4);

            ChartFormats chartFormats2 = new ChartFormats() { Count = (UInt32Value)2U };

            ChartFormat chartFormat3 = new ChartFormat() { Chart = (UInt32Value)0U, Format = (UInt32Value)0U, Series = true };

            PivotArea pivotArea5 = new PivotArea() { Type = PivotAreaValues.Data, Outline = false, FieldPosition = (UInt32Value)0U };

            PivotAreaReferences pivotAreaReferences5 = new PivotAreaReferences() { Count = (UInt32Value)1U };

            PivotAreaReference pivotAreaReference5 = new PivotAreaReference() { Field = (UInt32Value)4294967294U, Count = (UInt32Value)1U, Selected = false };
            FieldItem fieldItem53 = new FieldItem() { Val = (UInt32Value)0U };

            pivotAreaReference5.Append(fieldItem53);

            pivotAreaReferences5.Append(pivotAreaReference5);

            pivotArea5.Append(pivotAreaReferences5);

            chartFormat3.Append(pivotArea5);

            ChartFormat chartFormat4 = new ChartFormat() { Chart = (UInt32Value)0U, Format = (UInt32Value)1U, Series = true };

            PivotArea pivotArea6 = new PivotArea() { Type = PivotAreaValues.Data, Outline = false, FieldPosition = (UInt32Value)0U };

            PivotAreaReferences pivotAreaReferences6 = new PivotAreaReferences() { Count = (UInt32Value)1U };

            PivotAreaReference pivotAreaReference6 = new PivotAreaReference() { Field = (UInt32Value)4294967294U, Count = (UInt32Value)1U, Selected = false };
            FieldItem fieldItem54 = new FieldItem() { Val = (UInt32Value)1U };

            pivotAreaReference6.Append(fieldItem54);

            pivotAreaReferences6.Append(pivotAreaReference6);

            pivotArea6.Append(pivotAreaReferences6);

            chartFormat4.Append(pivotArea6);

            chartFormats2.Append(chartFormat3);
            chartFormats2.Append(chartFormat4);
            PivotTableStyle pivotTableStyle2 = new PivotTableStyle() { Name = "PivotStyleLight16", ShowRowHeaders = true, ShowColumnHeaders = true, ShowRowStripes = false, ShowColumnStripes = false, ShowLastColumn = true };

            PivotTableDefinitionExtensionList pivotTableDefinitionExtensionList2 = new PivotTableDefinitionExtensionList();

            PivotTableDefinitionExtension pivotTableDefinitionExtension2 = new PivotTableDefinitionExtension() { Uri = "{962EF5D1-5CA2-4c93-8EF4-DBF5C05439D2}" };
            pivotTableDefinitionExtension2.AddNamespaceDeclaration("x14", "http://schemas.microsoft.com/office/spreadsheetml/2009/9/main");

            X14.PivotTableDefinition pivotTableDefinition4 = new X14.PivotTableDefinition() { HideValuesRow = true };
            pivotTableDefinition4.AddNamespaceDeclaration("xm", "http://schemas.microsoft.com/office/excel/2006/main");

            pivotTableDefinitionExtension2.Append(pivotTableDefinition4);

            pivotTableDefinitionExtensionList2.Append(pivotTableDefinitionExtension2);

            pivotTableDefinition3.Append(location2);
            pivotTableDefinition3.Append(pivotFields2);
            pivotTableDefinition3.Append(rowFields2);
            pivotTableDefinition3.Append(rowItems2);
            pivotTableDefinition3.Append(columnFields2);
            pivotTableDefinition3.Append(columnItems2);
            pivotTableDefinition3.Append(pageFields2);
            pivotTableDefinition3.Append(dataFields2);
            pivotTableDefinition3.Append(chartFormats2);
            pivotTableDefinition3.Append(pivotTableStyle2);
            pivotTableDefinition3.Append(pivotTableDefinitionExtensionList2);

            pivotTablePart2.PivotTableDefinition = pivotTableDefinition3;
        }

        // Generates content of worksheetPart3.
        private void GenerateWorksheetPart3Content(WorksheetPart worksheetPart3)
        {
            Worksheet worksheet3 = new Worksheet() { MCAttributes = new MarkupCompatibilityAttributes() { Ignorable = "x14ac" } };
            worksheet3.AddNamespaceDeclaration("r", "http://schemas.openxmlformats.org/officeDocument/2006/relationships");
            worksheet3.AddNamespaceDeclaration("mc", "http://schemas.openxmlformats.org/markup-compatibility/2006");
            worksheet3.AddNamespaceDeclaration("x14ac", "http://schemas.microsoft.com/office/spreadsheetml/2009/9/ac");
            SheetDimension sheetDimension3 = new SheetDimension() { Reference = "A1:C24" };

            SheetViews sheetViews3 = new SheetViews();

            SheetView sheetView3 = new SheetView() { WorkbookViewId = (UInt32Value)0U };
            Selection selection3 = new Selection() { ActiveCell = "B55", SequenceOfReferences = new ListValue<StringValue>() { InnerText = "B55" } };

            sheetView3.Append(selection3);

            sheetViews3.Append(sheetView3);
            SheetFormatProperties sheetFormatProperties3 = new SheetFormatProperties() { DefaultRowHeight = 15D, DyDescent = 0.25D };

            Columns columns3 = new Columns();
            Column column6 = new Column() { Min = (UInt32Value)1U, Max = (UInt32Value)1U, Width = 27.28515625D, BestFit = true, CustomWidth = true };
            Column column7 = new Column() { Min = (UInt32Value)2U, Max = (UInt32Value)3U, Width = 24.28515625D, BestFit = true, CustomWidth = true };

            columns3.Append(column6);
            columns3.Append(column7);

            SheetData sheetData3 = new SheetData();

            Row row47 = new Row() { RowIndex = (UInt32Value)1U, Spans = new ListValue<StringValue>() { InnerText = "1:3" }, DyDescent = 0.25D };

            Cell cell137 = new Cell() { CellReference = "A1", StyleIndex = (UInt32Value)1U, DataType = CellValues.SharedString };
            CellValue cellValue137 = new CellValue();
            cellValue137.Text = "1";

            cell137.Append(cellValue137);

            Cell cell138 = new Cell() { CellReference = "B1", DataType = CellValues.SharedString };
            CellValue cellValue138 = new CellValue();
            cellValue138.Text = "61";

            cell138.Append(cellValue138);

            row47.Append(cell137);
            row47.Append(cell138);

            sheetData3.Append(row47);
            PageMargins pageMargins5 = new PageMargins() { Left = 0.7D, Right = 0.7D, Top = 0.75D, Bottom = 0.75D, Header = 0.3D, Footer = 0.3D };
            Drawing drawing3 = new Drawing() { Id = "rId2" };

            worksheet3.Append(sheetDimension3);
            worksheet3.Append(sheetViews3);
            worksheet3.Append(sheetFormatProperties3);
            worksheet3.Append(columns3);
            worksheet3.Append(sheetData3);
            worksheet3.Append(pageMargins5);
            worksheet3.Append(drawing3);

            worksheetPart3.Worksheet = worksheet3;
        }

        // Generates content of drawingsPart3.
        private void GenerateDrawingsPart3Content(DrawingsPart drawingsPart3)
        {
            Xdr.WorksheetDrawing worksheetDrawing3 = new Xdr.WorksheetDrawing();
            worksheetDrawing3.AddNamespaceDeclaration("xdr", "http://schemas.openxmlformats.org/drawingml/2006/spreadsheetDrawing");
            worksheetDrawing3.AddNamespaceDeclaration("a", "http://schemas.openxmlformats.org/drawingml/2006/main");

            Xdr.TwoCellAnchor twoCellAnchor3 = new Xdr.TwoCellAnchor();

            Xdr.FromMarker fromMarker3 = new Xdr.FromMarker();
            Xdr.ColumnId columnId5 = new Xdr.ColumnId();
            columnId5.Text = "0";
            Xdr.ColumnOffset columnOffset5 = new Xdr.ColumnOffset();
            columnOffset5.Text = "0";
            Xdr.RowId rowId5 = new Xdr.RowId();
            rowId5.Text = "25";
            Xdr.RowOffset rowOffset5 = new Xdr.RowOffset();
            rowOffset5.Text = "9524";

            fromMarker3.Append(columnId5);
            fromMarker3.Append(columnOffset5);
            fromMarker3.Append(rowId5);
            fromMarker3.Append(rowOffset5);

            Xdr.ToMarker toMarker3 = new Xdr.ToMarker();
            Xdr.ColumnId columnId6 = new Xdr.ColumnId();
            columnId6.Text = "6";
            Xdr.ColumnOffset columnOffset6 = new Xdr.ColumnOffset();
            columnOffset6.Text = "9525";
            Xdr.RowId rowId6 = new Xdr.RowId();
            rowId6.Text = "48";
            Xdr.RowOffset rowOffset6 = new Xdr.RowOffset();
            rowOffset6.Text = "38099";

            toMarker3.Append(columnId6);
            toMarker3.Append(columnOffset6);
            toMarker3.Append(rowId6);
            toMarker3.Append(rowOffset6);

            Xdr.GraphicFrame graphicFrame3 = new Xdr.GraphicFrame() { Macro = "" };

            Xdr.NonVisualGraphicFrameProperties nonVisualGraphicFrameProperties3 = new Xdr.NonVisualGraphicFrameProperties();
            Xdr.NonVisualDrawingProperties nonVisualDrawingProperties3 = new Xdr.NonVisualDrawingProperties() { Id = (UInt32Value)2U, Name = "Chart 1" };
            Xdr.NonVisualGraphicFrameDrawingProperties nonVisualGraphicFrameDrawingProperties3 = new Xdr.NonVisualGraphicFrameDrawingProperties();

            nonVisualGraphicFrameProperties3.Append(nonVisualDrawingProperties3);
            nonVisualGraphicFrameProperties3.Append(nonVisualGraphicFrameDrawingProperties3);

            Xdr.Transform transform3 = new Xdr.Transform();
            A.Offset offset3 = new A.Offset() { X = 0L, Y = 0L };
            A.Extents extents3 = new A.Extents() { Cx = 0L, Cy = 0L };

            transform3.Append(offset3);
            transform3.Append(extents3);

            A.Graphic graphic3 = new A.Graphic();

            A.GraphicData graphicData3 = new A.GraphicData() { Uri = "http://schemas.openxmlformats.org/drawingml/2006/chart" };

            C.ChartReference chartReference3 = new C.ChartReference() { Id = "rId1" };
            chartReference3.AddNamespaceDeclaration("c", "http://schemas.openxmlformats.org/drawingml/2006/chart");
            chartReference3.AddNamespaceDeclaration("r", "http://schemas.openxmlformats.org/officeDocument/2006/relationships");

            graphicData3.Append(chartReference3);

            graphic3.Append(graphicData3);

            graphicFrame3.Append(nonVisualGraphicFrameProperties3);
            graphicFrame3.Append(transform3);
            graphicFrame3.Append(graphic3);
            Xdr.ClientData clientData3 = new Xdr.ClientData();

            twoCellAnchor3.Append(fromMarker3);
            twoCellAnchor3.Append(toMarker3);
            twoCellAnchor3.Append(graphicFrame3);
            twoCellAnchor3.Append(clientData3);

            worksheetDrawing3.Append(twoCellAnchor3);

            drawingsPart3.WorksheetDrawing = worksheetDrawing3;
        }

        // Generates content of chartPart3.
        private void GenerateChartPart3Content(ChartPart chartPart3)
        {
            C.ChartSpace chartSpace3 = new C.ChartSpace();
            chartSpace3.AddNamespaceDeclaration("c", "http://schemas.openxmlformats.org/drawingml/2006/chart");
            chartSpace3.AddNamespaceDeclaration("a", "http://schemas.openxmlformats.org/drawingml/2006/main");
            chartSpace3.AddNamespaceDeclaration("r", "http://schemas.openxmlformats.org/officeDocument/2006/relationships");
            C.Date1904 date19043 = new C.Date1904() { Val = false };
            C.EditingLanguage editingLanguage3 = new C.EditingLanguage() { Val = "en-US" };
            C.RoundedCorners roundedCorners3 = new C.RoundedCorners() { Val = false };

            AlternateContent alternateContent3 = new AlternateContent();
            alternateContent3.AddNamespaceDeclaration("mc", "http://schemas.openxmlformats.org/markup-compatibility/2006");

            AlternateContentChoice alternateContentChoice3 = new AlternateContentChoice() { Requires = "c14" };
            alternateContentChoice3.AddNamespaceDeclaration("c14", "http://schemas.microsoft.com/office/drawing/2007/8/2/chart");
            C14.Style style5 = new C14.Style() { Val = 102 };

            alternateContentChoice3.Append(style5);

            AlternateContentFallback alternateContentFallback3 = new AlternateContentFallback();
            C.Style style6 = new C.Style() { Val = 2 };

            alternateContentFallback3.Append(style6);

            alternateContent3.Append(alternateContentChoice3);
            alternateContent3.Append(alternateContentFallback3);

            C.PivotSource pivotSource3 = new C.PivotSource();
            C.PivotTableName pivotTableName3 = new C.PivotTableName();
            pivotTableName3.Text = "需求工作量分析!PivotTable2";
            C.FormatId formatId3 = new C.FormatId() { Val = (UInt32Value)0U };

            pivotSource3.Append(pivotTableName3);
            pivotSource3.Append(formatId3);

            C.Chart chart3 = new C.Chart();

            C.Title title3 = new C.Title();

            C.ChartText chartText3 = new C.ChartText();

            C.RichText richText3 = new C.RichText();
            A.BodyProperties bodyProperties3 = new A.BodyProperties();
            A.ListStyle listStyle3 = new A.ListStyle();

            A.Paragraph paragraph3 = new A.Paragraph();

            A.ParagraphProperties paragraphProperties3 = new A.ParagraphProperties();
            A.DefaultRunProperties defaultRunProperties3 = new A.DefaultRunProperties();

            paragraphProperties3.Append(defaultRunProperties3);

            A.Run run3 = new A.Run();
            A.RunProperties runProperties3 = new A.RunProperties() { Language = "zh-CN", AlternativeLanguage = "en-US" };
            A.Text text3 = new A.Text();
            text3.Text = "需求工作量分析";

            run3.Append(runProperties3);
            run3.Append(text3);
            A.EndParagraphRunProperties endParagraphRunProperties3 = new A.EndParagraphRunProperties() { Language = "en-US" };

            paragraph3.Append(paragraphProperties3);
            paragraph3.Append(run3);
            paragraph3.Append(endParagraphRunProperties3);

            richText3.Append(bodyProperties3);
            richText3.Append(listStyle3);
            richText3.Append(paragraph3);

            chartText3.Append(richText3);
            C.Layout layout7 = new C.Layout();
            C.Overlay overlay5 = new C.Overlay() { Val = false };

            title3.Append(chartText3);
            title3.Append(layout7);
            title3.Append(overlay5);
            C.AutoTitleDeleted autoTitleDeleted3 = new C.AutoTitleDeleted() { Val = false };

            C.PivotFormats pivotFormats3 = new C.PivotFormats();

            C.PivotFormat pivotFormat5 = new C.PivotFormat();
            C.Index index9 = new C.Index() { Val = (UInt32Value)0U };

            C.Marker marker5 = new C.Marker();
            C.Symbol symbol5 = new C.Symbol() { Val = C.MarkerStyleValues.None };

            marker5.Append(symbol5);

            pivotFormat5.Append(index9);
            pivotFormat5.Append(marker5);

            C.PivotFormat pivotFormat6 = new C.PivotFormat();
            C.Index index10 = new C.Index() { Val = (UInt32Value)1U };

            C.Marker marker6 = new C.Marker();
            C.Symbol symbol6 = new C.Symbol() { Val = C.MarkerStyleValues.None };

            marker6.Append(symbol6);

            pivotFormat6.Append(index10);
            pivotFormat6.Append(marker6);

            pivotFormats3.Append(pivotFormat5);
            pivotFormats3.Append(pivotFormat6);

            C.PlotArea plotArea3 = new C.PlotArea();
            C.Layout layout8 = new C.Layout();

            C.BarChart barChart3 = new C.BarChart();
            C.BarDirection barDirection3 = new C.BarDirection() { Val = C.BarDirectionValues.Column };
            C.BarGrouping barGrouping3 = new C.BarGrouping() { Val = C.BarGroupingValues.Stacked };
            C.VaryColors varyColors3 = new C.VaryColors() { Val = false };

            C.BarChartSeries barChartSeries5 = new C.BarChartSeries();
            C.Index index11 = new C.Index() { Val = (UInt32Value)0U };
            C.Order order5 = new C.Order() { Val = (UInt32Value)0U };

            C.SeriesText seriesText5 = new C.SeriesText();

            C.StringReference stringReference9 = new C.StringReference();
            C.Formula formula13 = new C.Formula();
            formula13.Text = "\'需求工作量分析\'!$B$3";

            C.StringCache stringCache9 = new C.StringCache();
            C.PointCount pointCount13 = new C.PointCount() { Val = (UInt32Value)1U };

            C.StringPoint stringPoint85 = new C.StringPoint() { Index = (UInt32Value)0U };
            C.NumericValue numericValue165 = new C.NumericValue();
            numericValue165.Text = "未完成的工作项目总数";

            stringPoint85.Append(numericValue165);

            stringCache9.Append(pointCount13);
            stringCache9.Append(stringPoint85);

            stringReference9.Append(formula13);
            stringReference9.Append(stringCache9);

            seriesText5.Append(stringReference9);
            C.InvertIfNegative invertIfNegative5 = new C.InvertIfNegative() { Val = false };

            C.CategoryAxisData categoryAxisData5 = new C.CategoryAxisData();

            C.StringReference stringReference10 = new C.StringReference();
            C.Formula formula14 = new C.Formula();
            formula14.Text = "\'需求工作量分析\'!$A$4:$A$24";

            C.StringCache stringCache10 = new C.StringCache();
            C.PointCount pointCount14 = new C.PointCount() { Val = (UInt32Value)20U };

            C.StringPoint stringPoint86 = new C.StringPoint() { Index = (UInt32Value)0U };
            C.NumericValue numericValue166 = new C.NumericValue();
            numericValue166.Text = "集成团队管理";

            stringPoint86.Append(numericValue166);

            C.StringPoint stringPoint87 = new C.StringPoint() { Index = (UInt32Value)1U };
            C.NumericValue numericValue167 = new C.NumericValue();
            numericValue167.Text = "需求管理基本功能";

            stringPoint87.Append(numericValue167);

            C.StringPoint stringPoint88 = new C.StringPoint() { Index = (UInt32Value)2U };
            C.NumericValue numericValue168 = new C.NumericValue();
            numericValue168.Text = "需求管理中的进度跟踪";

            stringPoint88.Append(numericValue168);

            C.StringPoint stringPoint89 = new C.StringPoint() { Index = (UInt32Value)3U };
            C.NumericValue numericValue169 = new C.NumericValue();
            numericValue169.Text = "全文检索";

            stringPoint89.Append(numericValue169);

            C.StringPoint stringPoint90 = new C.StringPoint() { Index = (UInt32Value)4U };
            C.NumericValue numericValue170 = new C.NumericValue();
            numericValue170.Text = "整合配置管理系统";

            stringPoint90.Append(numericValue170);

            C.StringPoint stringPoint91 = new C.StringPoint() { Index = (UInt32Value)5U };
            C.NumericValue numericValue171 = new C.NumericValue();
            numericValue171.Text = "决策过程跟踪";

            stringPoint91.Append(numericValue171);

            C.StringPoint stringPoint92 = new C.StringPoint() { Index = (UInt32Value)6U };
            C.NumericValue numericValue172 = new C.NumericValue();
            numericValue172.Text = "风险管理";

            stringPoint92.Append(numericValue172);

            C.StringPoint stringPoint93 = new C.StringPoint() { Index = (UInt32Value)7U };
            C.NumericValue numericValue173 = new C.NumericValue();
            numericValue173.Text = "用户选择托管配置管理系统";

            stringPoint93.Append(numericValue173);

            C.StringPoint stringPoint94 = new C.StringPoint() { Index = (UInt32Value)8U };
            C.NumericValue numericValue174 = new C.NumericValue();
            numericValue174.Text = "用户身份验证";

            stringPoint94.Append(numericValue174);

            C.StringPoint stringPoint95 = new C.StringPoint() { Index = (UInt32Value)9U };
            C.NumericValue numericValue175 = new C.NumericValue();
            numericValue175.Text = "任务标签";

            stringPoint95.Append(numericValue175);

            C.StringPoint stringPoint96 = new C.StringPoint() { Index = (UInt32Value)10U };
            C.NumericValue numericValue176 = new C.NumericValue();
            numericValue176.Text = "权限管理";

            stringPoint96.Append(numericValue176);

            C.StringPoint stringPoint97 = new C.StringPoint() { Index = (UInt32Value)11U };
            C.NumericValue numericValue177 = new C.NumericValue();
            numericValue177.Text = "元素之间的相互转化";

            stringPoint97.Append(numericValue177);

            C.StringPoint stringPoint98 = new C.StringPoint() { Index = (UInt32Value)12U };
            C.NumericValue numericValue178 = new C.NumericValue();
            numericValue178.Text = "统计功能";

            stringPoint98.Append(numericValue178);

            C.StringPoint stringPoint99 = new C.StringPoint() { Index = (UInt32Value)13U };
            C.NumericValue numericValue179 = new C.NumericValue();
            numericValue179.Text = "会议管理";

            stringPoint99.Append(numericValue179);

            C.StringPoint stringPoint100 = new C.StringPoint() { Index = (UInt32Value)14U };
            C.NumericValue numericValue180 = new C.NumericValue();
            numericValue180.Text = "流程管理";

            stringPoint100.Append(numericValue180);

            C.StringPoint stringPoint101 = new C.StringPoint() { Index = (UInt32Value)15U };
            C.NumericValue numericValue181 = new C.NumericValue();
            numericValue181.Text = "支持国际化";

            stringPoint101.Append(numericValue181);

            C.StringPoint stringPoint102 = new C.StringPoint() { Index = (UInt32Value)16U };
            C.NumericValue numericValue182 = new C.NumericValue();
            numericValue182.Text = "实现通用的分页显示功能";

            stringPoint102.Append(numericValue182);

            C.StringPoint stringPoint103 = new C.StringPoint() { Index = (UInt32Value)17U };
            C.NumericValue numericValue183 = new C.NumericValue();
            numericValue183.Text = "团队时间管理";

            stringPoint103.Append(numericValue183);

            C.StringPoint stringPoint104 = new C.StringPoint() { Index = (UInt32Value)18U };
            C.NumericValue numericValue184 = new C.NumericValue();
            numericValue184.Text = "事故管理";

            stringPoint104.Append(numericValue184);

            C.StringPoint stringPoint105 = new C.StringPoint() { Index = (UInt32Value)19U };
            C.NumericValue numericValue185 = new C.NumericValue();
            numericValue185.Text = "添加CAPTCHA支持";

            stringPoint105.Append(numericValue185);

            stringCache10.Append(pointCount14);
            stringCache10.Append(stringPoint86);
            stringCache10.Append(stringPoint87);
            stringCache10.Append(stringPoint88);
            stringCache10.Append(stringPoint89);
            stringCache10.Append(stringPoint90);
            stringCache10.Append(stringPoint91);
            stringCache10.Append(stringPoint92);
            stringCache10.Append(stringPoint93);
            stringCache10.Append(stringPoint94);
            stringCache10.Append(stringPoint95);
            stringCache10.Append(stringPoint96);
            stringCache10.Append(stringPoint97);
            stringCache10.Append(stringPoint98);
            stringCache10.Append(stringPoint99);
            stringCache10.Append(stringPoint100);
            stringCache10.Append(stringPoint101);
            stringCache10.Append(stringPoint102);
            stringCache10.Append(stringPoint103);
            stringCache10.Append(stringPoint104);
            stringCache10.Append(stringPoint105);

            stringReference10.Append(formula14);
            stringReference10.Append(stringCache10);

            categoryAxisData5.Append(stringReference10);

            C.Values values5 = new C.Values();

            C.NumberReference numberReference5 = new C.NumberReference();
            C.Formula formula15 = new C.Formula();
            formula15.Text = "\'需求工作量分析\'!$B$4:$B$24";

            C.NumberingCache numberingCache5 = new C.NumberingCache();
            C.FormatCode formatCode5 = new C.FormatCode();
            formatCode5.Text = "General";
            C.PointCount pointCount15 = new C.PointCount() { Val = (UInt32Value)20U };

            C.NumericPoint numericPoint81 = new C.NumericPoint() { Index = (UInt32Value)0U };
            C.NumericValue numericValue186 = new C.NumericValue();
            numericValue186.Text = "12";

            numericPoint81.Append(numericValue186);

            C.NumericPoint numericPoint82 = new C.NumericPoint() { Index = (UInt32Value)1U };
            C.NumericValue numericValue187 = new C.NumericValue();
            numericValue187.Text = "12";

            numericPoint82.Append(numericValue187);

            C.NumericPoint numericPoint83 = new C.NumericPoint() { Index = (UInt32Value)2U };
            C.NumericValue numericValue188 = new C.NumericValue();
            numericValue188.Text = "10";

            numericPoint83.Append(numericValue188);

            C.NumericPoint numericPoint84 = new C.NumericPoint() { Index = (UInt32Value)3U };
            C.NumericValue numericValue189 = new C.NumericValue();
            numericValue189.Text = "10";

            numericPoint84.Append(numericValue189);

            C.NumericPoint numericPoint85 = new C.NumericPoint() { Index = (UInt32Value)4U };
            C.NumericValue numericValue190 = new C.NumericValue();
            numericValue190.Text = "10";

            numericPoint85.Append(numericValue190);

            C.NumericPoint numericPoint86 = new C.NumericPoint() { Index = (UInt32Value)5U };
            C.NumericValue numericValue191 = new C.NumericValue();
            numericValue191.Text = "7";

            numericPoint86.Append(numericValue191);

            C.NumericPoint numericPoint87 = new C.NumericPoint() { Index = (UInt32Value)6U };
            C.NumericValue numericValue192 = new C.NumericValue();
            numericValue192.Text = "7";

            numericPoint87.Append(numericValue192);

            C.NumericPoint numericPoint88 = new C.NumericPoint() { Index = (UInt32Value)7U };
            C.NumericValue numericValue193 = new C.NumericValue();
            numericValue193.Text = "5";

            numericPoint88.Append(numericValue193);

            C.NumericPoint numericPoint89 = new C.NumericPoint() { Index = (UInt32Value)8U };
            C.NumericValue numericValue194 = new C.NumericValue();
            numericValue194.Text = "5";

            numericPoint89.Append(numericValue194);

            C.NumericPoint numericPoint90 = new C.NumericPoint() { Index = (UInt32Value)9U };
            C.NumericValue numericValue195 = new C.NumericValue();
            numericValue195.Text = "4";

            numericPoint90.Append(numericValue195);

            C.NumericPoint numericPoint91 = new C.NumericPoint() { Index = (UInt32Value)10U };
            C.NumericValue numericValue196 = new C.NumericValue();
            numericValue196.Text = "4";

            numericPoint91.Append(numericValue196);

            C.NumericPoint numericPoint92 = new C.NumericPoint() { Index = (UInt32Value)11U };
            C.NumericValue numericValue197 = new C.NumericValue();
            numericValue197.Text = "3";

            numericPoint92.Append(numericValue197);

            C.NumericPoint numericPoint93 = new C.NumericPoint() { Index = (UInt32Value)12U };
            C.NumericValue numericValue198 = new C.NumericValue();
            numericValue198.Text = "3";

            numericPoint93.Append(numericValue198);

            C.NumericPoint numericPoint94 = new C.NumericPoint() { Index = (UInt32Value)13U };
            C.NumericValue numericValue199 = new C.NumericValue();
            numericValue199.Text = "2";

            numericPoint94.Append(numericValue199);

            C.NumericPoint numericPoint95 = new C.NumericPoint() { Index = (UInt32Value)14U };
            C.NumericValue numericValue200 = new C.NumericValue();
            numericValue200.Text = "2";

            numericPoint95.Append(numericValue200);

            C.NumericPoint numericPoint96 = new C.NumericPoint() { Index = (UInt32Value)15U };
            C.NumericValue numericValue201 = new C.NumericValue();
            numericValue201.Text = "1";

            numericPoint96.Append(numericValue201);

            C.NumericPoint numericPoint97 = new C.NumericPoint() { Index = (UInt32Value)16U };
            C.NumericValue numericValue202 = new C.NumericValue();
            numericValue202.Text = "0";

            numericPoint97.Append(numericValue202);

            C.NumericPoint numericPoint98 = new C.NumericPoint() { Index = (UInt32Value)17U };
            C.NumericValue numericValue203 = new C.NumericValue();
            numericValue203.Text = "0";

            numericPoint98.Append(numericValue203);

            C.NumericPoint numericPoint99 = new C.NumericPoint() { Index = (UInt32Value)18U };
            C.NumericValue numericValue204 = new C.NumericValue();
            numericValue204.Text = "0";

            numericPoint99.Append(numericValue204);

            C.NumericPoint numericPoint100 = new C.NumericPoint() { Index = (UInt32Value)19U };
            C.NumericValue numericValue205 = new C.NumericValue();
            numericValue205.Text = "0";

            numericPoint100.Append(numericValue205);

            numberingCache5.Append(formatCode5);
            numberingCache5.Append(pointCount15);
            numberingCache5.Append(numericPoint81);
            numberingCache5.Append(numericPoint82);
            numberingCache5.Append(numericPoint83);
            numberingCache5.Append(numericPoint84);
            numberingCache5.Append(numericPoint85);
            numberingCache5.Append(numericPoint86);
            numberingCache5.Append(numericPoint87);
            numberingCache5.Append(numericPoint88);
            numberingCache5.Append(numericPoint89);
            numberingCache5.Append(numericPoint90);
            numberingCache5.Append(numericPoint91);
            numberingCache5.Append(numericPoint92);
            numberingCache5.Append(numericPoint93);
            numberingCache5.Append(numericPoint94);
            numberingCache5.Append(numericPoint95);
            numberingCache5.Append(numericPoint96);
            numberingCache5.Append(numericPoint97);
            numberingCache5.Append(numericPoint98);
            numberingCache5.Append(numericPoint99);
            numberingCache5.Append(numericPoint100);

            numberReference5.Append(formula15);
            numberReference5.Append(numberingCache5);

            values5.Append(numberReference5);

            barChartSeries5.Append(index11);
            barChartSeries5.Append(order5);
            barChartSeries5.Append(seriesText5);
            barChartSeries5.Append(invertIfNegative5);
            barChartSeries5.Append(categoryAxisData5);
            barChartSeries5.Append(values5);

            C.BarChartSeries barChartSeries6 = new C.BarChartSeries();
            C.Index index12 = new C.Index() { Val = (UInt32Value)1U };
            C.Order order6 = new C.Order() { Val = (UInt32Value)1U };

            C.SeriesText seriesText6 = new C.SeriesText();

            C.StringReference stringReference11 = new C.StringReference();
            C.Formula formula16 = new C.Formula();
            formula16.Text = "\'需求工作量分析\'!$C$3";

            C.StringCache stringCache11 = new C.StringCache();
            C.PointCount pointCount16 = new C.PointCount() { Val = (UInt32Value)1U };

            C.StringPoint stringPoint106 = new C.StringPoint() { Index = (UInt32Value)0U };
            C.NumericValue numericValue206 = new C.NumericValue();
            numericValue206.Text = "已完成的工作项目总数";

            stringPoint106.Append(numericValue206);

            stringCache11.Append(pointCount16);
            stringCache11.Append(stringPoint106);

            stringReference11.Append(formula16);
            stringReference11.Append(stringCache11);

            seriesText6.Append(stringReference11);
            C.InvertIfNegative invertIfNegative6 = new C.InvertIfNegative() { Val = false };

            C.CategoryAxisData categoryAxisData6 = new C.CategoryAxisData();

            C.StringReference stringReference12 = new C.StringReference();
            C.Formula formula17 = new C.Formula();
            formula17.Text = "\'需求工作量分析\'!$A$4:$A$24";

            C.StringCache stringCache12 = new C.StringCache();
            C.PointCount pointCount17 = new C.PointCount() { Val = (UInt32Value)20U };

            C.StringPoint stringPoint107 = new C.StringPoint() { Index = (UInt32Value)0U };
            C.NumericValue numericValue207 = new C.NumericValue();
            numericValue207.Text = "集成团队管理";

            stringPoint107.Append(numericValue207);

            C.StringPoint stringPoint108 = new C.StringPoint() { Index = (UInt32Value)1U };
            C.NumericValue numericValue208 = new C.NumericValue();
            numericValue208.Text = "需求管理基本功能";

            stringPoint108.Append(numericValue208);

            C.StringPoint stringPoint109 = new C.StringPoint() { Index = (UInt32Value)2U };
            C.NumericValue numericValue209 = new C.NumericValue();
            numericValue209.Text = "需求管理中的进度跟踪";

            stringPoint109.Append(numericValue209);

            C.StringPoint stringPoint110 = new C.StringPoint() { Index = (UInt32Value)3U };
            C.NumericValue numericValue210 = new C.NumericValue();
            numericValue210.Text = "全文检索";

            stringPoint110.Append(numericValue210);

            C.StringPoint stringPoint111 = new C.StringPoint() { Index = (UInt32Value)4U };
            C.NumericValue numericValue211 = new C.NumericValue();
            numericValue211.Text = "整合配置管理系统";

            stringPoint111.Append(numericValue211);

            C.StringPoint stringPoint112 = new C.StringPoint() { Index = (UInt32Value)5U };
            C.NumericValue numericValue212 = new C.NumericValue();
            numericValue212.Text = "决策过程跟踪";

            stringPoint112.Append(numericValue212);

            C.StringPoint stringPoint113 = new C.StringPoint() { Index = (UInt32Value)6U };
            C.NumericValue numericValue213 = new C.NumericValue();
            numericValue213.Text = "风险管理";

            stringPoint113.Append(numericValue213);

            C.StringPoint stringPoint114 = new C.StringPoint() { Index = (UInt32Value)7U };
            C.NumericValue numericValue214 = new C.NumericValue();
            numericValue214.Text = "用户选择托管配置管理系统";

            stringPoint114.Append(numericValue214);

            C.StringPoint stringPoint115 = new C.StringPoint() { Index = (UInt32Value)8U };
            C.NumericValue numericValue215 = new C.NumericValue();
            numericValue215.Text = "用户身份验证";

            stringPoint115.Append(numericValue215);

            C.StringPoint stringPoint116 = new C.StringPoint() { Index = (UInt32Value)9U };
            C.NumericValue numericValue216 = new C.NumericValue();
            numericValue216.Text = "任务标签";

            stringPoint116.Append(numericValue216);

            C.StringPoint stringPoint117 = new C.StringPoint() { Index = (UInt32Value)10U };
            C.NumericValue numericValue217 = new C.NumericValue();
            numericValue217.Text = "权限管理";

            stringPoint117.Append(numericValue217);

            C.StringPoint stringPoint118 = new C.StringPoint() { Index = (UInt32Value)11U };
            C.NumericValue numericValue218 = new C.NumericValue();
            numericValue218.Text = "元素之间的相互转化";

            stringPoint118.Append(numericValue218);

            C.StringPoint stringPoint119 = new C.StringPoint() { Index = (UInt32Value)12U };
            C.NumericValue numericValue219 = new C.NumericValue();
            numericValue219.Text = "统计功能";

            stringPoint119.Append(numericValue219);

            C.StringPoint stringPoint120 = new C.StringPoint() { Index = (UInt32Value)13U };
            C.NumericValue numericValue220 = new C.NumericValue();
            numericValue220.Text = "会议管理";

            stringPoint120.Append(numericValue220);

            C.StringPoint stringPoint121 = new C.StringPoint() { Index = (UInt32Value)14U };
            C.NumericValue numericValue221 = new C.NumericValue();
            numericValue221.Text = "流程管理";

            stringPoint121.Append(numericValue221);

            C.StringPoint stringPoint122 = new C.StringPoint() { Index = (UInt32Value)15U };
            C.NumericValue numericValue222 = new C.NumericValue();
            numericValue222.Text = "支持国际化";

            stringPoint122.Append(numericValue222);

            C.StringPoint stringPoint123 = new C.StringPoint() { Index = (UInt32Value)16U };
            C.NumericValue numericValue223 = new C.NumericValue();
            numericValue223.Text = "实现通用的分页显示功能";

            stringPoint123.Append(numericValue223);

            C.StringPoint stringPoint124 = new C.StringPoint() { Index = (UInt32Value)17U };
            C.NumericValue numericValue224 = new C.NumericValue();
            numericValue224.Text = "团队时间管理";

            stringPoint124.Append(numericValue224);

            C.StringPoint stringPoint125 = new C.StringPoint() { Index = (UInt32Value)18U };
            C.NumericValue numericValue225 = new C.NumericValue();
            numericValue225.Text = "事故管理";

            stringPoint125.Append(numericValue225);

            C.StringPoint stringPoint126 = new C.StringPoint() { Index = (UInt32Value)19U };
            C.NumericValue numericValue226 = new C.NumericValue();
            numericValue226.Text = "添加CAPTCHA支持";

            stringPoint126.Append(numericValue226);

            stringCache12.Append(pointCount17);
            stringCache12.Append(stringPoint107);
            stringCache12.Append(stringPoint108);
            stringCache12.Append(stringPoint109);
            stringCache12.Append(stringPoint110);
            stringCache12.Append(stringPoint111);
            stringCache12.Append(stringPoint112);
            stringCache12.Append(stringPoint113);
            stringCache12.Append(stringPoint114);
            stringCache12.Append(stringPoint115);
            stringCache12.Append(stringPoint116);
            stringCache12.Append(stringPoint117);
            stringCache12.Append(stringPoint118);
            stringCache12.Append(stringPoint119);
            stringCache12.Append(stringPoint120);
            stringCache12.Append(stringPoint121);
            stringCache12.Append(stringPoint122);
            stringCache12.Append(stringPoint123);
            stringCache12.Append(stringPoint124);
            stringCache12.Append(stringPoint125);
            stringCache12.Append(stringPoint126);

            stringReference12.Append(formula17);
            stringReference12.Append(stringCache12);

            categoryAxisData6.Append(stringReference12);

            C.Values values6 = new C.Values();

            C.NumberReference numberReference6 = new C.NumberReference();
            C.Formula formula18 = new C.Formula();
            formula18.Text = "\'需求工作量分析\'!$C$4:$C$24";

            C.NumberingCache numberingCache6 = new C.NumberingCache();
            C.FormatCode formatCode6 = new C.FormatCode();
            formatCode6.Text = "General";
            C.PointCount pointCount18 = new C.PointCount() { Val = (UInt32Value)20U };

            C.NumericPoint numericPoint101 = new C.NumericPoint() { Index = (UInt32Value)0U };
            C.NumericValue numericValue227 = new C.NumericValue();
            numericValue227.Text = "0";

            numericPoint101.Append(numericValue227);

            C.NumericPoint numericPoint102 = new C.NumericPoint() { Index = (UInt32Value)1U };
            C.NumericValue numericValue228 = new C.NumericValue();
            numericValue228.Text = "3";

            numericPoint102.Append(numericValue228);

            C.NumericPoint numericPoint103 = new C.NumericPoint() { Index = (UInt32Value)2U };
            C.NumericValue numericValue229 = new C.NumericValue();
            numericValue229.Text = "0";

            numericPoint103.Append(numericValue229);

            C.NumericPoint numericPoint104 = new C.NumericPoint() { Index = (UInt32Value)3U };
            C.NumericValue numericValue230 = new C.NumericValue();
            numericValue230.Text = "0";

            numericPoint104.Append(numericValue230);

            C.NumericPoint numericPoint105 = new C.NumericPoint() { Index = (UInt32Value)4U };
            C.NumericValue numericValue231 = new C.NumericValue();
            numericValue231.Text = "6";

            numericPoint105.Append(numericValue231);

            C.NumericPoint numericPoint106 = new C.NumericPoint() { Index = (UInt32Value)5U };
            C.NumericValue numericValue232 = new C.NumericValue();
            numericValue232.Text = "0";

            numericPoint106.Append(numericValue232);

            C.NumericPoint numericPoint107 = new C.NumericPoint() { Index = (UInt32Value)6U };
            C.NumericValue numericValue233 = new C.NumericValue();
            numericValue233.Text = "0";

            numericPoint107.Append(numericValue233);

            C.NumericPoint numericPoint108 = new C.NumericPoint() { Index = (UInt32Value)7U };
            C.NumericValue numericValue234 = new C.NumericValue();
            numericValue234.Text = "0";

            numericPoint108.Append(numericValue234);

            C.NumericPoint numericPoint109 = new C.NumericPoint() { Index = (UInt32Value)8U };
            C.NumericValue numericValue235 = new C.NumericValue();
            numericValue235.Text = "0";

            numericPoint109.Append(numericValue235);

            C.NumericPoint numericPoint110 = new C.NumericPoint() { Index = (UInt32Value)9U };
            C.NumericValue numericValue236 = new C.NumericValue();
            numericValue236.Text = "0";

            numericPoint110.Append(numericValue236);

            C.NumericPoint numericPoint111 = new C.NumericPoint() { Index = (UInt32Value)10U };
            C.NumericValue numericValue237 = new C.NumericValue();
            numericValue237.Text = "0";

            numericPoint111.Append(numericValue237);

            C.NumericPoint numericPoint112 = new C.NumericPoint() { Index = (UInt32Value)11U };
            C.NumericValue numericValue238 = new C.NumericValue();
            numericValue238.Text = "0";

            numericPoint112.Append(numericValue238);

            C.NumericPoint numericPoint113 = new C.NumericPoint() { Index = (UInt32Value)12U };
            C.NumericValue numericValue239 = new C.NumericValue();
            numericValue239.Text = "0";

            numericPoint113.Append(numericValue239);

            C.NumericPoint numericPoint114 = new C.NumericPoint() { Index = (UInt32Value)13U };
            C.NumericValue numericValue240 = new C.NumericValue();
            numericValue240.Text = "0";

            numericPoint114.Append(numericValue240);

            C.NumericPoint numericPoint115 = new C.NumericPoint() { Index = (UInt32Value)14U };
            C.NumericValue numericValue241 = new C.NumericValue();
            numericValue241.Text = "0";

            numericPoint115.Append(numericValue241);

            C.NumericPoint numericPoint116 = new C.NumericPoint() { Index = (UInt32Value)15U };
            C.NumericValue numericValue242 = new C.NumericValue();
            numericValue242.Text = "0";

            numericPoint116.Append(numericValue242);

            C.NumericPoint numericPoint117 = new C.NumericPoint() { Index = (UInt32Value)16U };
            C.NumericValue numericValue243 = new C.NumericValue();
            numericValue243.Text = "0";

            numericPoint117.Append(numericValue243);

            C.NumericPoint numericPoint118 = new C.NumericPoint() { Index = (UInt32Value)17U };
            C.NumericValue numericValue244 = new C.NumericValue();
            numericValue244.Text = "0";

            numericPoint118.Append(numericValue244);

            C.NumericPoint numericPoint119 = new C.NumericPoint() { Index = (UInt32Value)18U };
            C.NumericValue numericValue245 = new C.NumericValue();
            numericValue245.Text = "0";

            numericPoint119.Append(numericValue245);

            C.NumericPoint numericPoint120 = new C.NumericPoint() { Index = (UInt32Value)19U };
            C.NumericValue numericValue246 = new C.NumericValue();
            numericValue246.Text = "0";

            numericPoint120.Append(numericValue246);

            numberingCache6.Append(formatCode6);
            numberingCache6.Append(pointCount18);
            numberingCache6.Append(numericPoint101);
            numberingCache6.Append(numericPoint102);
            numberingCache6.Append(numericPoint103);
            numberingCache6.Append(numericPoint104);
            numberingCache6.Append(numericPoint105);
            numberingCache6.Append(numericPoint106);
            numberingCache6.Append(numericPoint107);
            numberingCache6.Append(numericPoint108);
            numberingCache6.Append(numericPoint109);
            numberingCache6.Append(numericPoint110);
            numberingCache6.Append(numericPoint111);
            numberingCache6.Append(numericPoint112);
            numberingCache6.Append(numericPoint113);
            numberingCache6.Append(numericPoint114);
            numberingCache6.Append(numericPoint115);
            numberingCache6.Append(numericPoint116);
            numberingCache6.Append(numericPoint117);
            numberingCache6.Append(numericPoint118);
            numberingCache6.Append(numericPoint119);
            numberingCache6.Append(numericPoint120);

            numberReference6.Append(formula18);
            numberReference6.Append(numberingCache6);

            values6.Append(numberReference6);

            barChartSeries6.Append(index12);
            barChartSeries6.Append(order6);
            barChartSeries6.Append(seriesText6);
            barChartSeries6.Append(invertIfNegative6);
            barChartSeries6.Append(categoryAxisData6);
            barChartSeries6.Append(values6);

            C.DataLabels dataLabels3 = new C.DataLabels();
            C.ShowLegendKey showLegendKey3 = new C.ShowLegendKey() { Val = false };
            C.ShowValue showValue3 = new C.ShowValue() { Val = false };
            C.ShowCategoryName showCategoryName3 = new C.ShowCategoryName() { Val = false };
            C.ShowSeriesName showSeriesName3 = new C.ShowSeriesName() { Val = false };
            C.ShowPercent showPercent3 = new C.ShowPercent() { Val = false };
            C.ShowBubbleSize showBubbleSize3 = new C.ShowBubbleSize() { Val = false };

            dataLabels3.Append(showLegendKey3);
            dataLabels3.Append(showValue3);
            dataLabels3.Append(showCategoryName3);
            dataLabels3.Append(showSeriesName3);
            dataLabels3.Append(showPercent3);
            dataLabels3.Append(showBubbleSize3);
            C.GapWidth gapWidth3 = new C.GapWidth() { Val = (UInt16Value)75U };
            C.Overlap overlap3 = new C.Overlap() { Val = 100 };
            C.AxisId axisId9 = new C.AxisId() { Val = (UInt32Value)125868288U };
            C.AxisId axisId10 = new C.AxisId() { Val = (UInt32Value)125878272U };

            barChart3.Append(barDirection3);
            barChart3.Append(barGrouping3);
            barChart3.Append(varyColors3);
            barChart3.Append(barChartSeries5);
            barChart3.Append(barChartSeries6);
            barChart3.Append(dataLabels3);
            barChart3.Append(gapWidth3);
            barChart3.Append(overlap3);
            barChart3.Append(axisId9);
            barChart3.Append(axisId10);

            C.CategoryAxis categoryAxis3 = new C.CategoryAxis();
            C.AxisId axisId11 = new C.AxisId() { Val = (UInt32Value)125868288U };

            C.Scaling scaling5 = new C.Scaling();
            C.Orientation orientation5 = new C.Orientation() { Val = C.OrientationValues.MinMax };

            scaling5.Append(orientation5);
            C.Delete delete5 = new C.Delete() { Val = false };
            C.AxisPosition axisPosition5 = new C.AxisPosition() { Val = C.AxisPositionValues.Bottom };
            C.MajorTickMark majorTickMark5 = new C.MajorTickMark() { Val = C.TickMarkValues.None };
            C.MinorTickMark minorTickMark5 = new C.MinorTickMark() { Val = C.TickMarkValues.None };
            C.TickLabelPosition tickLabelPosition5 = new C.TickLabelPosition() { Val = C.TickLabelPositionValues.NextTo };
            C.CrossingAxis crossingAxis5 = new C.CrossingAxis() { Val = (UInt32Value)125878272U };
            C.Crosses crosses5 = new C.Crosses() { Val = C.CrossesValues.AutoZero };
            C.AutoLabeled autoLabeled3 = new C.AutoLabeled() { Val = true };
            C.LabelAlignment labelAlignment3 = new C.LabelAlignment() { Val = C.LabelAlignmentValues.Center };
            C.LabelOffset labelOffset3 = new C.LabelOffset() { Val = (UInt16Value)100U };
            C.NoMultiLevelLabels noMultiLevelLabels3 = new C.NoMultiLevelLabels() { Val = false };

            categoryAxis3.Append(axisId11);
            categoryAxis3.Append(scaling5);
            categoryAxis3.Append(delete5);
            categoryAxis3.Append(axisPosition5);
            categoryAxis3.Append(majorTickMark5);
            categoryAxis3.Append(minorTickMark5);
            categoryAxis3.Append(tickLabelPosition5);
            categoryAxis3.Append(crossingAxis5);
            categoryAxis3.Append(crosses5);
            categoryAxis3.Append(autoLabeled3);
            categoryAxis3.Append(labelAlignment3);
            categoryAxis3.Append(labelOffset3);
            categoryAxis3.Append(noMultiLevelLabels3);

            C.ValueAxis valueAxis3 = new C.ValueAxis();
            C.AxisId axisId12 = new C.AxisId() { Val = (UInt32Value)125878272U };

            C.Scaling scaling6 = new C.Scaling();
            C.Orientation orientation6 = new C.Orientation() { Val = C.OrientationValues.MinMax };

            scaling6.Append(orientation6);
            C.Delete delete6 = new C.Delete() { Val = false };
            C.AxisPosition axisPosition6 = new C.AxisPosition() { Val = C.AxisPositionValues.Left };
            C.MajorGridlines majorGridlines3 = new C.MajorGridlines();
            C.NumberingFormat numberingFormat3 = new C.NumberingFormat() { FormatCode = "General", SourceLinked = true };
            C.MajorTickMark majorTickMark6 = new C.MajorTickMark() { Val = C.TickMarkValues.None };
            C.MinorTickMark minorTickMark6 = new C.MinorTickMark() { Val = C.TickMarkValues.None };
            C.TickLabelPosition tickLabelPosition6 = new C.TickLabelPosition() { Val = C.TickLabelPositionValues.NextTo };

            C.ChartShapeProperties chartShapeProperties3 = new C.ChartShapeProperties();

            A.Outline outline3 = new A.Outline() { Width = 9525 };
            A.NoFill noFill3 = new A.NoFill();

            outline3.Append(noFill3);

            chartShapeProperties3.Append(outline3);
            C.CrossingAxis crossingAxis6 = new C.CrossingAxis() { Val = (UInt32Value)125868288U };
            C.Crosses crosses6 = new C.Crosses() { Val = C.CrossesValues.AutoZero };
            C.CrossBetween crossBetween3 = new C.CrossBetween() { Val = C.CrossBetweenValues.Between };

            valueAxis3.Append(axisId12);
            valueAxis3.Append(scaling6);
            valueAxis3.Append(delete6);
            valueAxis3.Append(axisPosition6);
            valueAxis3.Append(majorGridlines3);
            valueAxis3.Append(numberingFormat3);
            valueAxis3.Append(majorTickMark6);
            valueAxis3.Append(minorTickMark6);
            valueAxis3.Append(tickLabelPosition6);
            valueAxis3.Append(chartShapeProperties3);
            valueAxis3.Append(crossingAxis6);
            valueAxis3.Append(crosses6);
            valueAxis3.Append(crossBetween3);

            plotArea3.Append(layout8);
            plotArea3.Append(barChart3);
            plotArea3.Append(categoryAxis3);
            plotArea3.Append(valueAxis3);

            C.Legend legend3 = new C.Legend();
            C.LegendPosition legendPosition3 = new C.LegendPosition() { Val = C.LegendPositionValues.Bottom };
            C.Layout layout9 = new C.Layout();
            C.Overlay overlay6 = new C.Overlay() { Val = false };

            legend3.Append(legendPosition3);
            legend3.Append(layout9);
            legend3.Append(overlay6);
            C.PlotVisibleOnly plotVisibleOnly3 = new C.PlotVisibleOnly() { Val = true };
            C.DisplayBlanksAs displayBlanksAs3 = new C.DisplayBlanksAs() { Val = C.DisplayBlanksAsValues.Gap };
            C.ShowDataLabelsOverMaximum showDataLabelsOverMaximum3 = new C.ShowDataLabelsOverMaximum() { Val = false };

            chart3.Append(title3);
            chart3.Append(autoTitleDeleted3);
            chart3.Append(pivotFormats3);
            chart3.Append(plotArea3);
            chart3.Append(legend3);
            chart3.Append(plotVisibleOnly3);
            chart3.Append(displayBlanksAs3);
            chart3.Append(showDataLabelsOverMaximum3);

            C.PrintSettings printSettings3 = new C.PrintSettings();
            C.HeaderFooter headerFooter3 = new C.HeaderFooter();
            C.PageMargins pageMargins6 = new C.PageMargins() { Left = 0.7D, Right = 0.7D, Top = 0.75D, Bottom = 0.75D, Header = 0.3D, Footer = 0.3D };
            C.PageSetup pageSetup3 = new C.PageSetup();

            printSettings3.Append(headerFooter3);
            printSettings3.Append(pageMargins6);
            printSettings3.Append(pageSetup3);

            C.ChartSpaceExtensionList chartSpaceExtensionList3 = new C.ChartSpaceExtensionList();

            C.ChartSpaceExtension chartSpaceExtension3 = new C.ChartSpaceExtension() { Uri = "{781A3756-C4B2-4CAC-9D66-4F8BD8637D16}" };
            chartSpaceExtension3.AddNamespaceDeclaration("c14", "http://schemas.microsoft.com/office/drawing/2007/8/2/chart");

            C14.PivotOptions pivotOptions3 = new C14.PivotOptions();
            C14.DropZoneFilter dropZoneFilter3 = new C14.DropZoneFilter() { Val = true };
            C14.DropZoneCategories dropZoneCategories3 = new C14.DropZoneCategories() { Val = true };
            C14.DropZoneData dropZoneData3 = new C14.DropZoneData() { Val = true };
            C14.DropZoneSeries dropZoneSeries3 = new C14.DropZoneSeries() { Val = true };
            C14.DropZonesVisible dropZonesVisible3 = new C14.DropZonesVisible() { Val = true };

            pivotOptions3.Append(dropZoneFilter3);
            pivotOptions3.Append(dropZoneCategories3);
            pivotOptions3.Append(dropZoneData3);
            pivotOptions3.Append(dropZoneSeries3);
            pivotOptions3.Append(dropZonesVisible3);

            chartSpaceExtension3.Append(pivotOptions3);

            chartSpaceExtensionList3.Append(chartSpaceExtension3);

            chartSpace3.Append(date19043);
            chartSpace3.Append(editingLanguage3);
            chartSpace3.Append(roundedCorners3);
            chartSpace3.Append(alternateContent3);
            chartSpace3.Append(pivotSource3);
            chartSpace3.Append(chart3);
            chartSpace3.Append(printSettings3);
            chartSpace3.Append(chartSpaceExtensionList3);

            chartPart3.ChartSpace = chartSpace3;
        }

        // Generates content of pivotTablePart3.
        private void GeneratePivotTablePart3Content(PivotTablePart pivotTablePart3)
        {
            PivotTableDefinition pivotTableDefinition5 = new PivotTableDefinition() { Name = "PivotTable2", CacheId = (UInt32Value)7U, ApplyNumberFormats = false, ApplyBorderFormats = false, ApplyFontFormats = false, ApplyPatternFormats = false, ApplyAlignmentFormats = false, ApplyWidthHeightFormats = true, DataCaption = "Values", UpdatedVersion = 4, MinRefreshableVersion = 3, UseAutoFormatting = true, ItemPrintTitles = true, CreatedVersion = 4, Indent = (UInt32Value)0U, Outline = true, OutlineData = true, MultipleFieldFilters = false, ChartFormat = (UInt32Value)1U };
            Location location3 = new Location() { Reference = "A3:C24", FirstHeaderRow = (UInt32Value)0U, FirstDataRow = (UInt32Value)1U, FirstDataColumn = (UInt32Value)1U, RowPageCount = (UInt32Value)1U, ColumnsPerPage = (UInt32Value)1U };

            PivotFields pivotFields3 = new PivotFields() { Count = (UInt32Value)12U };

            PivotField pivotField25 = new PivotField() { Axis = PivotTableAxisValues.AxisRow, ShowAll = false, SortType = FieldSortValues.Descending };

            Items items5 = new Items() { Count = (UInt32Value)21U };
            Item item47 = new Item() { Index = (UInt32Value)1U };
            Item item48 = new Item() { Index = (UInt32Value)0U };
            Item item49 = new Item() { Index = (UInt32Value)11U };
            Item item50 = new Item() { Index = (UInt32Value)10U };
            Item item51 = new Item() { Index = (UInt32Value)9U };
            Item item52 = new Item() { Index = (UInt32Value)15U };
            Item item53 = new Item() { Index = (UInt32Value)2U };
            Item item54 = new Item() { Index = (UInt32Value)18U };
            Item item55 = new Item() { Index = (UInt32Value)19U };
            Item item56 = new Item() { Index = (UInt32Value)3U };
            Item item57 = new Item() { Index = (UInt32Value)17U };
            Item item58 = new Item() { Index = (UInt32Value)6U };
            Item item59 = new Item() { Index = (UInt32Value)7U };
            Item item60 = new Item() { Index = (UInt32Value)8U };
            Item item61 = new Item() { Index = (UInt32Value)5U };
            Item item62 = new Item() { Index = (UInt32Value)14U };
            Item item63 = new Item() { Index = (UInt32Value)4U };
            Item item64 = new Item() { Index = (UInt32Value)13U };
            Item item65 = new Item() { Index = (UInt32Value)12U };
            Item item66 = new Item() { Index = (UInt32Value)16U };
            Item item67 = new Item() { ItemType = ItemValues.Default };

            items5.Append(item47);
            items5.Append(item48);
            items5.Append(item49);
            items5.Append(item50);
            items5.Append(item51);
            items5.Append(item52);
            items5.Append(item53);
            items5.Append(item54);
            items5.Append(item55);
            items5.Append(item56);
            items5.Append(item57);
            items5.Append(item58);
            items5.Append(item59);
            items5.Append(item60);
            items5.Append(item61);
            items5.Append(item62);
            items5.Append(item63);
            items5.Append(item64);
            items5.Append(item65);
            items5.Append(item66);
            items5.Append(item67);

            AutoSortScope autoSortScope3 = new AutoSortScope();

            PivotArea pivotArea7 = new PivotArea() { DataOnly = false, Outline = false, FieldPosition = (UInt32Value)0U };

            PivotAreaReferences pivotAreaReferences7 = new PivotAreaReferences() { Count = (UInt32Value)1U };

            PivotAreaReference pivotAreaReference7 = new PivotAreaReference() { Field = (UInt32Value)4294967294U, Count = (UInt32Value)1U, Selected = false };
            FieldItem fieldItem55 = new FieldItem() { Val = (UInt32Value)0U };

            pivotAreaReference7.Append(fieldItem55);

            pivotAreaReferences7.Append(pivotAreaReference7);

            pivotArea7.Append(pivotAreaReferences7);

            autoSortScope3.Append(pivotArea7);

            pivotField25.Append(items5);
            pivotField25.Append(autoSortScope3);

            PivotField pivotField26 = new PivotField() { Axis = PivotTableAxisValues.AxisPage, ShowAll = false };

            Items items6 = new Items() { Count = (UInt32Value)2U };
            Item item68 = new Item() { Index = (UInt32Value)0U };
            Item item69 = new Item() { ItemType = ItemValues.Default };

            items6.Append(item68);
            items6.Append(item69);

            pivotField26.Append(items6);
            PivotField pivotField27 = new PivotField() { DataField = true, ShowAll = false };
            PivotField pivotField28 = new PivotField() { DataField = true, ShowAll = false };
            PivotField pivotField29 = new PivotField() { ShowAll = false };
            PivotField pivotField30 = new PivotField() { ShowAll = false };
            PivotField pivotField31 = new PivotField() { ShowAll = false };
            PivotField pivotField32 = new PivotField() { ShowAll = false };
            PivotField pivotField33 = new PivotField() { ShowAll = false };
            PivotField pivotField34 = new PivotField() { ShowAll = false };
            PivotField pivotField35 = new PivotField() { ShowAll = false };
            PivotField pivotField36 = new PivotField() { ShowAll = false };

            pivotFields3.Append(pivotField25);
            pivotFields3.Append(pivotField26);
            pivotFields3.Append(pivotField27);
            pivotFields3.Append(pivotField28);
            pivotFields3.Append(pivotField29);
            pivotFields3.Append(pivotField30);
            pivotFields3.Append(pivotField31);
            pivotFields3.Append(pivotField32);
            pivotFields3.Append(pivotField33);
            pivotFields3.Append(pivotField34);
            pivotFields3.Append(pivotField35);
            pivotFields3.Append(pivotField36);

            RowFields rowFields3 = new RowFields() { Count = (UInt32Value)1U };
            Field field5 = new Field() { Index = 0 };

            rowFields3.Append(field5);

            RowItems rowItems3 = new RowItems() { Count = (UInt32Value)21U };

            RowItem rowItem47 = new RowItem();
            MemberPropertyIndex memberPropertyIndex47 = new MemberPropertyIndex() { Val = 16 };

            rowItem47.Append(memberPropertyIndex47);

            RowItem rowItem48 = new RowItem();
            MemberPropertyIndex memberPropertyIndex48 = new MemberPropertyIndex() { Val = 18 };

            rowItem48.Append(memberPropertyIndex48);

            RowItem rowItem49 = new RowItem();
            MemberPropertyIndex memberPropertyIndex49 = new MemberPropertyIndex() { Val = 17 };

            rowItem49.Append(memberPropertyIndex49);

            RowItem rowItem50 = new RowItem();
            MemberPropertyIndex memberPropertyIndex50 = new MemberPropertyIndex() { Val = 4 };

            rowItem50.Append(memberPropertyIndex50);

            RowItem rowItem51 = new RowItem();
            MemberPropertyIndex memberPropertyIndex51 = new MemberPropertyIndex() { Val = 9 };

            rowItem51.Append(memberPropertyIndex51);

            RowItem rowItem52 = new RowItem();
            MemberPropertyIndex memberPropertyIndex52 = new MemberPropertyIndex() { Val = 5 };

            rowItem52.Append(memberPropertyIndex52);

            RowItem rowItem53 = new RowItem();
            MemberPropertyIndex memberPropertyIndex53 = new MemberPropertyIndex() { Val = 19 };

            rowItem53.Append(memberPropertyIndex53);

            RowItem rowItem54 = new RowItem();
            MemberPropertyIndex memberPropertyIndex54 = new MemberPropertyIndex() { Val = 14 };

            rowItem54.Append(memberPropertyIndex54);

            RowItem rowItem55 = new RowItem();
            MemberPropertyIndex memberPropertyIndex55 = new MemberPropertyIndex() { Val = 13 };

            rowItem55.Append(memberPropertyIndex55);

            RowItem rowItem56 = new RowItem();
            MemberPropertyIndex memberPropertyIndex56 = new MemberPropertyIndex() { Val = 1 };

            rowItem56.Append(memberPropertyIndex56);

            RowItem rowItem57 = new RowItem();
            MemberPropertyIndex memberPropertyIndex57 = new MemberPropertyIndex() { Val = 10 };

            rowItem57.Append(memberPropertyIndex57);

            RowItem rowItem58 = new RowItem();
            MemberPropertyIndex memberPropertyIndex58 = new MemberPropertyIndex() { Val = 3 };

            rowItem58.Append(memberPropertyIndex58);

            RowItem rowItem59 = new RowItem();
            MemberPropertyIndex memberPropertyIndex59 = new MemberPropertyIndex() { Val = 15 };

            rowItem59.Append(memberPropertyIndex59);

            RowItem rowItem60 = new RowItem();
            MemberPropertyIndex memberPropertyIndex60 = new MemberPropertyIndex() { Val = 2 };

            rowItem60.Append(memberPropertyIndex60);

            RowItem rowItem61 = new RowItem();
            MemberPropertyIndex memberPropertyIndex61 = new MemberPropertyIndex() { Val = 11 };

            rowItem61.Append(memberPropertyIndex61);

            RowItem rowItem62 = new RowItem();
            MemberPropertyIndex memberPropertyIndex62 = new MemberPropertyIndex() { Val = 8 };

            rowItem62.Append(memberPropertyIndex62);

            RowItem rowItem63 = new RowItem();
            MemberPropertyIndex memberPropertyIndex63 = new MemberPropertyIndex() { Val = 7 };

            rowItem63.Append(memberPropertyIndex63);

            RowItem rowItem64 = new RowItem();
            MemberPropertyIndex memberPropertyIndex64 = new MemberPropertyIndex() { Val = 6 };

            rowItem64.Append(memberPropertyIndex64);

            RowItem rowItem65 = new RowItem();
            MemberPropertyIndex memberPropertyIndex65 = new MemberPropertyIndex();

            rowItem65.Append(memberPropertyIndex65);

            RowItem rowItem66 = new RowItem();
            MemberPropertyIndex memberPropertyIndex66 = new MemberPropertyIndex() { Val = 12 };

            rowItem66.Append(memberPropertyIndex66);

            RowItem rowItem67 = new RowItem() { ItemType = ItemValues.Grand };
            MemberPropertyIndex memberPropertyIndex67 = new MemberPropertyIndex();

            rowItem67.Append(memberPropertyIndex67);

            rowItems3.Append(rowItem47);
            rowItems3.Append(rowItem48);
            rowItems3.Append(rowItem49);
            rowItems3.Append(rowItem50);
            rowItems3.Append(rowItem51);
            rowItems3.Append(rowItem52);
            rowItems3.Append(rowItem53);
            rowItems3.Append(rowItem54);
            rowItems3.Append(rowItem55);
            rowItems3.Append(rowItem56);
            rowItems3.Append(rowItem57);
            rowItems3.Append(rowItem58);
            rowItems3.Append(rowItem59);
            rowItems3.Append(rowItem60);
            rowItems3.Append(rowItem61);
            rowItems3.Append(rowItem62);
            rowItems3.Append(rowItem63);
            rowItems3.Append(rowItem64);
            rowItems3.Append(rowItem65);
            rowItems3.Append(rowItem66);
            rowItems3.Append(rowItem67);

            ColumnFields columnFields3 = new ColumnFields() { Count = (UInt32Value)1U };
            Field field6 = new Field() { Index = -2 };

            columnFields3.Append(field6);

            ColumnItems columnItems3 = new ColumnItems() { Count = (UInt32Value)2U };

            RowItem rowItem68 = new RowItem();
            MemberPropertyIndex memberPropertyIndex68 = new MemberPropertyIndex();

            rowItem68.Append(memberPropertyIndex68);

            RowItem rowItem69 = new RowItem() { Index = (UInt32Value)1U };
            MemberPropertyIndex memberPropertyIndex69 = new MemberPropertyIndex() { Val = 1 };

            rowItem69.Append(memberPropertyIndex69);

            columnItems3.Append(rowItem68);
            columnItems3.Append(rowItem69);

            PageFields pageFields3 = new PageFields() { Count = (UInt32Value)1U };
            PageField pageField3 = new PageField() { Field = 1, Hierarchy = -1 };

            pageFields3.Append(pageField3);

            DataFields dataFields3 = new DataFields() { Count = (UInt32Value)2U };
            DataField dataField5 = new DataField() { Name = "未完成的工作项目总数", Field = (UInt32Value)2U, BaseField = 0, BaseItem = (UInt32Value)1308056U };
            DataField dataField6 = new DataField() { Name = "已完成的工作项目总数", Field = (UInt32Value)3U, BaseField = 0, BaseItem = (UInt32Value)1308056U };

            dataFields3.Append(dataField5);
            dataFields3.Append(dataField6);

            ChartFormats chartFormats3 = new ChartFormats() { Count = (UInt32Value)2U };

            ChartFormat chartFormat5 = new ChartFormat() { Chart = (UInt32Value)0U, Format = (UInt32Value)0U, Series = true };

            PivotArea pivotArea8 = new PivotArea() { Type = PivotAreaValues.Data, Outline = false, FieldPosition = (UInt32Value)0U };

            PivotAreaReferences pivotAreaReferences8 = new PivotAreaReferences() { Count = (UInt32Value)1U };

            PivotAreaReference pivotAreaReference8 = new PivotAreaReference() { Field = (UInt32Value)4294967294U, Count = (UInt32Value)1U, Selected = false };
            FieldItem fieldItem56 = new FieldItem() { Val = (UInt32Value)0U };

            pivotAreaReference8.Append(fieldItem56);

            pivotAreaReferences8.Append(pivotAreaReference8);

            pivotArea8.Append(pivotAreaReferences8);

            chartFormat5.Append(pivotArea8);

            ChartFormat chartFormat6 = new ChartFormat() { Chart = (UInt32Value)0U, Format = (UInt32Value)1U, Series = true };

            PivotArea pivotArea9 = new PivotArea() { Type = PivotAreaValues.Data, Outline = false, FieldPosition = (UInt32Value)0U };

            PivotAreaReferences pivotAreaReferences9 = new PivotAreaReferences() { Count = (UInt32Value)1U };

            PivotAreaReference pivotAreaReference9 = new PivotAreaReference() { Field = (UInt32Value)4294967294U, Count = (UInt32Value)1U, Selected = false };
            FieldItem fieldItem57 = new FieldItem() { Val = (UInt32Value)1U };

            pivotAreaReference9.Append(fieldItem57);

            pivotAreaReferences9.Append(pivotAreaReference9);

            pivotArea9.Append(pivotAreaReferences9);

            chartFormat6.Append(pivotArea9);

            chartFormats3.Append(chartFormat5);
            chartFormats3.Append(chartFormat6);
            PivotTableStyle pivotTableStyle3 = new PivotTableStyle() { Name = "PivotStyleLight16", ShowRowHeaders = true, ShowColumnHeaders = true, ShowRowStripes = false, ShowColumnStripes = false, ShowLastColumn = true };

            PivotTableDefinitionExtensionList pivotTableDefinitionExtensionList3 = new PivotTableDefinitionExtensionList();

            PivotTableDefinitionExtension pivotTableDefinitionExtension3 = new PivotTableDefinitionExtension() { Uri = "{962EF5D1-5CA2-4c93-8EF4-DBF5C05439D2}" };
            pivotTableDefinitionExtension3.AddNamespaceDeclaration("x14", "http://schemas.microsoft.com/office/spreadsheetml/2009/9/main");

            X14.PivotTableDefinition pivotTableDefinition6 = new X14.PivotTableDefinition() { HideValuesRow = true };
            pivotTableDefinition6.AddNamespaceDeclaration("xm", "http://schemas.microsoft.com/office/excel/2006/main");

            pivotTableDefinitionExtension3.Append(pivotTableDefinition6);

            pivotTableDefinitionExtensionList3.Append(pivotTableDefinitionExtension3);

            pivotTableDefinition5.Append(location3);
            pivotTableDefinition5.Append(pivotFields3);
            pivotTableDefinition5.Append(rowFields3);
            pivotTableDefinition5.Append(rowItems3);
            pivotTableDefinition5.Append(columnFields3);
            pivotTableDefinition5.Append(columnItems3);
            pivotTableDefinition5.Append(pageFields3);
            pivotTableDefinition5.Append(dataFields3);
            pivotTableDefinition5.Append(chartFormats3);
            pivotTableDefinition5.Append(pivotTableStyle3);
            pivotTableDefinition5.Append(pivotTableDefinitionExtensionList3);

            pivotTablePart3.PivotTableDefinition = pivotTableDefinition5;
        }

        // Generates content of worksheetPart4.
        private void GenerateWorksheetPart4Content(WorksheetPart worksheetPart4)
        {
            Worksheet worksheet4 = new Worksheet() { MCAttributes = new MarkupCompatibilityAttributes() { Ignorable = "x14ac" } };
            worksheet4.AddNamespaceDeclaration("r", "http://schemas.openxmlformats.org/officeDocument/2006/relationships");
            worksheet4.AddNamespaceDeclaration("mc", "http://schemas.openxmlformats.org/markup-compatibility/2006");
            worksheet4.AddNamespaceDeclaration("x14ac", "http://schemas.microsoft.com/office/spreadsheetml/2009/9/ac");
            SheetDimension sheetDimension4 = new SheetDimension() { Reference = "A1:M25" };

            SheetViews sheetViews4 = new SheetViews();

            SheetView sheetView4 = new SheetView() { TabSelected = true, WorkbookViewId = (UInt32Value)0U };
            Selection selection4 = new Selection() { ActiveCell = "C54", SequenceOfReferences = new ListValue<StringValue>() { InnerText = "C54" } };

            sheetView4.Append(selection4);

            sheetViews4.Append(sheetView4);
            SheetFormatProperties sheetFormatProperties4 = new SheetFormatProperties() { DefaultRowHeight = 15D, DyDescent = 0.25D };

            Columns columns4 = new Columns();
            Column column8 = new Column() { Min = (UInt32Value)1U, Max = (UInt32Value)1U, Width = 27.28515625D, BestFit = true, CustomWidth = true };
            Column column9 = new Column() { Min = (UInt32Value)2U, Max = (UInt32Value)2U, Width = 9.5703125D, BestFit = true, CustomWidth = true };
            Column column10 = new Column() { Min = (UInt32Value)3U, Max = (UInt32Value)4U, Width = 22.85546875D, BestFit = true, CustomWidth = true };
            Column column11 = new Column() { Min = (UInt32Value)5U, Max = (UInt32Value)6U, Width = 18.42578125D, BestFit = true, CustomWidth = true };
            Column column12 = new Column() { Min = (UInt32Value)7U, Max = (UInt32Value)8U, Width = 14D, BestFit = true, CustomWidth = true };
            Column column13 = new Column() { Min = (UInt32Value)9U, Max = (UInt32Value)9U, Width = 5.28515625D, BestFit = true, CustomWidth = true };
            Column column14 = new Column() { Min = (UInt32Value)10U, Max = (UInt32Value)10U, Width = 6D, BestFit = true, CustomWidth = true };
            Column column15 = new Column() { Min = (UInt32Value)11U, Max = (UInt32Value)12U, Width = 18.42578125D, BestFit = true, CustomWidth = true };
            Column column16 = new Column() { Min = (UInt32Value)13U, Max = (UInt32Value)13U, Width = 74.5703125D, BestFit = true, CustomWidth = true };

            columns4.Append(column8);
            columns4.Append(column9);
            columns4.Append(column10);
            columns4.Append(column11);
            columns4.Append(column12);
            columns4.Append(column13);
            columns4.Append(column14);
            columns4.Append(column15);
            columns4.Append(column16);

            SheetData sheetData4 = new SheetData();

            Row row70 = new Row() { RowIndex = (UInt32Value)1U, Spans = new ListValue<StringValue>() { InnerText = "1:13" }, DyDescent = 0.25D };

            Cell cell205 = new Cell() { CellReference = "A1", DataType = CellValues.SharedString };
            CellValue cellValue205 = new CellValue();
            cellValue205.Text = "0";

            cell205.Append(cellValue205);

            Cell cell206 = new Cell() { CellReference = "B1", DataType = CellValues.SharedString };
            CellValue cellValue206 = new CellValue();
            cellValue206.Text = "1";

            cell206.Append(cellValue206);

            Cell cell207 = new Cell() { CellReference = "C1", DataType = CellValues.SharedString };
            CellValue cellValue207 = new CellValue();
            cellValue207.Text = "2";

            cell207.Append(cellValue207);

            Cell cell208 = new Cell() { CellReference = "D1", DataType = CellValues.SharedString };
            CellValue cellValue208 = new CellValue();
            cellValue208.Text = "3";

            cell208.Append(cellValue208);

            Cell cell209 = new Cell() { CellReference = "E1", DataType = CellValues.SharedString };
            CellValue cellValue209 = new CellValue();
            cellValue209.Text = "4";

            cell209.Append(cellValue209);

            Cell cell210 = new Cell() { CellReference = "F1", DataType = CellValues.SharedString };
            CellValue cellValue210 = new CellValue();
            cellValue210.Text = "5";

            cell210.Append(cellValue210);

            Cell cell211 = new Cell() { CellReference = "G1", DataType = CellValues.SharedString };
            CellValue cellValue211 = new CellValue();
            cellValue211.Text = "6";

            cell211.Append(cellValue211);

            Cell cell212 = new Cell() { CellReference = "H1", DataType = CellValues.SharedString };
            CellValue cellValue212 = new CellValue();
            cellValue212.Text = "7";

            cell212.Append(cellValue212);

            Cell cell213 = new Cell() { CellReference = "I1", DataType = CellValues.SharedString };
            CellValue cellValue213 = new CellValue();
            cellValue213.Text = "8";

            cell213.Append(cellValue213);

            Cell cell214 = new Cell() { CellReference = "J1", DataType = CellValues.SharedString };
            CellValue cellValue214 = new CellValue();
            cellValue214.Text = "9";

            cell214.Append(cellValue214);

            Cell cell215 = new Cell() { CellReference = "K1", DataType = CellValues.SharedString };
            CellValue cellValue215 = new CellValue();
            cellValue215.Text = "10";

            cell215.Append(cellValue215);

            Cell cell216 = new Cell() { CellReference = "L1", DataType = CellValues.SharedString };
            CellValue cellValue216 = new CellValue();
            cellValue216.Text = "11";

            cell216.Append(cellValue216);

            Cell cell217 = new Cell() { CellReference = "M1", DataType = CellValues.SharedString };
            CellValue cellValue217 = new CellValue();
            cellValue217.Text = "12";

            cell217.Append(cellValue217);

            row70.Append(cell205);
            row70.Append(cell206);
            row70.Append(cell207);
            row70.Append(cell208);
            row70.Append(cell209);
            row70.Append(cell210);
            row70.Append(cell211);
            row70.Append(cell212);
            row70.Append(cell213);
            row70.Append(cell214);
            row70.Append(cell215);
            row70.Append(cell216);
            row70.Append(cell217);

            sheetData4.Append(row70);
            PageMargins pageMargins7 = new PageMargins() { Left = 0.7D, Right = 0.7D, Top = 0.75D, Bottom = 0.75D, Header = 0.3D, Footer = 0.3D };

            worksheet4.Append(sheetDimension4);
            worksheet4.Append(sheetViews4);
            worksheet4.Append(sheetFormatProperties4);
            worksheet4.Append(columns4);
            worksheet4.Append(sheetData4);
            worksheet4.Append(pageMargins7);

            worksheetPart4.Worksheet = worksheet4;
        }

        // Generates content of sharedStringTablePart1.
        private void GenerateSharedStringTablePart1Content(SharedStringTablePart sharedStringTablePart1)
        {
            SharedStringTable sharedStringTable1 = new SharedStringTable() { Count = (UInt32Value)139U, UniqueCount = (UInt32Value)66U };

            SharedStringItem sharedStringItem1 = new SharedStringItem();
            Text text4 = new Text();
            text4.Text = "Requirement";

            sharedStringItem1.Append(text4);

            SharedStringItem sharedStringItem2 = new SharedStringItem();
            Text text5 = new Text();
            text5.Text = "需求级别";

            sharedStringItem2.Append(text5);

            SharedStringItem sharedStringItem3 = new SharedStringItem();
            Text text6 = new Text();
            text6.Text = "未完成的工作项目数目";

            sharedStringItem3.Append(text6);

            SharedStringItem sharedStringItem4 = new SharedStringItem();
            Text text7 = new Text();
            text7.Text = "已完成的工作项目数目";

            sharedStringItem4.Append(text7);

            SharedStringItem sharedStringItem5 = new SharedStringItem();
            Text text8 = new Text();
            text8.Text = "未完成的任务数目";

            sharedStringItem5.Append(text8);

            SharedStringItem sharedStringItem6 = new SharedStringItem();
            Text text9 = new Text();
            text9.Text = "已完成的任务数目";

            sharedStringItem6.Append(text9);

            SharedStringItem sharedStringItem7 = new SharedStringItem();
            Text text10 = new Text();
            text10.Text = "已完成的工时";

            sharedStringItem7.Append(text10);

            SharedStringItem sharedStringItem8 = new SharedStringItem();
            Text text11 = new Text();
            text11.Text = "未完成的工时";

            sharedStringItem8.Append(text11);

            SharedStringItem sharedStringItem9 = new SharedStringItem();
            Text text12 = new Text();
            text12.Text = "挣值";

            sharedStringItem9.Append(text12);

            SharedStringItem sharedStringItem10 = new SharedStringItem();
            Text text13 = new Text();
            text13.Text = "总值";

            sharedStringItem10.Append(text13);

            SharedStringItem sharedStringItem11 = new SharedStringItem();
            Text text14 = new Text();
            text14.Text = "未完成的需求数目";

            sharedStringItem11.Append(text14);

            SharedStringItem sharedStringItem12 = new SharedStringItem();
            Text text15 = new Text();
            text15.Text = "已完成的需求数目";

            sharedStringItem12.Append(text15);

            SharedStringItem sharedStringItem13 = new SharedStringItem();
            Text text16 = new Text();
            text16.Text = "需求详细信息链接";

            sharedStringItem13.Append(text16);

            SharedStringItem sharedStringItem14 = new SharedStringItem();
            Text text17 = new Text();
            text17.Text = "任务标签";

            sharedStringItem14.Append(text17);

            SharedStringItem sharedStringItem15 = new SharedStringItem();
            Text text18 = new Text();
            text18.Text = "http://HW-YISH-2/Requirement/Details/57bdf264-5d04-4957-a900-0d530791ca54";

            sharedStringItem15.Append(text18);

            SharedStringItem sharedStringItem16 = new SharedStringItem();
            Text text19 = new Text();
            text19.Text = "事故管理";

            sharedStringItem16.Append(text19);

            SharedStringItem sharedStringItem17 = new SharedStringItem();
            Text text20 = new Text();
            text20.Text = "http://HW-YISH-2/Requirement/Details/70caa128-6210-48be-a82c-0dfb8d9b0e82";

            sharedStringItem17.Append(text20);

            SharedStringItem sharedStringItem18 = new SharedStringItem();
            Text text21 = new Text();
            text21.Text = "团队时间管理";

            sharedStringItem18.Append(text21);

            SharedStringItem sharedStringItem19 = new SharedStringItem();
            Text text22 = new Text();
            text22.Text = "http://HW-YISH-2/Requirement/Details/845101c8-6364-48d9-8b9e-12c63746fd3e";

            sharedStringItem19.Append(text22);

            SharedStringItem sharedStringItem20 = new SharedStringItem();
            Text text23 = new Text();
            text23.Text = "整合配置管理系统";

            sharedStringItem20.Append(text23);

            SharedStringItem sharedStringItem21 = new SharedStringItem();
            Text text24 = new Text();
            text24.Text = "http://HW-YISH-2/Requirement/Details/222b0b83-5d93-473e-a3ec-2164f1dcdeb6";

            sharedStringItem21.Append(text24);

            SharedStringItem sharedStringItem22 = new SharedStringItem();
            Text text25 = new Text();
            text25.Text = "集成团队管理";

            sharedStringItem22.Append(text25);

            SharedStringItem sharedStringItem23 = new SharedStringItem();
            Text text26 = new Text();
            text26.Text = "http://HW-YISH-2/Requirement/Details/4316af2e-ab98-49f5-9038-3b2fc6bea502";

            sharedStringItem23.Append(text26);

            SharedStringItem sharedStringItem24 = new SharedStringItem();
            Text text27 = new Text();
            text27.Text = "用户选择托管配置管理系统";

            sharedStringItem24.Append(text27);

            SharedStringItem sharedStringItem25 = new SharedStringItem();
            Text text28 = new Text();
            text28.Text = "http://HW-YISH-2/Requirement/Details/5c4b240e-29e9-4bbd-a3a6-5c4c5989daf6";

            sharedStringItem25.Append(text28);

            SharedStringItem sharedStringItem26 = new SharedStringItem();
            Text text29 = new Text();
            text29.Text = "流程管理";

            sharedStringItem26.Append(text29);

            SharedStringItem sharedStringItem27 = new SharedStringItem();
            Text text30 = new Text();
            text30.Text = "http://HW-YISH-2/Requirement/Details/9d8d00d5-6917-437d-9aec-5f1e13f5c32a";

            sharedStringItem27.Append(text30);

            SharedStringItem sharedStringItem28 = new SharedStringItem();
            Text text31 = new Text();
            text31.Text = "添加CAPTCHA支持";

            sharedStringItem28.Append(text31);

            SharedStringItem sharedStringItem29 = new SharedStringItem();
            Text text32 = new Text();
            text32.Text = "http://HW-YISH-2/Requirement/Details/d7b98a2f-3c97-4398-9ca6-77335aa2c14f";

            sharedStringItem29.Append(text32);

            SharedStringItem sharedStringItem30 = new SharedStringItem();
            Text text33 = new Text();
            text33.Text = "用户身份验证";

            sharedStringItem30.Append(text33);

            SharedStringItem sharedStringItem31 = new SharedStringItem();
            Text text34 = new Text();
            text34.Text = "http://HW-YISH-2/Requirement/Details/c7ae4f10-e054-4ec2-9dbd-7b9515be8ec1";

            sharedStringItem31.Append(text34);

            SharedStringItem sharedStringItem32 = new SharedStringItem();
            Text text35 = new Text();
            text35.Text = "全文检索";

            sharedStringItem32.Append(text35);

            SharedStringItem sharedStringItem33 = new SharedStringItem();
            Text text36 = new Text();
            text36.Text = "http://HW-YISH-2/Requirement/Details/591b4b60-e15e-4c4b-a555-7c3174244914";

            sharedStringItem33.Append(text36);

            SharedStringItem sharedStringItem34 = new SharedStringItem();
            Text text37 = new Text();
            text37.Text = "元素之间的相互转化";

            sharedStringItem34.Append(text37);

            SharedStringItem sharedStringItem35 = new SharedStringItem();
            Text text38 = new Text();
            text38.Text = "http://HW-YISH-2/Requirement/Details/c771ee37-d08f-4236-8e60-7ed9ad7347a3";

            sharedStringItem35.Append(text38);

            SharedStringItem sharedStringItem36 = new SharedStringItem();
            Text text39 = new Text();
            text39.Text = "http://HW-YISH-2/Requirement/Details/09e34e4c-9c03-46bd-80f1-83d4e0275066";

            sharedStringItem36.Append(text39);

            SharedStringItem sharedStringItem37 = new SharedStringItem();
            Text text40 = new Text();
            text40.Text = "http://HW-YISH-2/Requirement/Details/54c06089-5560-463c-8daa-89a3b28cec41";

            sharedStringItem37.Append(text40);

            SharedStringItem sharedStringItem38 = new SharedStringItem();
            Text text41 = new Text();
            text41.Text = "http://HW-YISH-2/Requirement/Details/d84321df-ea7e-46e9-b4cb-8b4477fd8b73";

            sharedStringItem38.Append(text41);

            SharedStringItem sharedStringItem39 = new SharedStringItem();
            Text text42 = new Text();
            text42.Text = "会议管理";

            sharedStringItem39.Append(text42);

            SharedStringItem sharedStringItem40 = new SharedStringItem();
            Text text43 = new Text();
            text43.Text = "http://HW-YISH-2/Requirement/Details/333da846-76da-42d3-b89a-98080b24336a";

            sharedStringItem40.Append(text43);

            SharedStringItem sharedStringItem41 = new SharedStringItem();
            Text text44 = new Text();
            text44.Text = "需求管理基本功能";

            sharedStringItem41.Append(text44);

            SharedStringItem sharedStringItem42 = new SharedStringItem();
            Text text45 = new Text();
            text45.Text = "http://HW-YISH-2/Requirement/Details/84db06b6-117d-4311-8699-993cce6ca191";

            sharedStringItem42.Append(text45);

            SharedStringItem sharedStringItem43 = new SharedStringItem();
            Text text46 = new Text();
            text46.Text = "需求管理中的进度跟踪";

            sharedStringItem43.Append(text46);

            SharedStringItem sharedStringItem44 = new SharedStringItem();
            Text text47 = new Text();
            text47.Text = "http://HW-YISH-2/Requirement/Details/1a4f5103-760a-46c2-ad23-a9d7e9bc2f5c";

            sharedStringItem44.Append(text47);

            SharedStringItem sharedStringItem45 = new SharedStringItem();
            Text text48 = new Text();
            text48.Text = "统计功能";

            sharedStringItem45.Append(text48);

            SharedStringItem sharedStringItem46 = new SharedStringItem();
            Text text49 = new Text();
            text49.Text = "http://HW-YISH-2/Requirement/Details/098c2fee-c1e7-4f83-8176-b402f2e4cb66";

            sharedStringItem46.Append(text49);

            SharedStringItem sharedStringItem47 = new SharedStringItem();
            Text text50 = new Text();
            text50.Text = "http://HW-YISH-2/Requirement/Details/96c4dc25-8d6c-48c1-b999-bb1fb398cca5";

            sharedStringItem47.Append(text50);

            SharedStringItem sharedStringItem48 = new SharedStringItem();
            Text text51 = new Text();
            text51.Text = "决策过程跟踪";

            sharedStringItem48.Append(text51);

            SharedStringItem sharedStringItem49 = new SharedStringItem();
            Text text52 = new Text();
            text52.Text = "http://HW-YISH-2/Requirement/Details/eb0ccdd4-bdd5-4f28-8f5f-bdd9eaf2fc86";

            sharedStringItem49.Append(text52);

            SharedStringItem sharedStringItem50 = new SharedStringItem();
            Text text53 = new Text();
            text53.Text = "风险管理";

            sharedStringItem50.Append(text53);

            SharedStringItem sharedStringItem51 = new SharedStringItem();
            Text text54 = new Text();
            text54.Text = "http://HW-YISH-2/Requirement/Details/e949cfa3-94b3-4faf-86a7-c5ae7ad877cc";

            sharedStringItem51.Append(text54);

            SharedStringItem sharedStringItem52 = new SharedStringItem();
            Text text55 = new Text();
            text55.Text = "权限管理";

            sharedStringItem52.Append(text55);

            SharedStringItem sharedStringItem53 = new SharedStringItem();
            Text text56 = new Text();
            text56.Text = "http://HW-YISH-2/Requirement/Details/c97c3819-8131-41b3-805f-c6bf4310458f";

            sharedStringItem53.Append(text56);

            SharedStringItem sharedStringItem54 = new SharedStringItem();
            Text text57 = new Text();
            text57.Text = "实现通用的分页显示功能";

            sharedStringItem54.Append(text57);

            SharedStringItem sharedStringItem55 = new SharedStringItem();
            Text text58 = new Text();
            text58.Text = "http://HW-YISH-2/Requirement/Details/fd946acf-1170-4aa1-aa8f-c972c18c4bac";

            sharedStringItem55.Append(text58);

            SharedStringItem sharedStringItem56 = new SharedStringItem();
            Text text59 = new Text();
            text59.Text = "支持国际化";

            sharedStringItem56.Append(text59);

            SharedStringItem sharedStringItem57 = new SharedStringItem();
            Text text60 = new Text();
            text60.Text = "http://HW-YISH-2/Requirement/Details/b0e95544-80a5-4d2a-84ab-ec4f5f4f8aa9";

            sharedStringItem57.Append(text60);

            SharedStringItem sharedStringItem58 = new SharedStringItem();
            Text text61 = new Text();
            text61.Text = "Row Labels";

            sharedStringItem58.Append(text61);

            SharedStringItem sharedStringItem59 = new SharedStringItem();
            Text text62 = new Text();
            text62.Text = "Grand Total";

            sharedStringItem59.Append(text62);

            SharedStringItem sharedStringItem60 = new SharedStringItem();
            Text text63 = new Text();
            text63.Text = "未完成的工作项目总数";

            sharedStringItem60.Append(text63);

            SharedStringItem sharedStringItem61 = new SharedStringItem();
            Text text64 = new Text();
            text64.Text = "已完成的工作项目总数";

            sharedStringItem61.Append(text64);

            SharedStringItem sharedStringItem62 = new SharedStringItem();
            Text text65 = new Text();
            text65.Text = "(All)";

            sharedStringItem62.Append(text65);

            SharedStringItem sharedStringItem63 = new SharedStringItem();
            Text text66 = new Text();
            text66.Text = "挣值总计";

            sharedStringItem63.Append(text66);

            SharedStringItem sharedStringItem64 = new SharedStringItem();
            Text text67 = new Text();
            text67.Text = "需求商业价值总计";

            sharedStringItem64.Append(text67);

            SharedStringItem sharedStringItem65 = new SharedStringItem();
            Text text68 = new Text();
            text68.Text = "已完成的工时总计";

            sharedStringItem65.Append(text68);

            SharedStringItem sharedStringItem66 = new SharedStringItem();
            Text text69 = new Text();
            text69.Text = "未完成的工时总计";

            sharedStringItem66.Append(text69);

            sharedStringTable1.Append(sharedStringItem1);
            sharedStringTable1.Append(sharedStringItem2);
            sharedStringTable1.Append(sharedStringItem3);
            sharedStringTable1.Append(sharedStringItem4);
            sharedStringTable1.Append(sharedStringItem5);
            sharedStringTable1.Append(sharedStringItem6);
            sharedStringTable1.Append(sharedStringItem7);
            sharedStringTable1.Append(sharedStringItem8);
            sharedStringTable1.Append(sharedStringItem9);
            sharedStringTable1.Append(sharedStringItem10);
            sharedStringTable1.Append(sharedStringItem11);
            sharedStringTable1.Append(sharedStringItem12);
            sharedStringTable1.Append(sharedStringItem13);
            sharedStringTable1.Append(sharedStringItem14);
            sharedStringTable1.Append(sharedStringItem15);
            sharedStringTable1.Append(sharedStringItem16);
            sharedStringTable1.Append(sharedStringItem17);
            sharedStringTable1.Append(sharedStringItem18);
            sharedStringTable1.Append(sharedStringItem19);
            sharedStringTable1.Append(sharedStringItem20);
            sharedStringTable1.Append(sharedStringItem21);
            sharedStringTable1.Append(sharedStringItem22);
            sharedStringTable1.Append(sharedStringItem23);
            sharedStringTable1.Append(sharedStringItem24);
            sharedStringTable1.Append(sharedStringItem25);
            sharedStringTable1.Append(sharedStringItem26);
            sharedStringTable1.Append(sharedStringItem27);
            sharedStringTable1.Append(sharedStringItem28);
            sharedStringTable1.Append(sharedStringItem29);
            sharedStringTable1.Append(sharedStringItem30);
            sharedStringTable1.Append(sharedStringItem31);
            sharedStringTable1.Append(sharedStringItem32);
            sharedStringTable1.Append(sharedStringItem33);
            sharedStringTable1.Append(sharedStringItem34);
            sharedStringTable1.Append(sharedStringItem35);
            sharedStringTable1.Append(sharedStringItem36);
            sharedStringTable1.Append(sharedStringItem37);
            sharedStringTable1.Append(sharedStringItem38);
            sharedStringTable1.Append(sharedStringItem39);
            sharedStringTable1.Append(sharedStringItem40);
            sharedStringTable1.Append(sharedStringItem41);
            sharedStringTable1.Append(sharedStringItem42);
            sharedStringTable1.Append(sharedStringItem43);
            sharedStringTable1.Append(sharedStringItem44);
            sharedStringTable1.Append(sharedStringItem45);
            sharedStringTable1.Append(sharedStringItem46);
            sharedStringTable1.Append(sharedStringItem47);
            sharedStringTable1.Append(sharedStringItem48);
            sharedStringTable1.Append(sharedStringItem49);
            sharedStringTable1.Append(sharedStringItem50);
            sharedStringTable1.Append(sharedStringItem51);
            sharedStringTable1.Append(sharedStringItem52);
            sharedStringTable1.Append(sharedStringItem53);
            sharedStringTable1.Append(sharedStringItem54);
            sharedStringTable1.Append(sharedStringItem55);
            sharedStringTable1.Append(sharedStringItem56);
            sharedStringTable1.Append(sharedStringItem57);
            sharedStringTable1.Append(sharedStringItem58);
            sharedStringTable1.Append(sharedStringItem59);
            sharedStringTable1.Append(sharedStringItem60);
            sharedStringTable1.Append(sharedStringItem61);
            sharedStringTable1.Append(sharedStringItem62);
            sharedStringTable1.Append(sharedStringItem63);
            sharedStringTable1.Append(sharedStringItem64);
            sharedStringTable1.Append(sharedStringItem65);
            sharedStringTable1.Append(sharedStringItem66);

            sharedStringTablePart1.SharedStringTable = sharedStringTable1;
        }
    }
}
